package com.studio.nx.framework.config.json;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;

import java.io.IOException;
import java.math.BigDecimal;

/**
 * 功能描述：格式化BigDecimal数据类型【保留两位小数】
 * 使用：
 *
 * @JsonSerialize(using = BigDecimalSerialize.class)
 * private BigDecimal paidPrice;
 * @Author 梁展鹏
 * @Date 2021/6/28 23:18
 */
public class BigDecimalSerialize extends JsonSerializer<BigDecimal> {

    @Override
    public void serialize(BigDecimal value, JsonGenerator gen, SerializerProvider serializerProvider) throws IOException {
        if (value != null && !"".equals(value)) {
            gen.writeString(value.setScale(2, BigDecimal.ROUND_HALF_DOWN) + "");
        } else {
            gen.writeString(value + "");
        }
    }

}
