package com.studio.nx.framework.jwt.config;

import com.studio.nx.framework.jwt.interceptor.JWTAuthInterceptor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.util.ArrayList;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/8 11:25
 */
@Configuration
public class AuthInterceptorConfig implements WebMvcConfigurer {

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(jwtAuthInterceptor())
                .addPathPatterns("/**")
                .excludePathPatterns(new ArrayList<String>(){{
                    add("/swagger-ui/**");
                    add("/swagger-resources/**");
                    add("/v2/api-docs");
                }});
    }

    @Bean
    public JWTAuthInterceptor jwtAuthInterceptor() {
        return new JWTAuthInterceptor();
    }

    /**
     * 功能描述：开启跨域
     * 结合Security之后，还需要在SecurityConfig中设置`and().cors().and().csrf().disable()`
     *
     * @param registry
     */
    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry.addMapping("/**") // 设置允许跨域的路径
                .allowCredentials(true) // 是否允许证书，不再默认开启
                .allowedOriginPatterns("*") // 设置允许跨域请求的域名
                .allowedHeaders("*")
                .allowedMethods("*")
                .maxAge(60 * 60); // 跨域允许时间
    }

}
