package com.studio.nx.framework.jwt.interceptor;

import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.exception.JWTException;
import com.studio.nx.framework.jwt.util.JWTUtil;
import org.springframework.core.Ordered;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

/**
 * 功能描述：JWTAuth拦截
 *
 * @Author 梁展鹏
 * @Date 2021/6/8 09:25
 */
@Component
public class JWTAuthInterceptor implements HandlerInterceptor, Ordered {

    @Override
    public int getOrder() {
        return -1;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object object) throws Exception {
        //支持跨域请求
//        response.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
//        response.setHeader("Access-Control-Max-Age", "3600");
//        response.setHeader("Access-Control-Allow-Credentials", "true");
//        response.setHeader("Access-Control-Allow-Headers", "x-requested-with,X-Nideshop-Token,X-URL-PATH");
//        response.setHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));

        String token = JWTUtil.getToken(); // 从 http 请求头中取出 token

        // 如果不是映射到方法直接通过
        if (!(object instanceof HandlerMethod)) {
            return true;
        }

        HandlerMethod handlerMethod = (HandlerMethod) object;
        Method method = handlerMethod.getMethod();
        /* 检查是否有IgnoreAuth注释，有则跳过认证 */
        if (method.isAnnotationPresent(IgnoreAuth.class)) {
            IgnoreAuth passToken = method.getAnnotation(IgnoreAuth.class);
            if (passToken.required()) {
                return true;
            }
        }

        // 检查有没有需要用户权限的注解
//        if (method.isAnnotationPresent(Auth.class)) {
//            Auth _auth = method.getAnnotation(Auth.class);
//            if (_auth != null) {
//            }
//        }

        // 默认所有方法走认证
        if (!StringUtils.hasText(token)) {
            throw new JWTException("无token，请重新登录");
        }

        try {
            JWTUtil.verify(token);
        } catch (Exception e) {
            throw new JWTException("401");
        }

        return true;
    }

}
