package com.studio.nx.framework.controller;

import com.studio.nx.framework.controller.request.FrontErrorDto;
import com.studio.nx.framework.controller.request.OperationLogRequestDto;
import com.studio.nx.framework.dto.OperationLogDto;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.service.OperationLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = "日志")
@Slf4j
@RestController("FrameworkLogController")
@RequestMapping("/framework/log")
public class OperationLogController {

    @Autowired
    private OperationLogService operationLogService;

    @ApiOperation(value = "查询[日志]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<OperationLogDto>>> page(@Valid @RequestBody OperationLogRequestDto dto) {
        return ResponseEntity.ok(BaseResponse.ok(operationLogService.page(dto)));
    }

    @ApiOperation(value = "删除所有日志")
    @DeleteMapping("/del/all/{type}")
    public ResponseEntity<BaseResponse> delAll(@PathVariable("type") String type) {
        operationLogService.delAll(type);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "上报客户端错误日志")
    @PostMapping("/report")
    public ResponseEntity<BaseResponse> report(@Valid @RequestBody FrontErrorDto dto, HttpServletRequest request) {
        operationLogService.insertFrontErrorLog(request, dto);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
