package com.studio.nx.framework.util;

import org.joda.time.DateTime;
import org.joda.time.Interval;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 功能描述：时间工具类
 * 需要在pom.xml引入joda-time
 */
public class DateUtils {

    public static final String yyyyMMddHHmmss = "yyyyMMddHHmmss";
    public static final String yyyyMMdd_HHmmss = "yyyy-MM-dd HH:mm:ss";

    /**
     * 功能描述：
     *
     * @param date
     * @return
     */
    public static String format(Date date, String formatStr) {
        if (date == null) {
            return "";
        }

        DateTime dateTime = new DateTime(date);
        return dateTime.toString(formatStr);
    }

    /**
     * 功能描述：
     *
     * @param date
     * @return
     */
    public static Date format(String date) {
        DateTimeFormatter fmt = DateTimeFormat.forPattern(yyyyMMdd_HHmmss);
        return fmt.parseDateTime(date).toDate();
    }

    /**
     * 功能描述：
     *
     * @param date
     * @param pattern
     * @return
     */
    public static Date format(String date, String pattern) {
        DateTimeFormatter fmt = DateTimeFormat.forPattern(pattern);
        return fmt.parseDateTime(date).toDate();
    }

    /**
     * 功能描述：获取当前年份
     *
     * @return
     */
    public static String getYear() {
        return new DateTime().toString("yyyy");
    }

    /**
     * 功能描述：获取当前月份
     *
     * @return
     */
    public static String getMonth() {
        return new DateTime().toString("MM");
    }

    /**
     * 功能描述：判断时间是否在区间内
     *
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @param compareTime 比较时间
     * @return true在/false不在
     */
    public static Boolean between(Date startTime, Date endTime, Date compareTime) {
        Interval interval = new Interval(startTime.getTime(), endTime.getTime());
        return interval.contains(compareTime.getTime());
    }

    /**
     * 功能描述：[date]时间增加x分钟
     *
     * @param date
     * @param num
     * @return
     */
    public static Date plusMinutes(Date date, Integer num) {
        DateTime dateTime = new DateTime(date);
        return dateTime.plusMinutes(num).toDate();
    }

    public static Long getDifDay(Date startDate, Date endDate) {
        return (endDate.getTime() - startDate.getTime()) / (1000 * 3600 * 24);
    }

    public static Integer getDifMonth(Date startDate, Date endDate) {
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(startDate);
        end.setTime(endDate);
        int result = end.get(Calendar.MONTH) - start.get(Calendar.MONTH);
        int month = (end.get(Calendar.YEAR) - start.get(Calendar.YEAR)) * 12;
        return Math.abs(month + result);
    }

    /**
     * 功能描述：计算【year + "岁" + month + "个月" + day + "天"】
     *
     * @param birthday
     * @return
     */
    public static String getBirthDistanceNow(Date birthday, Boolean isFullText) {
        Calendar now = Calendar.getInstance();
        Calendar birth = Calendar.getInstance();
        birth.setTime(birthday);
        int year = now.get(Calendar.YEAR) - birth.get(Calendar.YEAR);
        int month = now.get(Calendar.MONTH) - birth.get(Calendar.MONTH);
        int day = now.get(Calendar.DAY_OF_MONTH) - birth.get(Calendar.DAY_OF_MONTH);
        if (month < 0) {
            month = 12 - birth.get(Calendar.MONTH) + now.get(Calendar.MONTH);
            year -= 1;
        }
        if (day < 0) {
            day = birth.getMaximum(Calendar.DAY_OF_MONTH) - birth.get(Calendar.DAY_OF_MONTH) + now.get(Calendar.DAY_OF_MONTH);
            month -= 1;
        }

        if (isFullText) {
            return String.format("%s岁%s个月%s天", year, month, day);
        } else {
            StringBuilder sb = new StringBuilder();
            if (year > 0) {
                sb.append(year).append("岁");
            }
            if (month > 0) {
                sb.append(month).append("个月");
            }
            sb.append(day).append("天");
            return sb.toString();
        }
    }

    /**
     * 功能描述：计算【year + "岁" + month + "个月" + day + "天"】
     *
     * @param birthday
     * @return
     */
    public static String getBirthDistanceRecord(Date birthday, Date recordDay, Boolean isFullText) {
        Calendar record = Calendar.getInstance();
        Calendar birth = Calendar.getInstance();
        birth.setTime(birthday);
        record.setTime(recordDay);
        int year = record.get(Calendar.YEAR) - birth.get(Calendar.YEAR);
        int month = record.get(Calendar.MONTH) - birth.get(Calendar.MONTH);
        int day = record.get(Calendar.DAY_OF_MONTH) - birth.get(Calendar.DAY_OF_MONTH);
        if (month < 0) {
            month = 12 - birth.get(Calendar.MONTH) + record.get(Calendar.MONTH);
            year -= 1;
        }
        if (day < 0) {
            day = birth.getMaximum(Calendar.DAY_OF_MONTH) - birth.get(Calendar.DAY_OF_MONTH) + record.get(Calendar.DAY_OF_MONTH);
            month -= 1;
        }

        if (year < 0 || month < 0) {
            throw new RuntimeException("记录时间似乎比宝宝出生还早哦~");
        }

        if (isFullText) {
            return String.format("%s岁%s个月%s天", year, month, day);
        } else {
            StringBuilder sb = new StringBuilder();
            if (year > 0) {
                sb.append(year).append("岁");
            }
            if (month > 0) {
                sb.append(month).append("个月");
            }
            sb.append(day).append("天");
            return sb.toString();
        }
    }


    /**
     * 日期转星期
     *
     * @param datetime
     * @return
     */
    public static String dateToWeek(String datetime) {
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd");
        String[] weekDays = { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
        // 获得一个日历
        Calendar cal = Calendar.getInstance();
        Date datet = null;
        try {
            datet = f.parse(datetime);
            cal.setTime(datet);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        // 指示一个星期中的某天。
        int w = cal.get(Calendar.DAY_OF_WEEK) - 1;
        if (w < 0) {
            w = 0;
        }
        return weekDays[w];
    }


    /**
     * 月份转为中文月份
     *
     * @param month
     * @return
     */
    public static String monthToString(Integer month) {
        String[] weekDays = { "一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月" };
        return weekDays[month-1];
    }

    /**
     * @Author：
     * @Description：输入日期，获取输入日期的前一天
     * @Date：
     * @strData：参数格式：yyyy-MM-dd
     * @return：返回格式：yyyy-MM-dd
     */
    public static String getPreDateByDate(String strData) {
        String preDate = "";
        Calendar c = Calendar.getInstance();
        SimpleDateFormat sdf =new SimpleDateFormat("yyyy-MM-dd");
        Date date = null;
        try {
            date = sdf.parse(strData);
        } catch (java.text.ParseException e) {
            e.printStackTrace();
        }

        c.setTime(date);
        int day1 = c.get(Calendar.DATE);
        c.set(Calendar.DATE, day1 - 1);
        preDate = sdf.format(c.getTime());
        return preDate;
    }

}
