package com.studio.nx.framework.util.entity;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2020/5/23 11:07
 */
public class EntityUtil {

    /**
     * 将数组数据转换为实体类
     * 此处数组元素的顺序必须与实体类构造函数中的属性顺序一致
     *
     * @param list  数组对象集合
     * @param clazz 实体类
     * @param <T>   实体类
     * @param model 实例化的实体类
     * @return 实体类集合
     */
    public static <T> List<T> castEntity(List<Object[]> list, Class<T> clazz, Object model) {
        List<T> returnList = new ArrayList<T>();
        if (list.isEmpty()) {
            return returnList;
        }

        List<Map> attributeInfoList = getFieldsInfo(model);
        if (attributeInfoList.size() != list.get(0).length) {
            return returnList;
        }
        Class[] c2 = new Class[attributeInfoList.size()];
        for (int i = 0; i < attributeInfoList.size(); i++) {
            c2[i] = (Class) attributeInfoList.get(i).get("type");
        }
        try {
            for (Object[] o : list) {
                Constructor<T> constructor = clazz.getConstructor(c2);
                returnList.add(constructor.newInstance(o));
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return returnList;
        }

        return returnList;
    }

    public static <T> List<T> castEntity(List<Object[]> list, Class<T> clazz) {
        List<T> returnList = new ArrayList<T>();
        Object[] co = list.get(0);
        Class[] c2 = new Class[co.length];

        for (int i = 0; i < co.length; i++) {
            c2[i] = co[i].getClass();
        }

        try {
            for (Object[] o : list) {
                Constructor<T> constructor = clazz.getConstructor(c2);
                returnList.add(constructor.newInstance(o));
            }
        } catch (Exception ex) {
            return returnList;
        }

        return returnList;
    }

    private static Object getFieldValueByName(String fieldName, Object modle) {
        try {
            String firstLetter = fieldName.substring(0, 1).toUpperCase();
            String getter = "get" + firstLetter + fieldName.substring(1);
            Method method = modle.getClass().getMethod(getter, new Class[]{});
            Object value = method.invoke(modle, new Object[]{});
            return value;
        } catch (Exception e) {
            return null;
        }
    }

    private static List<Map> getFieldsInfo(Object model) {
        List<Field> fieldList = getAllFields(model);
        List<Map> list = new ArrayList(fieldList.size());
        for (Field field : fieldList) {
            // todo add by 梁展鹏 20200529 暂时先hardcode，BaseEntity的两个非持久化字段不需要做对象映射
            if ("createdByName".equals(field.getName()) || "lastModifiedByName".equals(field.getName())) {
                continue;
            }

            list.add(new HashMap<String, Object>() {{
                put("type", field.getType());
                put("name", field.getName());
                put("value", getFieldValueByName(field.getName(), model));
            }});
        }

        return list;
    }

    /**
     * 功能描述：获取对象的所有字段（包括多级父类）
     * 注意字段顺序，父类的的属性放在最前面
     *
     * @param model
     * @return
     */
    public static List<Field> getAllFields(Object model) {
        Class clazz = model.getClass();
        List<Field> fieldList = new ArrayList();
        while (clazz != null) {
            List<Field> clazzFieldList = new ArrayList<>(Arrays.asList(clazz.getDeclaredFields())); // 获取类的所有属性
            Collections.reverse(clazzFieldList); // 倒叙
            for (Field field : clazzFieldList) {
                fieldList.add(0, field); // 往首位插入对象
            }

            clazz = clazz.getSuperclass();
        }

        return fieldList;
    }

}
