package com.studio.nx.uiac.controller;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.Auth;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.uiac.constant.ImgCodeEnum;
import com.studio.nx.uiac.controller.request.auth.MobileNumberSmsCodeRequest;
import com.studio.nx.uiac.controller.request.auth.SignInRequest;
import com.studio.nx.uiac.controller.response.AccountInfoResponse;
import com.studio.nx.uiac.controller.response.ImgCodeDto;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.service.AuthMobileNumberSmsService;
import com.studio.nx.uiac.service.AuthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = "统一认证 - 管理后台")
@Slf4j
@RestController("FrameworkAuthAdminController")
@RequestMapping("/framework/uiac/admin")
public class AuthAdminController {

    @Autowired
    private AuthService authService;

    @Autowired
    private AuthMobileNumberSmsService authMobileNumberSmsService;

    @IgnoreAuth
    @ApiOperation(value = "获取图片验证码（登录图片验证码）")
    @GetMapping("/sign-in/img-code")
    public ResponseEntity<BaseResponse<ImgCodeDto>> signInImgCode() {
        return ResponseEntity.ok(BaseResponse.ok(authService.imgCode(ImgCodeEnum.SignUp)));
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码登录：1. 获取图片验证码")
    @GetMapping("/mobile-number-sms/img-code")
    public ResponseEntity<BaseResponse<ImgCodeDto>> mobileNumberSmsImgCode() {
        return ResponseEntity.ok(BaseResponse.ok(authService.imgCode(ImgCodeEnum.MobileNumberSms)));
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码登录：2. 获取短信验证码")
    @PostMapping("/mobile-number-sms/send-code")
    public ResponseEntity<BaseResponse<?>> mobileNumberSMSCode(@Valid @RequestBody BaseRequest<MobileNumberSmsCodeRequest> request) {
        authMobileNumberSmsService.sendMobileNumberSMSCode(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @IgnoreAuth
    @ApiOperation(value = "统一登录")
    @PostMapping("/sign-in")
    public ResponseEntity<BaseResponse<SignInResponse>> signIn(@Valid @RequestBody BaseRequest<SignInRequest> request, HttpServletRequest _request) {
        // todo add by 梁展鹏 20210830 判断允许登录管理后台的权限
        SignInResponse _response = authService.login(request.getData(), _request);
        return ResponseEntity.ok(BaseResponse.ok(_response));
    }

    @Auth
    @ApiOperation(value = "获取当前登录人信息")
    @GetMapping("/myself")
    public ResponseEntity<BaseResponse<AccountInfoResponse>> myself() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(authService.adminMyself(accountId)));
    }

    @Auth
    @ApiOperation(value = "刷新token")
    @GetMapping("/refresh")
    @Deprecated
    public ResponseEntity<BaseResponse> refresh(@RequestParam String refreshToken) {
//        String username = (String) redisTemplate.opsForHash().get(refreshToken, "username");
//        if (StringUtils.isEmpty(username)) {
//            return new ResponseEntity<>(new Result(HttpStatus.UNAUTHORIZED, "refreshToken error", null), HttpStatus.UNAUTHORIZED);
//        }

        /* 生成新的token */
//        UserDetails userDetails = null;
//        Authentication authentication = new UsernamePasswordAuthenticationToken(userDetails, "", null);
//        String newToken = jwtUtils.generateToken(authentication); // JWT签名
//        redisTemplate.opsForHash().put(refreshToken, "token", newToken);

//        newToken, refreshToken
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @Auth
    @ApiOperation(value = "退出登录")
    @GetMapping("/logout")
    public ResponseEntity<?> logout(HttpServletRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        authService.logout(accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
