package com.studio.nx.uiac.controller;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.uiac.constant.ImgCodeEnum;
import com.studio.nx.uiac.controller.request.auth.MobileNumberSmsCodeRequest;
import com.studio.nx.uiac.controller.request.auth.MobileNumberSmsSignUpRequest;
import com.studio.nx.uiac.controller.request.auth.SignInRequest;
import com.studio.nx.uiac.controller.response.ImgCodeDto;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.dto.AccountInfoDto;
import com.studio.nx.uiac.service.AuthMobileNumberSmsService;
import com.studio.nx.uiac.service.AuthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = {"统一认证 - 前台"})
@Slf4j
@RestController("FrameworkAuthController")
@RequestMapping("/framework/uiac")
public class AuthController {

    @Autowired
    private AuthService authService;

    @Autowired
    private AuthMobileNumberSmsService authMobileNumberSmsService;

    @IgnoreAuth
    @ApiOperation(value = "获取图片验证码（登录图片验证码）")
    @GetMapping("/sign-in/img-code")
    public ResponseEntity<BaseResponse<ImgCodeDto>> signInImgCode() {
        return ResponseEntity.ok(BaseResponse.ok(authService.imgCode(ImgCodeEnum.SignUp)));
    }

    @IgnoreAuth
    @ApiOperation(value = "统一登录")
    @PostMapping("/sign-in")
    public ResponseEntity<BaseResponse<SignInResponse>> signIn(@Valid @RequestBody BaseRequest<SignInRequest> request, HttpServletRequest _request) {
        SignInResponse _response = authService.login(request.getData(), _request);
        return ResponseEntity.ok(BaseResponse.ok(_response));
    }

    @ApiOperation(value = "当前登录人信息")
    @GetMapping("/myself")
    public ResponseEntity<BaseResponse<AccountInfoDto>> myself() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        return ResponseEntity.ok(BaseResponse.ok(authService.appMyself(accountId, nickname)));
    }

    @ApiOperation(value = "刷新token")
    @GetMapping("/refresh")
    @Deprecated
    public ResponseEntity<BaseResponse> refresh(@RequestParam String refreshToken) {
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码注册：1. 获取图片验证码")
    @GetMapping("/mobile-number-sms/img-code")
    public ResponseEntity<BaseResponse<ImgCodeDto>> mobileNumberSmsImgCode() {
        return ResponseEntity.ok(BaseResponse.ok(authService.imgCode(ImgCodeEnum.MobileNumberSms)));
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码注册：2. 获取短信验证码")
    @PostMapping("/mobile-number-sms/send-code")
    public ResponseEntity<BaseResponse> mobileNumberSMSCode(@Valid @RequestBody BaseRequest<MobileNumberSmsCodeRequest> request) {
        authMobileNumberSmsService.sendMobileNumberSMSCode(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码注册：3. 注册登录")
    @PostMapping("/mobile-number-sms/sign-up")
    public ResponseEntity<BaseResponse<?>> mobileNumberSMSSignUp(@Valid @RequestBody BaseRequest<MobileNumberSmsSignUpRequest> request, HttpServletRequest _request) {
        return ResponseEntity.ok(BaseResponse.ok(authMobileNumberSmsService.mobileNumberSMSSignUp(request.getData(), _request)));
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号码+短信验证码注册：3. 注册登录")
    @PostMapping("/h5/mobile-number-sms/sign-up")
    public ResponseEntity<BaseResponse<?>> h5MobileNumberSMSSignUp(@Valid @RequestBody BaseRequest<MobileNumberSmsSignUpRequest> request, HttpServletRequest _request) {
        authMobileNumberSmsService.h5MobileNumberSMSSignUp(request.getData(), _request);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @IgnoreAuth
    @ApiOperation(value = "忘记密码")
    @PostMapping("/forget-password")
    @Deprecated
    public ResponseEntity<BaseResponse<?>> forgetPassword() {
        return ResponseEntity.ok(BaseResponse.ok("感谢支持，敬请期待！"));
    }

    @IgnoreAuth
    @ApiOperation(value = "账号密码注册")
    @PostMapping("/account-password/sign-up")
    @Deprecated
    public ResponseEntity<BaseResponse<?>> accountPasswordSignUp() {
        return ResponseEntity.ok(BaseResponse.ok("感谢支持，敬请期待！"));
    }

    @IgnoreAuth
    @ApiOperation(value = "手机号密码注册")
    @PostMapping("/mobile-number-password/sign-up")
    @Deprecated
    public ResponseEntity<BaseResponse<?>> mobileNumberPasswordSignUp() {
        return ResponseEntity.ok(BaseResponse.ok("感谢支持，敬请期待！"));
    }

    @ApiOperation(value = "退出登录")
    @GetMapping("/logout")
    public ResponseEntity<?> logout(HttpServletRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        authService.logout(accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
