package com.studio.nx.uiac.controller;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.util.NodeUtils;
import com.studio.nx.uiac.controller.search.DeptSearchDto;
import com.studio.nx.uiac.dto.DeptResponseDto;
import com.studio.nx.uiac.entity.Dept;
import com.studio.nx.uiac.service.DeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "部门管理")
@Slf4j
@RestController("FrameworkDeptController")
@RequestMapping("/framework/uiac/dept")
public class DeptController {

    @Autowired
    private DeptService service;

    @ApiOperation(value = "查询[非顶级部门]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<List<DeptResponseDto>>> list(@Valid @RequestBody BaseRequest<DeptSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.list(request.getData())));
    }

    @ApiOperation(value = "查询[部门]节点树")
    @PostMapping("/tree")
    public ResponseEntity<BaseResponse<List<NodeUtils.ResponseNodeDto>>> tree() {
        return ResponseEntity.ok(BaseResponse.ok(service.tree()));
    }

    @OperationLog("保存 - 部门")
    @ApiOperation(value = "保存[部门]")
    @PostMapping("/save")
    public ResponseEntity<BaseResponse> save(@Valid @RequestBody Dept dto) {
        service.save(dto);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("更新 - 部门")
    @ApiOperation(value = "更新[部门]")
    @PostMapping("/update")
    public ResponseEntity<BaseResponse> update(@Valid @RequestBody Dept dto) {
        service.update(dto);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换删除状态 - 部门")
    @ApiOperation(value = "替换[部门]删除状态")
    @DeleteMapping("/del/{id:\\d+}")
    public ResponseEntity<BaseResponse> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
