package com.studio.nx.uiac.controller;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.uiac.controller.request.auth.Oauth2WeixinRequest;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.service.AuthOauth2Service;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = {"第三方登录 - 微信"})
@Slf4j
@RestController("FrameworkOauth2WeixinController")
@RequestMapping("/framework/oauth/weixin")
public class Oauth2WeixinController {

    @Autowired
    private AuthOauth2Service oauth2Service;

    @IgnoreAuth
    @ApiOperation(
            value = "绑定微信账号",
            notes = "备注：<br/>" +
                    "1. 已【手机号码】注册，绑定【微信账号】。<br/>" +
                    "1.1 方法入参必须传入【accountId】作为已有【手机号码】绑定。<br/>" +
                    "2. 未【手机号码】注册，绑定【微信账号】。<br/>" +
                    "2.1 先注册【微信账号】，后必须绑定【手机号码（短信验证码）】"
    )
    @PostMapping("/binding")
    public ResponseEntity<BaseResponse<SignInResponse>> binding(@Valid @RequestBody BaseRequest<Oauth2WeixinRequest> request, HttpServletRequest _request) {
        return ResponseEntity.ok(BaseResponse.ok(oauth2Service.bindingWeixin(request.getData(), _request)));
    }

}
