package com.studio.nx.uiac.controller;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.util.NodeUtils;
import com.studio.nx.uiac.controller.search.MenuSearchDto;
import com.studio.nx.uiac.dto.MenuResponseDto;
import com.studio.nx.uiac.entity.Resources;
import com.studio.nx.uiac.service.ResourcesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "资源管理")
@Slf4j
@RestController("FrameworkResourceController")
@RequestMapping("/framework/uiac/resource")
public class ResourcesController {

    @Autowired
    private ResourcesService service;

    @ApiOperation(value = "查询[非顶级菜单]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<List<MenuResponseDto>>> list(@Valid @RequestBody BaseRequest<MenuSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.list(request.getData())));
    }

    @ApiOperation(value = "查询[菜单]节点树")
    @PostMapping("/tree")
    public ResponseEntity<BaseResponse<List<NodeUtils.ResponseNodeDto>>> tree() {
        return ResponseEntity.ok(BaseResponse.ok(service.tree()));
    }

    @OperationLog("保存 - 菜单")
    @ApiOperation(value = "保存[菜单]")
    @PostMapping("/save")
    public ResponseEntity<BaseResponse> save(@Valid @RequestBody BaseRequest<Resources> request) {
        service.save(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("更新 - 菜单")
    @ApiOperation(value = "更新[菜单]")
    @PostMapping("/update")
    public ResponseEntity<BaseResponse> update(@Valid @RequestBody BaseRequest<Resources> request) {
        service.update(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换删除状态 - 菜单")
    @ApiOperation(value = "替换[菜单]删除状态")
    @DeleteMapping("/del/{id:\\d+}")
    public ResponseEntity<BaseResponse> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
