package com.studio.nx.uiac.controller;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.uiac.controller.response.RoleResponse;
import com.studio.nx.uiac.controller.search.RoleSearchDto;
import com.studio.nx.uiac.dto.RoleDto;
import com.studio.nx.uiac.dto.RoleMenuDto;
import com.studio.nx.uiac.service.ResourcesService;
import com.studio.nx.uiac.service.RoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "角色管理")
@Slf4j
@RestController("FrameworkRoleController")
@RequestMapping("/framework/uiac/role")
public class RoleController {

    @Autowired
    private RoleService service;

    @Autowired
    private ResourcesService resourcesService;

    @ApiOperation(value = "查询[角色]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<RoleResponse>>> page(@Valid @RequestBody BasePageRequest<RoleSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.page(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "查询[角色]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<List<RoleResponse>>> list(@Valid @RequestBody BaseRequest<RoleSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.list(request.getData())));
    }

    @OperationLog("保存 - 角色")
    @ApiOperation(value = "保存[角色]")
    @PostMapping("/save")
    public ResponseEntity<?> save(@Valid @RequestBody BaseRequest<RoleDto> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.save(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("更新 - 角色")
    @ApiOperation(value = "更新[角色]")
    @PostMapping("/update")
    public ResponseEntity<?> update(@Valid @RequestBody BaseRequest<RoleDto> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.update(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换删除状态 - 角色")
    @ApiOperation(value = "替换[角色]删除状态")
    @DeleteMapping("/del/{id:\\d+}")
    public ResponseEntity<?> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("绑定 - 角色与菜单的权限")
    @ApiOperation(value = "绑定 - 角色与菜单的权限")
    @PostMapping("/save/resources")
    public ResponseEntity<?> saveRoleResources(@Valid @RequestBody BaseRequest<RoleMenuDto> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.saveRoleResources(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
