package com.studio.nx.uiac.feign;

import com.studio.nx.uiac.config.FeignConfiguration;
import com.studio.nx.uiac.dto.weixin.WeixinAccessTokenResponse;
import com.studio.nx.uiac.dto.weixin.WeixinUserInfoResponse;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 功能描述：
 * https://developers.weixin.qq.com/doc/oplatform/Mobile_App/WeChat_Login/Development_Guide.html
 *
 * @Author 梁展鹏
 * @Date 2021/6/21 10:54
 */
@FeignClient(
        name = "WeixinFeign",
        url = "https://api.weixin.qq.com/sns",
        fallback = WeixinFeignFallback.class,
        configuration = FeignConfiguration.class
)
public interface WeixinFeign {

    /**
     * 功能描述：通过Authorization Code请求Access Token
     *
     * @param appId
     * @param appSecret
     * @param authorizationCode 客户端提供
     */
    @GetMapping(value = "/oauth2/access_token?appid={appId}&secret={appSecret}&code={authorizationCode}&grant_type=authorization_code")
    WeixinAccessTokenResponse accessToken(@RequestParam("appId") String appId, @RequestParam("appSecret") String appSecret, @RequestParam("authorizationCode") String authorizationCode);

    /**
     * 功能描述：通过Access Token请求Union Id
     *
     * @param accessToken accessToken接口返回
     * @param openId
     */
    @GetMapping(value = "/userinfo?access_token={accessToken}&openid={openId}")
    WeixinUserInfoResponse userinfo(@RequestParam("accessToken") String accessToken, @RequestParam("openId") String openId);

}
