package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.mapper.AccountAuthsMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class AccountAuthsService {

    @Autowired
    private AccountAuthsMapper mapper;

    public Optional<AccountAuths> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    public List<AccountAuths> findByAccountId(String accountId) {
        return mapper.selectList(new QueryWrapper<AccountAuths>().lambda()
                .eq(AccountAuths::getAccountId, accountId)
        );
    }

    public Optional<AccountAuths> findByUsername(String username) {
        return Optional.ofNullable(mapper.selectOne(new QueryWrapper<AccountAuths>().lambda()
                .eq(AccountAuths::getIdentityType, AccountAuths.IdentityType.ACCOUNT_PASSWORD)
                .eq(AccountAuths::getIdentifier, username)
        ));
    }

    public Optional<AccountAuths> findByMobileNumber(String mobileNumber) {
        return Optional.ofNullable(mapper.selectOne(new QueryWrapper<AccountAuths>().lambda()
                .eq(AccountAuths::getIdentityType, AccountAuths.IdentityType.MOBILE_NUMBER)
                .eq(AccountAuths::getIdentifier, mobileNumber)
        ));
    }

    public Optional<AccountAuths> findByAccountId(AccountAuths.IdentityType identityType, String accountId) {
        return Optional.ofNullable(mapper.selectOne(new QueryWrapper<AccountAuths>().lambda()
                .eq(AccountAuths::getAccountId, accountId)
                .eq(AccountAuths::getIdentityType, identityType)
        ));
    }

    public Optional<AccountAuths> findByIdentifier(AccountAuths.IdentityType identityType, String identifier) {
        return Optional.ofNullable(mapper.selectOne(new QueryWrapper<AccountAuths>().lambda()
                .eq(AccountAuths::getIdentityType, identityType)
                .eq(AccountAuths::getIdentifier, identifier)
        ));
    }

    public AccountAuths save(String accountId, AccountAuths.IdentityType identityType, String identifier, String credential, String salt) {
        AccountAuths _auths = new AccountAuths();
        _auths.setAccountId(accountId);
        _auths.setIdentityType(identityType.toString());
        _auths.setIdentifier(identifier);
        _auths.setCredential(credential);
        _auths.setSalt(salt);
        mapper.insert(_auths);

        return _auths;
    }

    public AccountAuths update(AccountAuths accountAuths) {
        AccountAuths _auths = findById(accountAuths.getId()).get();
        _auths.setIdentityType(accountAuths.getIdentityType());
        _auths.setIdentifier(accountAuths.getIdentifier());
        _auths.setCredential(accountAuths.getCredential());
        _auths.setSalt(accountAuths.getSalt());
        mapper.updateById(_auths);

        return _auths;
    }

}
