package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.studio.nx.uiac.entity.AccountInvite;
import com.studio.nx.uiac.mapper.AccountInviteMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/20 14:30
 */
@Service
public class AccountInviteService {

    @Autowired
    private AccountInviteMapper mapper;

    /**
     * 查询还没奖励的我的邀请人记录（实际上只有一条，因为只有注册后才会查询关联数据）
     *
     * @param inviteAccountId
     * @return
     */
    public List<AccountInvite> findByInviteAccountId(String inviteAccountId) {
        return mapper.selectList(new LambdaQueryWrapper<AccountInvite>()
                .eq(AccountInvite::getInviteAccountId, inviteAccountId)
                .eq(AccountInvite::getIsReward, Boolean.FALSE)
        );
    }

    /**
     * 功能描述：查询还没奖励的邀请记录
     *
     * @return
     */
    @Deprecated
    public List<AccountInvite> isReward() {
        return mapper.selectList(new LambdaQueryWrapper<AccountInvite>()
                .eq(AccountInvite::getIsReward, Boolean.FALSE)
        );
    }

    /**
     * 功能描述：设置邀请记录为奖励
     *
     * @param accountInviteId
     */
    public void setReward(String accountInviteId) {
        mapper.update(null, new LambdaUpdateWrapper<AccountInvite>()
                .eq(AccountInvite::getId, accountInviteId)
                .set(AccountInvite::getIsReward, Boolean.TRUE)
        );
    }

    /**
     * 功能描述：保存邀请记录
     *
     * @param accountId             邀请人
     * @param inviteAccountId       被邀请人
     * @param inviteAccountNickname 被邀请人
     */
    public void save(String accountId, String inviteAccountId, String inviteAccountNickname) {
        AccountInvite _entity = new AccountInvite();
        _entity.setAccountId(accountId);
        _entity.setInviteAccountId(inviteAccountId);
        _entity.setInviteAccountNickname(inviteAccountNickname);
        _entity.setIsReward(Boolean.FALSE);
        mapper.insert(_entity);
    }

}
