package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.entity.BaseEntity;
import com.studio.nx.uiac.constant.RedisKeyConstant;
import com.studio.nx.uiac.controller.request.my.PersonInfoRequest;
import com.studio.nx.uiac.controller.response.AccountAdminResponse;
import com.studio.nx.uiac.controller.search.AccountSearchDto;
import com.studio.nx.uiac.dto.AccountDto;
import com.studio.nx.uiac.entity.Account;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.entity.AccountDept;
import com.studio.nx.uiac.entity.AccountRole;
import com.studio.nx.uiac.mapper.AccountAuthsMapper;
import com.studio.nx.uiac.mapper.AccountDeptMapper;
import com.studio.nx.uiac.mapper.AccountMapper;
import com.studio.nx.uiac.mapper.AccountRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class AccountService {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private AccountMapper mapper;

    @Autowired
    private AccountAuthsMapper accountAuthsMapper;

    @Autowired
    private AccountAuthsService accountAuthsService;

    @Autowired
    private AccountDeptMapper accountDeptMapper;

    @Autowired
    private AccountRoleMapper accountRoleMapper;

    public Optional<Account> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    public AccountDto findCacheById(String id) {
        return (AccountDto) redisTemplate.opsForValue().get(RedisKeyConstant.PlatformAccountLogin(id));
    }

    public BasePageResponse<AccountAdminResponse> adminPage(AccountSearchDto params, Integer page, Integer size) {
        IPage<AccountAdminResponse> p = mapper.adminPage(params, new Page(page, size));

        for (AccountAdminResponse record : p.getRecords()) {
            String accountId = record.getId();
            List<AccountRole> accountRoleList = accountRoleMapper.selectList(new QueryWrapper<AccountRole>().lambda().eq(AccountRole::getAccountId, accountId).eq(AccountRole::getIsDel, Boolean.FALSE));
            record.setRoleIdList(accountRoleList.stream().map(AccountRole::getRoleId).collect(Collectors.toList()));
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public List<AccountAdminResponse> searchSelect(String keyword) {
        return mapper.searchSelect(keyword);
    }

    public Account save(Account _account) {
        _account.setPasswordFailureDateTime(LocalDateTime.now());
        _account.setEndDateTime(LocalDateTime.now());
        _account.setIsLocked(Boolean.FALSE);

        mapper.insert(_account);
        return _account;
    }

    public Account update(Account account) {
        Account _account = findById(account.getId()).get();
        _account.setNickname(account.getNickname());
        _account.setRealname(account.getRealname());
        _account.setAvatarUrl(account.getAvatarUrl());
        _account.setGender(account.getGender());
        // 以下字段暂不提供自行修改
//        _account.setType(account.getType());
//        _account.setPasswordFailureDateTime(LocalDateTime.now());
//        _account.setEndDateTime(LocalDateTime.now());
//        _account.setIsLocked(Boolean.FALSE);

        mapper.updateById(_account);
        return _account;
    }

    @Transactional
    public Account save(AccountDto account) {
        Account _account;
        if (StringUtils.isEmpty(account.getId())) {
            _account = new Account();
            _account.setNickname(account.getNickname());
            _account.setIsLocked(account.getIsLocked());
            LocalDateTime date = account.getEndDateTime();
//            if (date != null) { todo
//                Instant instant = date.toInstant();
//                ZoneId zoneId = ZoneId.systemDefault();
//                _account.setEndDateTime(instant.atZone(zoneId).toLocalDateTime());
//            }
            _account.setPasswordFailureDateTime(null);
            _account.setType(account.getType());
            _account.setGender(account.getGender());
            mapper.insert(_account);

            AccountAuths auths = new AccountAuths();
            auths.setAccountId(_account.getId());
            auths.setIdentityType(AccountAuths.IdentityType.ACCOUNT_PASSWORD.getName());
//            auths.setIdentifier(account.getIdentifier());
//            auths.setCredential(account.getCredential());
//            accountAuthsService.save(auths);

            AccountDept dept = new AccountDept();
            dept.setAccountId(_account.getId());
//            dept.setDeptId(account.getDeptId());
            accountDeptMapper.insert(dept);

//            for (String roleId : account.getRoleIds()) {
//                AccountRole accountRole = new AccountRole();
//                accountRole.setAccountId(_account.getId());
//                accountRole.setRoleId(roleId);
//                accountRoleMapper.insert(accountRole);
//            }
        } else {
            _account = findById(account.getId()).get();
            LambdaUpdateWrapper<Account> wrapper = new UpdateWrapper<Account>().lambda();
            wrapper.eq(BaseEntity::getId, _account.getId());
            wrapper.set(Account::getNickname, account.getNickname());
            wrapper.set(Account::getIsLocked, account.getIsLocked());
            wrapper.set(Account::getType, account.getType());
            wrapper.set(Account::getGender, account.getGender());
            LocalDateTime date = account.getEndDateTime();
//            if (date != null) { todo
//                Instant instant = date.toInstant();
//                ZoneId zoneId = ZoneId.systemDefault();
//                wrapper.set(Account::getEndDateTime, instant.atZone(zoneId).toLocalDateTime());
//            } else {
//                wrapper.set(Account::getEndDateTime, null);
//            }
            mapper.update(null, wrapper);

            List<AccountAuths> authsList = accountAuthsService.findByAccountId(account.getId());
            AccountAuths auths;
            if (authsList.isEmpty()) {
                auths = new AccountAuths();
                auths.setAccountId(account.getId());
            } else {
                auths = authsList.get(0);
            }
            auths.setIdentityType(AccountAuths.IdentityType.ACCOUNT_PASSWORD.getName());
//            auths.setIdentifier(account.getIdentifier());
//            auths.setCredential(account.getCredential());
//            accountAuthsService.save(auths);

            AccountDept dept = accountDeptMapper.selectOne(new QueryWrapper<AccountDept>().lambda().eq(AccountDept::getAccountId, account.getId()));
            if (dept == null) {
                dept = new AccountDept();
                dept.setAccountId(account.getId());
//                dept.setDeptId(account.getDeptId());
                accountDeptMapper.insert(dept);
            } else {
//                dept.setDeptId(account.getDeptId());
                accountDeptMapper.updateById(dept);
            }

            accountRoleMapper.delete(new QueryWrapper<AccountRole>().lambda().eq(AccountRole::getAccountId, account.getId()));
//            for (String roleId : account.getRoleIds()) {
//                AccountRole accountRole = new AccountRole();
//                accountRole.setAccountId(_account.getId());
//                accountRole.setRoleId(roleId);
//                accountRoleMapper.insert(accountRole);
//            }
        }

        return _account;
    }

    @Transactional
    public Account update(AccountDto account) {
        Account _account;
        if (StringUtils.isEmpty(account.getId())) {
            _account = new Account();
            _account.setNickname(account.getNickname());
            _account.setIsLocked(account.getIsLocked());
            LocalDateTime date = account.getEndDateTime();
//            if (date != null) { todo
//                Instant instant = date.toInstant();
//                ZoneId zoneId = ZoneId.systemDefault();
//                _account.setEndDateTime(instant.atZone(zoneId).toLocalDateTime());
//            }
            _account.setPasswordFailureDateTime(null);
            _account.setType(account.getType());
            _account.setGender(account.getGender());
            mapper.insert(_account);

            AccountAuths auths = new AccountAuths();
            auths.setAccountId(_account.getId());
            auths.setIdentityType(AccountAuths.IdentityType.ACCOUNT_PASSWORD.getName());
//            auths.setIdentifier(account.getIdentifier());
//            auths.setCredential(account.getCredential());
//            accountAuthsService.save(auths);

            AccountDept dept = new AccountDept();
            dept.setAccountId(_account.getId());
//            dept.setDeptId(account.getDeptId());
            accountDeptMapper.insert(dept);

//            for (String roleId : account.getRoleIds()) {
//                AccountRole accountRole = new AccountRole();
//                accountRole.setAccountId(_account.getId());
//                accountRole.setRoleId(roleId);
//                accountRoleMapper.insert(accountRole);
//            }
        } else {
            _account = findById(account.getId()).get();
            LambdaUpdateWrapper<Account> wrapper = new UpdateWrapper<Account>().lambda();
            wrapper.eq(BaseEntity::getId, _account.getId());
            wrapper.set(Account::getNickname, account.getNickname());
            wrapper.set(Account::getIsLocked, account.getIsLocked());
            wrapper.set(Account::getType, account.getType());
            wrapper.set(Account::getGender, account.getGender());
            LocalDateTime date = account.getEndDateTime();
//            if (date != null) { todo
//                Instant instant = date.toInstant();
//                ZoneId zoneId = ZoneId.systemDefault();
//                wrapper.set(Account::getEndDateTime, instant.atZone(zoneId).toLocalDateTime());
//            } else {
//                wrapper.set(Account::getEndDateTime, null);
//            }
            mapper.update(null, wrapper);

            List<AccountAuths> authsList = accountAuthsService.findByAccountId(account.getId());
            AccountAuths auths;
            if (authsList.isEmpty()) {
                auths = new AccountAuths();
                auths.setAccountId(account.getId());
            } else {
                auths = authsList.get(0);
            }
            auths.setIdentityType(AccountAuths.IdentityType.ACCOUNT_PASSWORD.getName());
//            auths.setIdentifier(account.getIdentifier());
//            auths.setCredential(account.getCredential());
//            accountAuthsService.save(auths);

            AccountDept dept = accountDeptMapper.selectOne(new QueryWrapper<AccountDept>().lambda().eq(AccountDept::getAccountId, account.getId()));
            if (dept == null) {
                dept = new AccountDept();
                dept.setAccountId(account.getId());
//                dept.setDeptId(account.getDeptId());
                accountDeptMapper.insert(dept);
            } else {
//                dept.setDeptId(account.getDeptId());
                accountDeptMapper.updateById(dept);
            }

            accountRoleMapper.delete(new QueryWrapper<AccountRole>().lambda().eq(AccountRole::getAccountId, account.getId()));
//            for (String roleId : account.getRoleIds()) {
//                AccountRole accountRole = new AccountRole();
//                accountRole.setAccountId(_account.getId());
//                accountRole.setRoleId(roleId);
//                accountRoleMapper.insert(accountRole);
//            }
        }

        return _account;
    }

    @Transactional
    public void handleIsDel(String id) {
        Optional<Account> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Account account = optional.get();
            String accountId = account.getId();
            accountDeptMapper.delete(new QueryWrapper<AccountDept>().lambda().eq(AccountDept::getAccountId, accountId));
            accountAuthsMapper.delete(new QueryWrapper<AccountAuths>().lambda().eq(AccountAuths::getAccountId, accountId));
            accountRoleMapper.delete(new QueryWrapper<AccountRole>().lambda().eq(AccountRole::getAccountId, accountId));
            account.setIsDel(!account.getIsDel());
            mapper.updateById(account);
        }
    }

    /**
     * App 更新个人信息
     * @param request
     * @param accountId
     */
    public void appUpdateMine(PersonInfoRequest request, String accountId) {
        Optional<Account> optional = Optional.ofNullable(mapper.selectById(accountId));
        if (optional.isPresent()) {
            Account account = optional.get();
            if (StringUtils.hasText(request.getAvatarUrl())) {
                account.setAvatarUrl(request.getAvatarUrl());
            }
            if (StringUtils.hasText(request.getNickname())) {
                account.setNickname(request.getNickname());
            }
            if (request.getGender() != null) {
                account.setGender(request.getGender());
            }

            mapper.updateById(account);
        }
    }

}
