package com.studio.nx.uiac.service;

import com.studio.nx.framework.error.exception.NxFrameworkException;
import com.studio.nx.uiac.entity.Account;
import com.studio.nx.uiac.entity.AccountAuths;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Optional;
import java.util.regex.Pattern;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/8 17:21
 */
@Service
public class AuthCheckAccountService {

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountAuthsService accountAuthsService;

    /**
     * 功能描述：
     *
     * @param mobileNumber 手机号码
     * @return
     */
    public Account checkAccountMobileNumber(String mobileNumber) {
        /* 只考虑国内11位手机号码 */
        if (!Pattern.compile("^\\d{11}$").matcher(mobileNumber.trim()).matches()) {
            throw new NxFrameworkException(String.format("手机号码 [%s] 格式非法", mobileNumber));
        }

        Optional<AccountAuths> optional = accountAuthsService.findByMobileNumber(mobileNumber);
        if (!optional.isPresent()) {
            throw new NxFrameworkException("不存在该用户!");
        }

        /* 校验账号 */
        Account _account = checkAccount(optional.get().getAccountId());

        return _account;
    }

    /**
     * 功能描述：
     *
     * @param account  账号/邮箱/手机号码
     * @param password 密码
     * @return
     */
    public Account checkAccountPassword(String account, String password) {
        /* todo 目前是根据账号查询，后期可以扩展 */
        Optional<AccountAuths> optional = accountAuthsService.findByUsername(account);
        if (!optional.isPresent()) {
            throw new NxFrameworkException("不存在该用户!");
        }

        /* 验证密码 */
        String dbPassword = optional.get().getCredential();
        if (!dbPassword.equals(password)) {
            throw new NxFrameworkException(String.format("账号 [%s] 密码非法"));
        }

        /* 校验账号 */
        Account _account = checkAccount(optional.get().getAccountId());

        return _account;
    }

    private Account checkAccount(String accountId) {
        Account _account = accountService.findById(accountId).get();
        if (!Optional.ofNullable(_account).isPresent()) {
            throw new NxFrameworkException("账号异常!");
        }

        if (_account.getIsDel()) {
            throw new NxFrameworkException(String.format("账号 [%s] 已被禁用", _account.getNickname()));
        } else if (_account.getIsLocked()) {
            throw new NxFrameworkException(String.format("账号 [%s] 已被锁定", _account.getNickname()));
        } else if (null != _account.getEndDateTime() && _account.getEndDateTime().isAfter(LocalDateTime.now())) {
            throw new NxFrameworkException(String.format("账号 [%s] 过期", _account.getNickname()));
        }

        return _account;
    }

}
