package com.studio.nx.uiac.service;

import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.CircleCaptcha;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONUtil;
import com.studio.nx.framework.error.exception.NxFrameworkException;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.framework.service.OperationLogService;
import com.studio.nx.oss.local.config.LocalYaml;
import com.studio.nx.uiac.constant.ImgCodeEnum;
import com.studio.nx.uiac.constant.RedisKeyConstant;
import com.studio.nx.uiac.controller.request.auth.SignInRequest;
import com.studio.nx.uiac.controller.response.AccountInfoResponse;
import com.studio.nx.uiac.controller.response.ImgCodeDto;
import com.studio.nx.uiac.controller.response.RoleResponse;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.dto.AccountDto;
import com.studio.nx.uiac.dto.AccountInfoDto;
import com.studio.nx.uiac.entity.Account;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.entity.AccountInvite;
import com.studio.nx.uiac.entity.AccountMapStruct;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 */
@RequiredArgsConstructor
@Service
public class AuthService {

    @Value("${spring.profiles.active}")
    private String active;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private LocalYaml localYaml;

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountInviteService accountInviteService;

    @Autowired
    private AccountAuthsService accountAuthsService;

    @Autowired
    private AuthMobileNumberSmsService authMobileNumberSmsService;

    @Autowired
    private AuthCheckAccountService authCheckAccountService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private OnlineAccountService onlineAccountService;

    @Autowired
    private OperationLogService operationLogService;

    /**
     * 功能描述：生成图片验证码
     * 1. 通过hutool生成图片验证码
     * 2. 生成uuid随机字符
     * 3. 把uuid作为key，图片验证码字符作为value，存入redis中。有效期为60妙
     *
     * @return
     */
    public ImgCodeDto imgCode(ImgCodeEnum imgCodeEnum) {
        CircleCaptcha captcha = CaptchaUtil.createCircleCaptcha(130, 48, 4, 20);
        String verCode = captcha.getCode().toLowerCase();
        String uuid = IdUtil.simpleUUID();

        // 存入redis并设置过期时间为60秒
        stringRedisTemplate.opsForValue().set(RedisKeyConstant.PlatformLoginImgCode(imgCodeEnum.name(), uuid), verCode, 60, TimeUnit.SECONDS);

        // 将key和base64返回给前端
        ImgCodeDto dto = new ImgCodeDto();
        dto.setBase64Img(captcha.getImageBase64Data());
        dto.setUuid(uuid);

        return dto;
    }

    public void checkImgCode(ImgCodeEnum imgCodeEnum, String uuid, String code) {
        /* 校验验证码是否有效 */
        String verCode = stringRedisTemplate.opsForValue().get(RedisKeyConstant.PlatformLoginImgCode(imgCodeEnum.name(), uuid));
        if (StringUtils.isEmpty(verCode)) {
            throw new NxFrameworkException("验证码不存在或已过期");
        }
        if (!verCode.equalsIgnoreCase(code.trim())) {
            throw new NxFrameworkException("验证码错误");
        }

        /* 验证码通过后才手动删除 */
        stringRedisTemplate.delete(RedisKeyConstant.PlatformLoginImgCode(imgCodeEnum.name(), uuid));
    }

    public SignInResponse login(SignInRequest request, HttpServletRequest _request) {
//        Account _account = null;
//        switch (request.getSignInType()) {
//            case ACCOUNT_PASSWORD:
//            case MOBILE_NUMBER_PASSWORD:
//                /* 校验图片验证码 */
//                checkImgCode(ImgCodeEnum.SignUp, request.getUuid(), request.getCode());
//
//                /* 检查账号、密码是否正确 */
//                _account = authCheckAccountService.checkAccountPassword(request.getAccount(), request.getPassword());
//                break;
//            case MOBILE_NUMBER_SMS:
//                /* 判断登录的手机号码是白名单 */
//                if (StringUtils.hasText(getWhiteList().get(request.getAccount()))) {
//                    String code = getWhiteList().get(request.getAccount());
//                    Assert.isTrue(code.equalsIgnoreCase(request.getSmsCode().trim()), "验证码错误");
//                } else {
//                    authMobileNumberSmsService.checkMobileNumberSMSCode(request.getAccount(), request.getSmsCode());
//                }
//
//                _account = authCheckAccountService.checkAccountMobileNumber(request.getAccount());
//                break;
//        }
//
//        if (!Optional.ofNullable(_account).isPresent()) {
//            throw new NxFrameworkException("账号数据异常");
//        }

        //写死方便调试
        return createSignInResponse("1418474590370840577", "梁展鹏", "", _request);
    }

    public SignInResponse createSignInResponse(String accountId, String nickname, String avatarUrl, HttpServletRequest _request) {
        /* 生成访问令牌 */
        String token = JWTUtil.createJWT(accountId);
        String refreshToken = createRefreshToken(token, accountId);

        /* 保存在线账号 */
        onlineAccountService.goOnline(accountId, nickname, avatarUrl, _request, token, refreshToken);
        operationLogService.insertLoginLog(_request, nickname, "客户端登录");

        /* 一个账号只能同时登录一个地方(dev环境暂不约束) */
        if (!"dev".equals(active)) {
            onlineAccountService.checkLogin(accountId, token);
        }

        /* 登录账号前奖励邀请人积分 */
        List<AccountInvite> myInviteList = accountInviteService.findByInviteAccountId(accountId);
        myInviteList.stream().forEach(o -> {
            stringRedisTemplate.opsForList().leftPush("yxl:integral-cash:integral_invite_to_sign_up", JSONUtil.toJsonStr(o));
        });

        SignInResponse _response = new SignInResponse();
        _response.setToken(token);
        _response.setRefreshToken(refreshToken);
        return _response;
    }

    private String createRefreshToken(String jwt, String accountId) {
        String refreshToken = UUID.randomUUID().toString();

        // 存入redis并设置过期时间为60秒
        stringRedisTemplate.opsForHash().put(RedisKeyConstant.PlatformRefreshToken(refreshToken), "jwt", jwt);
        stringRedisTemplate.opsForHash().put(RedisKeyConstant.PlatformRefreshToken(refreshToken), "accountId", accountId);
        stringRedisTemplate.expire(RedisKeyConstant.PlatformRefreshToken(refreshToken), JWTUtil.REFRESH_TOKEN_EXPIRE_TIME, TimeUnit.MILLISECONDS);

        return refreshToken;
    }

    public void logout(String accountId) {
        AccountDto accountDto = accountService.findCacheById(accountId);
        if (Optional.ofNullable(accountDto).isPresent()) {
            onlineAccountService.logout(accountDto.getId());
        }
    }

    public AccountInfoDto appMyself(String accountId, String nickname) {
        Optional<Account> optionalAccount = accountService.findById(accountId);
        if (!optionalAccount.isPresent()) {
            throw new NxFrameworkException(String.format("账号 [%s] 出错", nickname));
        }

        Account _account = optionalAccount.get();
        AccountInfoDto _response = AccountMapStruct.INSTANCES.toResponse(_account);
        if (!StringUtils.hasText(_response.getAvatarUrl())) {
            _response.setAvatarUrl(getDefaultAvatarUrl());
        }

        List<RoleResponse> roleResponseList = roleService.findRoleByAccountId(accountId);
        _response.setRoleList(roleResponseList.stream().filter(o -> Arrays.asList("user", "photoer", "kindergarten").indexOf(o.getCode()) >= 0).collect(Collectors.toList())); // 角色集合
        _response.setDeptList(Collections.EMPTY_LIST); // 部门集合

        Optional<AccountAuths> accountAuthsMobileNumberOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.MOBILE_NUMBER, accountId);
        if (accountAuthsMobileNumberOptional.isPresent()) {
            _response.setMobileNumber(accountAuthsMobileNumberOptional.get().getIdentifier()); // 手机号码
        }

        Optional<AccountAuths> accountAuthsWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, accountId);
        if (accountAuthsWeixinOpenAppOpenOptional.isPresent()) {
            _response.setWeixinOpenAppOpen(accountAuthsWeixinOpenAppOpenOptional.get().getIdentifier()); // 微信开放平台-移动应用-openid
        }

        /* 用默认头像字段判断是否需要到完善信息页面 */
        _response.setIsPerfectForm(_response.getAvatarUrl().indexOf("/static/yxl/default-avatar-url.png") >= 0);

        return _response;
    }

    public AccountInfoResponse adminMyself(String accountId) {
        AccountDto accountDto = accountService.findCacheById(accountId);
        if (!Optional.ofNullable(accountDto).isPresent()) {
            throw new NxFrameworkException("登录态失效.");
        }

        AccountInfoResponse _response = new AccountInfoResponse();
        _response.setNickname(accountDto.getNickname());
        if (!StringUtils.hasText(_response.getAvatar())) {
            _response.setAvatar(getDefaultAvatarUrl());
        }
        _response.setIntroduction("这是简介");

        /* 返回账号的角色code集合 */
        List<RoleResponse> roleResponseDtoList = roleService.findRoleByAccountId(accountId);
        List<String> roleCodeList = roleResponseDtoList.stream().map(RoleResponse::getCode).collect(Collectors.toList());
        _response.setRoles(roleCodeList);

        return _response;
    }

    public String getDefaultAvatarUrl() {
        return String.format("%s/static/yxl/default-avatar-url.png", localYaml.getHost());
    }

    public Map<String, String> getWhiteList() {
        /* 校验短信验证码 */
        Map<String, String> map = new HashMap<>();
        map.put("13822625622", "337821"); // 超级用户
        map.put("13450765204", "000000"); // 超级用户
        map.put("18938447892", "464739"); // 设计
        map.put("13030268517", "984947"); // 摄影师管理专员
//        map.put("17688442306", "859660"); // 照片管理专员
        map.put("15724082576", "247493"); // 运营
        return map;
    }

}
