package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.entity.BaseEntity;
import com.studio.nx.framework.util.NodeUtils;
import com.studio.nx.uiac.controller.search.DeptSearchDto;
import com.studio.nx.uiac.dto.DeptResponseDto;
import com.studio.nx.uiac.entity.Dept;
import com.studio.nx.uiac.mapper.DeptMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
public class DeptService {

    @Autowired
    private DeptMapper mapper;

    public BasePageResponse<DeptResponseDto> page(DeptSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Dept> wrapper = new QueryWrapper<Dept>().lambda();
        if (!StringUtils.isEmpty(dto.getName())) { // 有传入部门名称时，按部门名称查询
            wrapper.like(Dept::getName, dto.getName());
        } else { // 默认查询顶级部门
            wrapper.isNull(Dept::getParentId);
        }
        wrapper.eq(Dept::getIsDel, dto.getIsDel());
        wrapper.orderByDesc(Dept::getSort);
        Page<Dept> p = mapper.selectPage(new Page(page, size), wrapper);

        List<DeptResponseDto> list = new ArrayList();
        for (Dept d : p.getRecords()) { // 判断节点是否有子节点
            DeptResponseDto respDto = new DeptResponseDto();
            BeanUtils.copyProperties(d, respDto);
            wrapper = new QueryWrapper<Dept>().lambda();
            wrapper.eq(Dept::getParentId, d.getId()).eq(BaseEntity::getIsDel, Boolean.FALSE);
            Integer count = mapper.selectCount(wrapper);
            if (count > 0) {
                respDto.setHasChildren(true);
            } else {
                respDto.setHasChildren(false);
            }
            list.add(respDto);
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    public List<NodeUtils.ResponseNodeDto> tree() {
        LambdaQueryWrapper<Dept> wrapper = new QueryWrapper<Dept>().lambda().eq(BaseEntity::getIsDel, Boolean.FALSE);
        List<Dept> deptList = mapper.selectList(wrapper);

        List<NodeUtils.ResponseNodeDto> list = new ArrayList();
        for (Dept dept : deptList) {
            NodeUtils.ResponseNodeDto nodeDto = new NodeUtils.ResponseNodeDto();
            BeanUtils.copyProperties(dept, nodeDto);

            nodeDto.setLabel(dept.getName());
            nodeDto.setValue(dept.getId());
            list.add(nodeDto);
        }

        return NodeUtils.buildTree(list);
    }

    public List<DeptResponseDto> list(DeptSearchDto dto) {
        LambdaQueryWrapper<Dept> wrapper = new QueryWrapper<Dept>().lambda();
        if (!StringUtils.isEmpty(dto.getParentId())) {
            wrapper.eq(Dept::getParentId, dto.getParentId());
        }
        wrapper.eq(Dept::getIsDel, Boolean.FALSE);
        wrapper.orderByDesc(Dept::getSort);
        List<Dept> deptList = mapper.selectList(wrapper);
        if (deptList.isEmpty()) {
            return null;
        }
        List<DeptResponseDto> list = new ArrayList();
        for (Dept dept : deptList) {
            DeptResponseDto respDto = new DeptResponseDto();
            BeanUtils.copyProperties(dept, respDto);
            wrapper = new QueryWrapper<Dept>().lambda();
            wrapper.eq(Dept::getParentId, dept.getId()).eq(BaseEntity::getIsDel, Boolean.FALSE);
            Integer count = mapper.selectCount(wrapper);
            if (count > 0) {
                respDto.setHasChildren(true);
            } else {
                respDto.setHasChildren(false);
            }
            list.add(respDto);
        }
        return list;
    }

    public Dept save(Dept dto) {
        Dept _dept = new Dept();
        if (StringUtils.isEmpty(dto.getParentId())) {
            _dept.setParentId(null);
        } else {
            _dept.setParentId(dto.getParentId());
        }
        _dept.setName(dto.getName());
        _dept.setSort(dto.getSort());
        mapper.insert(_dept);

        return _dept;
    }

    public Dept update(Dept dto) {
        Dept _dept = Optional.ofNullable(mapper.selectById(dto.getId())).orElseGet(Dept::new);
        LambdaUpdateWrapper<Dept> wrapper = new UpdateWrapper<Dept>().lambda();
        wrapper.eq(BaseEntity::getId, _dept.getId());
        // 当parentId为空时，把该字段置为null，即节点修改为顶级节点
        if (StringUtils.isEmpty(dto.getParentId())) {
            wrapper.set(Dept::getParentId, null);
        } else {
            wrapper.set(Dept::getParentId, dto.getParentId());
        }

        wrapper.set(Dept::getName, dto.getName());
        wrapper.set(Dept::getSort, dto.getSort());
        mapper.update(null, wrapper);

        return _dept;
    }

    public void handleIsDel(String id) {
        LambdaQueryWrapper<Dept> wrapper = new QueryWrapper<Dept>().lambda();
        wrapper.eq(BaseEntity::getIsDel, Boolean.FALSE).eq(Dept::getParentId, id);
        List<Dept> list = mapper.selectList(wrapper);
        Optional<Dept> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Dept dept = optional.get();
            dept.setIsDel(!dept.getIsDel());
            mapper.updateById(dept);
        }

        // 如果该节点有子节点，继续遍历删除其子节点
        if (!list.isEmpty()) {
            for (Dept dept : list) {
                handleIsDel(dept.getId());
            }
        }
    }

}
