package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.entity.BaseEntity;
import com.studio.nx.framework.util.NodeUtils;
import com.studio.nx.uiac.controller.search.MenuSearchDto;
import com.studio.nx.uiac.dto.MenuResponseDto;
import com.studio.nx.uiac.entity.Resources;
import com.studio.nx.uiac.mapper.ResourcesMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
public class ResourcesService {

    @Autowired
    private ResourcesMapper mapper;

    public BasePageResponse<MenuResponseDto> page(MenuSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Resources> wrapper = new QueryWrapper<Resources>().lambda();
        if (!StringUtils.isEmpty(dto.getName())) { // 有传入名称时，按名称查询
            wrapper.like(Resources::getName, dto.getName());
        } else { // 默认查询顶级菜单
            wrapper.eq(Resources::getResourceType, Resources.ResourceType.CATALOG).isNull(Resources::getParentId);
        }
        wrapper.eq(Resources::getIsDel, dto.getIsDel());
        wrapper.orderByDesc(Resources::getSort);
        Page<Resources> p = mapper.selectPage(new Page(page, size), wrapper);

        List<MenuResponseDto> list = new ArrayList();
        for (Resources m : p.getRecords()) { // 判断节点是否有子节点
            MenuResponseDto respDto = new MenuResponseDto();
            BeanUtils.copyProperties(m, respDto);
            wrapper = new QueryWrapper<Resources>().lambda();
            wrapper.eq(Resources::getParentId, m.getId()).eq(BaseEntity::getIsDel, Boolean.FALSE);
            Integer count = mapper.selectCount(wrapper);
            if (count > 0) {
                respDto.setHasChildren(true);
            } else {
                respDto.setHasChildren(false);
            }
            list.add(respDto);
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    public List<NodeUtils.ResponseNodeDto> tree() {
        List<Resources> menuList = mapper.selectList(new QueryWrapper<Resources>().lambda().eq(BaseEntity::getIsDel, Boolean.FALSE));

        List<NodeUtils.ResponseNodeDto> list = new ArrayList();
        for (Resources menu : menuList) {
            NodeUtils.ResponseNodeDto nodeDto = new NodeUtils.ResponseNodeDto();
            BeanUtils.copyProperties(menu, nodeDto);

            nodeDto.setLabel(menu.getName());
            nodeDto.setValue(menu.getId());
            list.add(nodeDto);
        }

        return NodeUtils.buildTree(list);
    }

    public List<MenuResponseDto> list(MenuSearchDto dto) {
        LambdaQueryWrapper<Resources> wrapper = new QueryWrapper<Resources>().lambda();
        if (!StringUtils.isEmpty(dto.getParentId())) {
            wrapper.eq(Resources::getParentId, dto.getParentId());
        }
        wrapper.eq(Resources::getIsDel, Boolean.FALSE);
        wrapper.orderByDesc(Resources::getSort);
        List<Resources> menuList = mapper.selectList(wrapper);
        if (menuList.isEmpty()) {
            return null;
        }
        List<MenuResponseDto> list = new ArrayList();
        for (Resources menu : menuList) {
            MenuResponseDto respDto = new MenuResponseDto();
            BeanUtils.copyProperties(menu, respDto);
            wrapper = new QueryWrapper<Resources>().lambda();
            wrapper.eq(Resources::getParentId, menu.getId()).eq(BaseEntity::getIsDel, Boolean.FALSE);
            Integer count = mapper.selectCount(wrapper);
            if (count > 0) {
                respDto.setHasChildren(true);
            } else {
                respDto.setHasChildren(false);
            }
            list.add(respDto);
        }

        return list;
    }

    public Resources save(Resources dto) {
        // 改变菜单类型时清空无用数据
        if (dto.getResourceType().equals(Resources.ResourceType.CATALOG)) {
            dto.setPermission("");
            dto.setComponentName("");
            dto.setComponentPath("");
        } else if (dto.getResourceType().equals(Resources.ResourceType.BUTTON)) {
            dto.setPath("");
            dto.setComponentPath("");
            dto.setComponentName("");
        }

        Resources _resources = new Resources();
        if (StringUtils.isEmpty(dto.getParentId())) {
            _resources.setParentId(null);
        } else {
            _resources.setParentId(dto.getParentId());
        }
        _resources.setComponentName(dto.getComponentName());
        _resources.setIcon(dto.getIcon());
        _resources.setCache(dto.getCache());
        _resources.setSort(dto.getSort());
        _resources.setHidden(dto.getHidden());
        _resources.setPath(dto.getPath());
        _resources.setComponentPath(dto.getComponentPath());
        _resources.setLink(dto.getLink());
        _resources.setPermission(dto.getPermission());
        _resources.setName(dto.getName());
        _resources.setResourceType(dto.getResourceType());
        _resources.setRemarks(dto.getRemarks());
        mapper.insert(_resources);

        return _resources;
    }

    public Resources update(Resources dto) {
        // 改变菜单类型时清空无用数据
        if (Resources.ResourceType.CATALOG.equals(dto.getResourceType())) { // 如果当前修改为目录时
            dto.setPermission(""); // 权限字符串
            dto.setComponentName(""); // 组件名称
            dto.setComponentPath(""); // 组件路径
        } else if (Resources.ResourceType.BUTTON.equals(dto.getResourceType())) { // 如果当前修改为按钮时
            dto.setPath(""); // 路由地址
            dto.setComponentName(""); // 组件名称
            dto.setComponentPath(""); // 组件路径
        }

        Resources _resources = null;
        Optional<Resources> newsOptional = Optional.ofNullable(mapper.selectById(dto.getId()));
        if (newsOptional.isPresent()) {
            _resources = newsOptional.get();
            LambdaUpdateWrapper<Resources> wrapper = new UpdateWrapper<Resources>().lambda();
            wrapper.eq(BaseEntity::getId, _resources.getId());
            // 当parentId为空时，把该字段置为null，即节点修改为顶级节点
            if (StringUtils.isEmpty(dto.getParentId())) {
                wrapper.set(Resources::getParentId, null);
            } else {
                wrapper.set(Resources::getParentId, dto.getParentId());
            }
            wrapper.set(Resources::getComponentName, dto.getComponentName());
            wrapper.set(Resources::getIcon, dto.getIcon());
            wrapper.set(Resources::getCache, dto.getCache());
            wrapper.set(Resources::getSort, dto.getSort());
            wrapper.set(Resources::getHidden, dto.getHidden());
            wrapper.set(Resources::getPath, dto.getPath());
            wrapper.set(Resources::getComponentPath, dto.getComponentPath());
            wrapper.set(Resources::getLink, dto.getLink());
            wrapper.set(Resources::getPermission, dto.getPermission());
            wrapper.set(Resources::getName, dto.getName());
            wrapper.set(Resources::getResourceType, dto.getResourceType());
            wrapper.set(Resources::getRemarks, dto.getRemarks());
            mapper.update(null, wrapper);
        }

        return _resources;
    }

    public void handleIsDel(String id) {
        LambdaQueryWrapper<Resources> wrapper = new QueryWrapper<Resources>().lambda();
        wrapper.eq(Resources::getIsDel, Boolean.FALSE).eq(Resources::getParentId, id);
        List<Resources> list = mapper.selectList(wrapper);
        Optional<Resources> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Resources menu = optional.get();
            menu.setIsDel(!menu.getIsDel());
            mapper.updateById(menu);
        }

        // 如果该节点有子节点，继续遍历删除其子节点
        if (!list.isEmpty()) {
            for (Resources menu : list) {
                handleIsDel(menu.getId());
            }
        }
    }

}
