package com.studio.nx.uiac.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.entity.BaseEntity;
import com.studio.nx.uiac.controller.response.RoleResponse;
import com.studio.nx.uiac.controller.search.RoleSearchDto;
import com.studio.nx.uiac.dto.RoleDeptSaveDto;
import com.studio.nx.uiac.dto.RoleDto;
import com.studio.nx.uiac.dto.RoleMenuDto;
import com.studio.nx.uiac.dto.RoleResourceSaveDto;
import com.studio.nx.uiac.entity.Role;
import com.studio.nx.uiac.entity.RoleDept;
import com.studio.nx.uiac.entity.RoleMapStruct;
import com.studio.nx.uiac.entity.RoleResources;
import com.studio.nx.uiac.mapper.ResourcesMapper;
import com.studio.nx.uiac.mapper.RoleDeptMapper;
import com.studio.nx.uiac.mapper.RoleMapper;
import com.studio.nx.uiac.mapper.RoleResourcesMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class RoleService {

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private RoleDeptMapper roleDeptMapper;

    @Autowired
    private RoleResourcesMapper roleResourcesMapper;

    @Autowired
    private ResourcesMapper resourceMapper;

    public BasePageResponse<RoleResponse> page(RoleSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Role> wrapper = new QueryWrapper<Role>().lambda();
        if (StringUtils.hasText(dto.getName())) {
            wrapper.like(Role::getName, dto.getName());
        }
        wrapper.eq(Role::getIsDel, dto.getIsDel());
        Page<Role> p = roleMapper.selectPage(new Page(page, size), wrapper);

        ArrayList<RoleResponse> list = new ArrayList();
        for (Role record : p.getRecords()) {
            RoleResponse roleDto = new RoleResponse();
            BeanUtils.copyProperties(record, roleDto);

            List<RoleDept> roleDepts = roleDeptMapper.selectList(new QueryWrapper<RoleDept>().lambda().eq(RoleDept::getRoleId, record.getId()));
            ArrayList<String> rDeptIds = new ArrayList();
            for (RoleDept roleDept : roleDepts) {
                rDeptIds.add(roleDept.getDeptId());
            }
            roleDto.setDeptIds(rDeptIds);

            List<RoleResources> roleResources = roleResourcesMapper.selectList(new QueryWrapper<RoleResources>().lambda().eq(RoleResources::getRoleId, record.getId()));
            List<String> rResourceIds = new ArrayList();
            for (RoleResources roleResource : roleResources) {
                rResourceIds.add(roleResource.getResourcesId());
            }
            roleDto.setMenuIds(rResourceIds);
            list.add(roleDto);
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    public List<RoleResponse> list(RoleSearchDto dto) {
        LambdaQueryWrapper<Role> wrapper = new QueryWrapper<Role>().lambda();
        if (StringUtils.hasText(dto.getName())) {
            wrapper.like(Role::getName, dto.getName());
        }
        wrapper.eq(Role::getIsDel, dto.getIsDel());
        List<Role> roleList = roleMapper.selectList(wrapper);

        ArrayList<RoleResponse> list = new ArrayList();
        for (Role record : roleList) {
            RoleResponse roleDto = new RoleResponse();
            BeanUtils.copyProperties(record, roleDto);

            List<RoleDept> roleDepts = roleDeptMapper.selectList(new QueryWrapper<RoleDept>().lambda().eq(RoleDept::getRoleId, record.getId()));
            ArrayList<String> rDeptIds = new ArrayList();
            for (RoleDept roleDept : roleDepts) {
                rDeptIds.add(roleDept.getDeptId());
            }
            roleDto.setDeptIds(rDeptIds);

            List<RoleResources> roleResources = roleResourcesMapper.selectList(new QueryWrapper<RoleResources>().lambda().eq(RoleResources::getRoleId, record.getId()));
            List<String> rResourceIds = new ArrayList();
            for (RoleResources roleResource : roleResources) {
                rResourceIds.add(roleResource.getResourcesId());
            }
            roleDto.setMenuIds(rResourceIds);
            list.add(roleDto);
        }

        return list;
    }

    @Transactional
    @CacheEvict(cacheNames = "role-list", keyGenerator = "redisDefaultKeyGenerator")
    public Role save(RoleDto dto, String accountId) {
        Assert.isTrue(!findByCode(dto.getCode()).isPresent(), "数据错误");

        Role _role = new Role();
        _role.setName(dto.getName());
        _role.setCode(dto.getCode());
        _role.setRemarks(dto.getRemarks());
        roleMapper.insert(_role);

        /* 绑定角色与部门的关联关系 */
        if (dto.getDeptIds().size() > 0) {
            RoleDeptSaveDto _dto = new RoleDeptSaveDto();
            _dto.setRoleId(_role.getId());
            _dto.setAccountId(accountId);
            _dto.setDeptIdList(dto.getDeptIds());
            roleDeptMapper.saveRoleDept(_dto);
        }

        return _role;
    }

    @Transactional
    @CacheEvict(cacheNames = "role-list", keyGenerator = "redisDefaultKeyGenerator")
    public Role update(RoleDto dto, String accountId) {
        Role _role = Optional.ofNullable(roleMapper.selectById(dto.getId())).orElseGet(Role::new);

        _role.setName(dto.getName());
        _role.setCode(dto.getCode());
        _role.setRemarks(dto.getRemarks());

        if (StringUtils.hasText(_role.getId())) {
            roleMapper.updateById(_role);
        } else {
            roleMapper.insert(_role);
        }

        /* 删除角色与部门的关联关系 */
        roleDeptMapper.update(null, new UpdateWrapper<RoleDept>().lambda().eq(RoleDept::getRoleId, _role.getId()).set(RoleDept::getIsDel, Boolean.TRUE));

        /* 绑定角色与部门的关联关系 */
        if (dto.getDeptIds().size() > 0) {
            RoleDeptSaveDto _dto = new RoleDeptSaveDto();
            _dto.setRoleId(_role.getId());
            _dto.setAccountId(accountId);
            _dto.setDeptIdList(dto.getDeptIds());
            roleDeptMapper.saveRoleDept(_dto);
        }

        return _role;
    }

    @Transactional
    @CacheEvict(cacheNames = "role-list", keyGenerator = "redisDefaultKeyGenerator")
    public void handleIsDel(String id) {
        Optional<Role> optional = Optional.ofNullable(roleMapper.selectById(id));
        if (optional.isPresent()) {
            Role role = optional.get();
            roleMapper.update(null, new UpdateWrapper<Role>().lambda().eq(Role::getId, role.getId()).set(BaseEntity::getIsDel, !role.getIsDel()));
            roleDeptMapper.update(null, new UpdateWrapper<RoleDept>().lambda().eq(RoleDept::getRoleId, role.getId()).set(BaseEntity::getIsDel, !role.getIsDel()));
        }
    }

    @Transactional
    public void saveRoleResources(RoleMenuDto dto, String accountId) {
        /* 删除角色与资源的关联关系 */
        roleResourcesMapper.update(null, new UpdateWrapper<RoleResources>().lambda().eq(RoleResources::getRoleId, dto.getRoleId()).set(RoleResources::getIsDel, Boolean.TRUE));

        /* 绑定角色与资源的关联关系 */
        RoleResourceSaveDto _dto = new RoleResourceSaveDto();
        _dto.setRoleId(dto.getRoleId());
        _dto.setAccountId(accountId);
        _dto.setResourceIdList(dto.getMenuIds());
        roleResourcesMapper.saveRoleResource(_dto);
    }

    @Cacheable(cacheNames = "role-list", keyGenerator = "redisDefaultKeyGenerator")
    public List<RoleResponse> list() {
        List<RoleResponse> list = new ArrayList<>();
        List<Role> roleList = roleMapper.selectList(new LambdaQueryWrapper<Role>()
                .eq(Role::getIsDel, Boolean.FALSE)
        );

        for (Role role : roleList) {
            list.add(RoleMapStruct.INSTANCES.toResponse(role));
        }

        return list;
    }

    public Optional<RoleResponse> findByCode(String code) {
        return list().stream().filter(o -> code.equals(o.getCode())).findFirst();
    }

    public List<RoleResponse> findRoleByAccountId(String accountId) {
        List<RoleResponse> list = new ArrayList<>();
        List<Role> roleList = roleMapper.findRoleByAccountId(accountId);
        for (Role role : roleList) {
            list.add(RoleMapStruct.INSTANCES.toResponse(role));
        }

        return list;
    }

}
