package com.studio.nx.address.controller;

import com.studio.nx.address.entity.Address;
import com.studio.nx.address.service.AddressService;
import com.studio.nx.framework.dto.resp.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotBlank;
import java.util.List;

@Api(tags = "地址")
@Slf4j
@RestController
@RequestMapping("/service/address")
public class AddressController {

    @Autowired
    private AddressService addressService;

    @ApiOperation(value = "查询[地址名称]底下的地址信息", response = Address.class)
    @ApiImplicitParams(
            @ApiImplicitParam(name = "name", value = "地址名称", dataType = "String", paramType = "path", required = true, defaultValue = "广州市")
    )
    @GetMapping("/{name}/child")
    public BaseResponse<List<Address>> addressFindNextLevel(@NotBlank @PathVariable("name") String name) {
        return BaseResponse.ok(addressService.findNextLevel(name));
    }

}
