package com.studio.nx.app.controller;

import com.studio.nx.app.controller.request.AppSaveRequest;
import com.studio.nx.app.controller.response.AppResponse;
import com.studio.nx.app.controller.search.AppSearch;
import com.studio.nx.app.entity.App;
import com.studio.nx.app.service.AppService;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

@Api(tags = "App自更新")
@Slf4j
@Validated
@RestController("AppController")
@RequestMapping("/service/app")
public class AppController {

    @Autowired
    private AppService service;

    @ApiOperation(value = "根据平台查询[App自更新]")
    @GetMapping("/{versionCode:\\d+}/{platform}")
    public ResponseEntity<BaseResponse<AppResponse>> getByPlatform(@NotNull @PathVariable("versionCode") Integer versionCode, @NotBlank @PathVariable("platform") String platform) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.getByPlatform("幼享乐", versionCode, platform)));
    }

    @ApiOperation(value = "查询[App自更新]列表")
    @PostMapping("/admin/page")
    public ResponseEntity<BaseResponse<BasePageResponse<App>>> page(@Valid @RequestBody BasePageRequest<AppSearch> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    @OperationLog("保存[App自更新]")
    @ApiOperation(value = "保存[App自更新]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<AppSaveRequest> request) {
        service.save(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换[App自更新]显示/隐藏状态")
    @ApiOperation(value = "替换[App自更新]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> isShow(@Valid @ApiParam("标识") @PathVariable String id) {
        service.isShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换[App自更新]是否强制更新状态")
    @ApiOperation(value = "替换[App自更新]是否强制更新状态")
    @PutMapping("/{id:\\d+}/is-force-upgrade")
    public ResponseEntity<BaseResponse<?>> isForceUpgrade(@Valid @ApiParam("标识") @PathVariable String id) {
        service.isForceUpgrade(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
