package com.studio.nx.app.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.app.controller.request.AppSaveRequest;
import com.studio.nx.app.controller.response.AppResponse;
import com.studio.nx.app.controller.search.AppSearch;
import com.studio.nx.app.entity.App;
import com.studio.nx.app.entity.AppMapStruct;
import com.studio.nx.app.mapper.AppMapper;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AppService {

    @Autowired
    private AppMapper mapper;

    public AppResponse getByPlatform(String productName, Integer versionCode, String platform) {
        List<App> appList = mapper.selectList(new LambdaQueryWrapper<App>()
                .eq(App::getProductName, productName)
                .eq(App::getPlatform, platform)
                .gt(App::getVersionCode, versionCode)
                .eq(App::getIsDel, Boolean.FALSE)
                .orderByDesc(App::getCreateAt)
        );

        if (CollectionUtils.isEmpty(appList)) {
            return new AppResponse().setCode(0);
        }

        AppResponse response = AppMapStruct.INSTANCES.toResponse(appList.get(0));
        if (appList.stream().filter(o -> o.getIsForceUpgrade()).collect(Collectors.toList()).size() > 0) {
            response.setCode(2);
        } else {
            response.setCode(1);
        }

        return response;
    }

    public BasePageResponse<App> adminPage(AppSearch search, Integer page, Integer size) {
        LambdaQueryWrapper<App> wrapper = new QueryWrapper().lambda();
        if (StringUtils.hasText(search.getProductName())) {
            wrapper.eq(App::getProductName, search.getProductName());
        }
        if (StringUtils.hasText(search.getPlatform())) {
            wrapper.eq(App::getPlatform, search.getPlatform());
        }
        if (null != search.getVersionCode()) {
            wrapper.eq(App::getVersionCode, search.getVersionCode());
        }
        if (StringUtils.hasText(search.getVersionNumber())) {
            wrapper.like(App::getVersionNumber, search.getVersionNumber());
        }
        if (null != search.getIsShow()) {
            wrapper.eq(App::getIsShow, search.getIsShow());
        }
        if (null != search.getIsForceUpgrade()) {
            wrapper.eq(App::getIsForceUpgrade, search.getIsForceUpgrade());
        }
        wrapper.orderByDesc(App::getCreateAt);

        Page<App> p = mapper.selectPage(new Page(page, size), wrapper);

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public void save(AppSaveRequest request) {
        App _entity = new App();
        _entity.setProductName(request.getProductName()); // 产品名称
        _entity.setPlatform(request.getPlatform()); // 平台
        _entity.setVersionCode(request.getVersionCode()); // 版本编码
        _entity.setVersionNumber(request.getVersionNumber()); // 版本号
        _entity.setIsShow(Boolean.FALSE); // 是否有效
        _entity.setIsForceUpgrade(request.getIsForceUpgrade()); // 是否强制更新
        _entity.setUpgradeDesc(request.getUpgradeDesc()); // 升级描述
        _entity.setDownloadUrl(request.getDownloadUrl()); // 下载地址

        mapper.insert(_entity);
    }

    public void update(String id, AppSaveRequest request) {
        App _entity = mapper.selectById(id);
        if (!Optional.ofNullable(_entity).isPresent()) {
            return;
        }

        _entity.setProductName(request.getProductName()); // 产品名称
        _entity.setPlatform(request.getPlatform()); // 平台
        _entity.setVersionCode(request.getVersionCode()); // 版本编码
        _entity.setVersionNumber(request.getVersionNumber()); // 版本号
//        _entity.setIsShow(request.getIsShow()); // 是否有效
        _entity.setIsForceUpgrade(request.getIsForceUpgrade()); // 是否强制更新
        _entity.setUpgradeDesc(request.getUpgradeDesc()); // 升级描述
        _entity.setDownloadUrl(request.getDownloadUrl()); // 下载地址

        mapper.updateById(_entity);
    }

    public void isShow(String id) {
        App _entity = mapper.selectById(id);
        if (!Optional.ofNullable(_entity).isPresent()) {
            return;
        }

        /* 下架当前同产品、同平台的所以记录 */
        mapper.update(null, new LambdaUpdateWrapper<App>()
                .eq(App::getProductName, _entity.getProductName())
                .eq(App::getPlatform, _entity.getPlatform())
                .set(App::getIsShow, Boolean.FALSE)
        );

        /* 上架当前记录 */
        if (!_entity.getIsShow()) {
            _entity.setIsShow(Boolean.TRUE); // 是否有效
            mapper.updateById(_entity);
        }
    }

    public void isForceUpgrade(String id) {
        App _entity = mapper.selectById(id);
        if (!Optional.ofNullable(_entity).isPresent()) {
            return;
        }

        _entity.setIsForceUpgrade(!_entity.getIsForceUpgrade());
        mapper.updateById(_entity);
    }

}
