package com.studio.nx.config.controller;

import com.studio.nx.config.controller.request.BusinessConfigRequest;
import com.studio.nx.config.entity.BusinessConfig;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "业务配置")
@Slf4j
@Validated
@RestController("ServiceBusinessConfigController")
@RequestMapping("/service/business-config")
public class BusinessConfigController {

    @Autowired
    private BusinessConfigService service;

    @ApiOperation(value = "查询[业务配置]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<List<BusinessConfig>>> list(@Valid @RequestBody BaseRequest request) {
        return ResponseEntity.ok(BaseResponse.ok(service.list()));
    }

    @OperationLog("保存[业务配置]")
    @ApiOperation(value = "保存[业务配置]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<List<BusinessConfigRequest>> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.save(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
