package com.studio.nx.config.service;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.config.controller.request.BusinessConfigRequest;
import com.studio.nx.config.entity.BusinessConfig;
import com.studio.nx.config.mapper.BusinessConfigMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class BusinessConfigService {

    @Autowired
    private BusinessConfigMapper mapper;

    @Cacheable(cacheNames = "service:business-config", keyGenerator = "redisDefaultKeyGenerator")
    public List<BusinessConfig> list() {
        return mapper.selectList(new LambdaQueryWrapper<BusinessConfig>());
    }

    @CacheEvict(cacheNames = "service:business-config", keyGenerator = "redisDefaultKeyGenerator")
    public void save(List<BusinessConfigRequest> list, String accountId) {
        for (BusinessConfigRequest request : list) {
            save(request, accountId);
        }
    }

    @CacheEvict(cacheNames = "service:business-config", keyGenerator = "redisDefaultKeyGenerator")
    public void save(BusinessConfigRequest request, String accountId) {
        String id = IdUtil.createSnowflake(1L, 1L).nextIdStr();
        String code = request.getCode();
        String value = request.getValue();
        mapper.save(id, code, value, accountId);
    }

    public String getByKey(String code) {
        Optional<BusinessConfig> optional = list().stream().filter(o -> code.equals(o.getCode())).findFirst();
        return optional.isPresent() ? optional.get().getValue() : "";
    }

}
