package com.studio.nx.datadict.controller;

import com.studio.nx.datadict.controller.request.DictEntrySearchDto;
import com.studio.nx.datadict.controller.request.DictSearchDto;
import com.studio.nx.datadict.entity.Dict;
import com.studio.nx.datadict.entity.DictEntry;
import com.studio.nx.datadict.service.DictService;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * https://www.cnblogs.com/wm-dv/p/11155588.html
 */
@Api(tags = "字典管理")
@Slf4j
@RestController("ServiceDictController")
@RequestMapping("/service/dict")
public class DictController {

    @Autowired
    private DictService dictService;

    @ApiOperation(value = "查询[字典]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<Dict>>> page(@Valid @RequestBody BasePageRequest<DictSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(dictService.page(request.getData(), request.getPage(), request.getLimit())));
    }

    @OperationLog("更新 - 字典")
    @ApiOperation(value = "保存/更新[字典]")
    @PostMapping("/save-or-update")
    public ResponseEntity<BaseResponse<?>> saveOrUpdate(@Valid @RequestBody BasePageRequest<Dict> request) {
        if (dictService.saveOrUpdate(request.getData()) != null) {
            return ResponseEntity.ok(BaseResponse.ok());
        }
        return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "字典名称或已存在"));
    }

    @OperationLog("替换删除状态 - 字典")
    @ApiOperation(value = "替换[字典]删除状态")
    @PostMapping("/del/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> handleIsDel(@Valid @ApiParam("标识") @PathVariable String id) {
        dictService.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "查询[字典项]列表")
    @PostMapping("/entry/list")
    public ResponseEntity<BaseResponse<List<DictEntry>>> entryList(@Valid @RequestBody BaseRequest<DictEntrySearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(dictService.entryList(request.getData())));
    }

    @OperationLog("更新 - 字典项")
    @ApiOperation(value = "保存/更新[字典项]")
    @PostMapping("/entry/save-or-update")
    public ResponseEntity<BaseResponse<?>> entrySaveOrUpdate(@Valid @RequestBody BasePageRequest<DictEntry> request) {
        if (dictService.entrySaveOrUpdate(request.getData()) != null) {
            return ResponseEntity.ok(BaseResponse.ok());
        }
        return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "字典名称或已存在"));
    }

    @OperationLog("替换删除状态 - 字典项")
    @ApiOperation(value = "替换[字典项]删除状态")
    @PostMapping("/entry/del/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> entryHandleIsDel(@Valid @ApiParam("标识") @PathVariable String id) {
        dictService.entryHandleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
