package com.studio.nx.datadict.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.datadict.controller.request.DictEntrySearchDto;
import com.studio.nx.datadict.controller.request.DictSearchDto;
import com.studio.nx.datadict.entity.Dict;
import com.studio.nx.datadict.entity.DictEntry;
import com.studio.nx.datadict.mapper.DictEntryMapper;
import com.studio.nx.datadict.mapper.DictMapper;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

@Service
public class DictService {

    @Autowired
    private DictMapper dictMapper;

    @Autowired
    private DictEntryMapper dictEntryMapper;

    @Transactional
    public Dict saveOrUpdate(Dict dict) {
        Dict _dict = null;
        if (StringUtils.isEmpty(dict.getId())) {
            Integer count = dictMapper.selectCount(new QueryWrapper<Dict>().lambda()
                    .eq(Dict::getName, dict.getName())
                    .eq(Dict::getIsDel, Boolean.FALSE)
            );
            if (count != 0) {
                return null;
            }
            _dict = new Dict();
            _dict.setName(dict.getName());
            _dict.setRemarks(dict.getRemarks());
            dictMapper.insert(_dict);
        } else {
            Optional<Dict> newsOptional = Optional.ofNullable(dictMapper.selectById(dict.getId()));
            if (newsOptional.isPresent()) {
                _dict = newsOptional.get();
                Integer count = dictMapper.selectCount(new QueryWrapper<Dict>().lambda()
                        .ne(Dict::getId, _dict.getId())
                        .eq(Dict::getName, dict.getName())
                        .eq(Dict::getIsDel, Boolean.FALSE)
                );
                if (count != 0) {
                    return null;
                }
                _dict.setRemarks(dict.getRemarks());
                _dict.setName(dict.getName());
                dictMapper.updateById(_dict);
            }
        }

        return _dict;
    }

    public BasePageResponse<Dict> page(DictSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Dict> wrapper = new QueryWrapper().lambda();
        if (!StringUtils.isEmpty(dto.getCode())) {
            wrapper.like(Dict::getCode, dto.getCode());
        }
        if (!StringUtils.isEmpty(dto.getName())) {
            wrapper.like(Dict::getName, dto.getName());
        }
        wrapper.eq(Dict::getIsDel, dto.getIsDel());

        Page<Dict> p = dictMapper.selectPage(new Page(page, size), wrapper);
        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public void handleIsDel(String id) {
        Optional<Dict> optional = Optional.ofNullable(dictMapper.selectById(id));
        if (optional.isPresent()) {
            Dict dict = optional.get();
            dict.setIsDel(!dict.getIsDel());
            dictMapper.updateById(dict);
        }
    }

    public List<DictEntry> entryList(DictEntrySearchDto request) {
        LambdaQueryWrapper<Dict> dictWrapper = new LambdaQueryWrapper();
        if (StringUtils.hasText(request.getDictId())) {
            dictWrapper.eq(Dict::getId, request.getDictId());
        }
        if (StringUtils.hasText(request.getDictCode())) {
            dictWrapper.eq(Dict::getCode, request.getDictCode());
        }
        if (null != request.getIsDel()) {
            dictWrapper.eq(Dict::getIsDel, request.getIsDel());
        }
        Dict dict = dictMapper.selectOne(dictWrapper);

        if (Optional.ofNullable(dict).isPresent()) {
            LambdaQueryWrapper<DictEntry> dictEntryWrapper = new LambdaQueryWrapper();

            dictEntryWrapper.eq(DictEntry::getDictId, dict.getId());
            if (null != request.getIsDel()) {
                dictEntryWrapper.eq(DictEntry::getIsDel, request.getIsDel());
            }
            dictEntryWrapper.orderByDesc(DictEntry::getSort);
            dictEntryWrapper.orderByAsc(DictEntry::getId);

            return dictEntryMapper.selectList(dictEntryWrapper);
        }

        return Collections.EMPTY_LIST;
    }

    public DictEntry entrySaveOrUpdate(DictEntry entry) {
        DictEntry _dictEntry = null;
        if (StringUtils.isEmpty(entry.getId())) {
            Integer count = dictEntryMapper.selectCount(new QueryWrapper<DictEntry>().lambda()
                    .eq(DictEntry::getName, entry.getName())
                    .eq(DictEntry::getIsDel, Boolean.FALSE)
                    .eq(DictEntry::getDictId, entry.getDictId())
            );
            if (count != 0) {
                return null;
            }
            _dictEntry = new DictEntry();
            _dictEntry.setName(entry.getName());
            _dictEntry.setDictId(entry.getDictId());
            _dictEntry.setValue(entry.getValue());
            _dictEntry.setSort(entry.getSort());
            dictEntryMapper.insert(_dictEntry);
        } else {
            Optional<DictEntry> newsOptional = Optional.ofNullable(dictEntryMapper.selectById(entry.getId()));
            if (newsOptional.isPresent()) {
                _dictEntry = newsOptional.get();
                Integer count = dictEntryMapper.selectCount(new QueryWrapper<DictEntry>().lambda()
                        .ne(DictEntry::getId, entry.getId())
                        .eq(DictEntry::getName, entry.getName())
                        .eq(DictEntry::getIsDel, Boolean.FALSE)
                        .eq(DictEntry::getDictId, entry.getDictId())
                );
                if (count != 0) {
                    return null;
                }
                _dictEntry.setName(entry.getName());
                _dictEntry.setDictId(entry.getDictId());
                _dictEntry.setValue(entry.getValue());
                _dictEntry.setSort(entry.getSort());
                dictEntryMapper.updateById(_dictEntry);
            }
        }
        return _dictEntry;
    }

    public void entryHandleIsDel(String id) {
        Optional<DictEntry> optional = Optional.ofNullable(dictEntryMapper.selectById(id));
        if (optional.isPresent()) {
            DictEntry dictEntry = optional.get();
            dictEntry.setIsDel(!dictEntry.getIsDel());
            dictEntryMapper.updateById(dictEntry);
        }
    }

}
