package com.studio.nx.feedback.controller;

import com.studio.nx.feedback.controller.request.FeedbackApplyRequest;
import com.studio.nx.feedback.controller.response.FeedbackAdminPageResponse;
import com.studio.nx.feedback.controller.search.FeedbackSearch;
import com.studio.nx.feedback.service.FeedbackService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.local.service.LocalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

@Api(tags = "意见反馈")
@Slf4j
@RestController("ServiceFeedbackController")
@RequestMapping("/framework/feedback")
public class FeedbackController {

    @Autowired
    private FeedbackService service;

    @Autowired
    private LocalService localService;

    @ApiOperation(value = "上传[意见反馈]图片")
    @PostMapping("/kindergarten/public/upload")
    public ResponseEntity<BaseResponse<?>> kindergartenPublicUpload(@RequestPart(name = "file", required = false) MultipartFile file) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        FileResponse response = localService.upload(file, "/feedback/" + accountId);
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    /**
     * 功能描述：提交[意见反馈]记录
     *
     * @return
     */
    @ApiOperation(value = "提交[意见反馈]记录")
    @PostMapping("/apply")
    public ResponseEntity<BaseResponse<?>> apply(@Valid @RequestBody BasePageRequest<FeedbackApplyRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        service.apply(request.getData(), accountId, nickname);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：查询[意见反馈]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[意见反馈]列表")
    @PostMapping("/admin/page")
    public ResponseEntity<BaseResponse<BasePageResponse<FeedbackAdminPageResponse>>> page(@Valid @RequestBody BasePageRequest<FeedbackSearch> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

}
