package com.studio.nx.feedback.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.feedback.controller.request.FeedbackApplyRequest;
import com.studio.nx.feedback.controller.response.FeedbackAdminPageResponse;
import com.studio.nx.feedback.controller.search.FeedbackSearch;
import com.studio.nx.feedback.entity.Feedback;
import com.studio.nx.feedback.mapper.FeedbackMapper;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.oss.local.service.LocalService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class FeedbackService {

    @Autowired
    private LocalService localService;

    @Autowired
    private FeedbackMapper mapper;

    /**
     * 功能描述：查询[问题内容]列表
     */
    public BasePageResponse<FeedbackAdminPageResponse> adminPage(FeedbackSearch search, Integer page, Integer size) {
        IPage<FeedbackAdminPageResponse> p = mapper.adminPage(search, new Page(page, size));
        List<FeedbackAdminPageResponse> list = p.getRecords();
        for (FeedbackAdminPageResponse response : list) {
            /* 如果该反馈没有图片信息则跳过拼接URL前缀 */
            if (!StringUtils.hasText(response.getImgUrl())) {
                continue;
            }

            String imgUrl = Arrays.asList(response.getImgUrl().split(",")).stream()
                    .map(o -> localService.fullPath(o))
                    .collect(Collectors.joining(","));
            response.setImgUrl(imgUrl);
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    public void apply(FeedbackApplyRequest request, String accountId, String nickname) {
        Feedback _entity = new Feedback();

        _entity.setAccountId(accountId); // 账号标识
        _entity.setContent(request.getContent()); // 反馈内容

        String imgUrl = Arrays.asList(request.getImgUrl().split(",")).stream()
                .map(o -> localService.cutPath(o)) // 截取URL地址
                .collect(Collectors.joining(","));
        _entity.setImgUrl(imgUrl); // 反馈图片地址（可多个）

        mapper.insert(_entity);
    }

}
