package com.studio.nx.help.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.help.controller.search.HelpContentSearchDto;
import com.studio.nx.help.dto.HelpContentItemDto;
import com.studio.nx.help.dto.HelpContentListDto;
import com.studio.nx.help.entity.HelpContent;
import com.studio.nx.help.mapper.HelpContentMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class HelpContentService {

    @Autowired
    private HelpContentMapper mapper;

    @Autowired
    private HelpGroupService helpGroupService;

    /**
     * 功能描述：查询[问题内容]详情
     *
     * @param id 标识
     * @return
     */
    public Optional<HelpContent> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    /**
     * 功能描述：查询[问题内容]列表
     */
    public BasePageResponse<HelpContent> page(HelpContentSearchDto dto, Integer page, Integer size) {
        IPage<HelpContentListDto> _page = mapper.adminPage(dto,new Page(page, size));
        return new BasePageResponse().setTotal(_page.getTotal()).setData(_page.getRecords());
    }

    /**
     * 功能描述：保存[问题内容]
     *
     * @param dto
     */
    public HelpContent saveItem(HelpContentItemDto dto) {
        HelpContent _helpContent = new HelpContent();
        _helpContent.setName(dto.getName());
        _helpContent.setContent(dto.getContent());
        _helpContent.setHelpGroupId(dto.getHelpGroupId());
        _helpContent.setSort(dto.getSort());
        _helpContent.setIsShow(dto.getIsShow());
        mapper.insert(_helpContent);

        return _helpContent;
    }

    /**
     * 功能描述：更新[问题内容]
     *
     * @param id
     * @param dto
     */
    public HelpContent updateItem(String id, HelpContentItemDto dto) {
        Optional<HelpContent> optional = findById(id);
        HelpContent _helpContent = optional.get();
        _helpContent.setName(dto.getName());
        _helpContent.setContent(dto.getContent());
        _helpContent.setHelpGroupId(dto.getHelpGroupId());
        _helpContent.setSort(dto.getSort());
        _helpContent.setIsShow(dto.getIsShow());
        mapper.updateById(_helpContent);

        return _helpContent;
    }

    /**
     * 功能描述：替换[问题内容]显示状态
     *
     * @param id
     */
    public void handleIsShow(String id) {
        Optional<HelpContent> optional = findById(id);
        if (optional.isPresent()) {
            HelpContent _helpContent = optional.get();
            _helpContent.setIsShow(!_helpContent.getIsShow());
            mapper.updateById(_helpContent);
        }
    }

    /**
     * 功能描述：替换[问题内容]删除状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        Optional<HelpContent> optional = findById(id);
        if (optional.isPresent()) {
            HelpContent _helpContent = optional.get();
            _helpContent.setIsDel(!_helpContent.getIsDel());
            mapper.updateById(_helpContent);
        }
    }

    /**
     * 精确查寻H5的分类下的内容列表
     * @param parentId
     * @return
     */
    public List<HelpContent> getListByH5(String parentId) {
        return mapper.selectList(new QueryWrapper<HelpContent>().lambda().eq(HelpContent::getHelpGroupId, parentId).eq(HelpContent::getIsShow, true).eq(HelpContent::getIsDel, false));
    }
}
