package com.studio.nx.notice.service;

import com.studio.nx.framework.error.exception.NxFrameworkException;
import com.studio.nx.notice.config.EmailYml;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.util.Arrays;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Service
public class EmailService {

    @Autowired
    private TemplateEngine templateEngine;

    @Autowired
    private EmailYml emailYml;

    @Autowired
    private JavaMailSender mailSender;

    @SneakyThrows
    public void sendTextMil(String subject, String[] toList, String text) {
        vailEmail(toList);
        if (ObjectUtils.isEmpty(toList)) {
            return;
        }

        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom(emailYml.getUsername()); // 发件人邮箱
        message.setSubject(subject); // 邮件主题
        message.setTo(toList); // 收信人邮箱
        message.setText(text); // 邮件内容

        try {
            mailSender.send(message); //发送
        } catch (MailException e) {
            log.error("纯文本邮件发送失败->message:{}", e.getMessage());
            throw new NxFrameworkException("邮件发送失败");
        }
    }

    public void sendHtmlMil(String subject, String[] toList, String text, Boolean isShowHtml) {
        vailEmail(toList);
        if (ObjectUtils.isEmpty(toList)) {
            return;
        }

        try {
            MimeMessage message = mailSender.createMimeMessage();

            //是否发送的邮件是富文本（附件，图片，html等）
            MimeMessageHelper messageHelper = new MimeMessageHelper(message, true);
            messageHelper.setFrom(emailYml.getUsername());// 发件人邮箱
            messageHelper.setSubject(subject); // 邮件主题
            messageHelper.setTo(toList); // 收信人邮箱
            messageHelper.setText(text, isShowHtml); //false，显示原始html代码，无效果

            //判断是否有附加图片等
//            if (mailDO.getAttachment() != null && mailDO.getAttachment().size() > 0) {
//                mailDO.getAttachment().entrySet().stream().forEach(entrySet -> {
//                    try {
//                        File file = new File(String.valueOf(entrySet.getValue()));
//                        if (file.exists()) {
//                            messageHelper.addAttachment(entrySet.getKey(), new FileSystemResource(file));
//                        }
//                    } catch (MessagingException e) {
//                        log.error("附件发送失败->message:{}", e.getMessage());
//                        throw new NxFrameworkException("附件发送失败");
//                    }
//                });
//            }

            mailSender.send(message);
        } catch (MessagingException e) {
            log.error("富文本邮件发送失败->message:{}", e.getMessage());
            throw new NxFrameworkException("邮件发送失败");
        }
    }

    public void sendTemplateMail(String subject, String[] toList, Map<String, Object> variables, String template) {
        vailEmail(toList);
        if (ObjectUtils.isEmpty(toList)) {
            return;
        }

        try {
            MimeMessage message = mailSender.createMimeMessage();
            MimeMessageHelper messageHelper = new MimeMessageHelper(message, true);
            messageHelper.setFrom(emailYml.getUsername());// 发件人邮箱
            messageHelper.setSubject(subject); // 邮件主题
            messageHelper.setTo(toList); // 收信人邮箱

            // 使用模板thymeleaf
            Context context = new Context();
            context.setVariables(variables); // 定义模板数据
            String emailContent = templateEngine.process(template, context); // 指定模板路径
            messageHelper.setText(emailContent, true);

            mailSender.send(message);
        } catch (MessagingException e) {
            log.error("模板邮件发送失败->message:{}", e.getMessage());
            throw new NxFrameworkException("邮件发送失败");
        }
    }

    private void vailEmail(String[] emails) {
        Arrays.asList(emails).stream().forEach(email -> {
            if (StringUtils.hasText(email)) {
                Pattern pattern = Pattern.compile("([a-z0-9A-Z]+[-|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}");
                Matcher matcher = pattern.matcher(email);
                if (!matcher.matches()) {
                    throw new NxFrameworkException("邮箱格式错误");
                }
            }
        });
    }

}
