package com.studio.nx.oss.alioss.controller;

import com.aliyun.oss.model.ObjectMetadata;
import com.aliyuncs.sts.model.v20150401.AssumeRoleResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.oss.alioss.config.AliOssYaml;
import com.studio.nx.oss.alioss.service.AliOssService;
import com.studio.nx.oss.dto.FileResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/8/3 17:49
 */
@Api(tags = {"AliOss"})
@Slf4j
@RestController
@RequestMapping("/framework/oss/ali-oss")
public class AliOssController {

    @Autowired
    private AliOssYaml aliOssYaml;

    @Autowired
    private AliOssService aliOssService;

    /**
     * https://help.aliyun.com/document_detail/31920.html
     */
    @ApiOperation(value = "STS直传服务")
    @GetMapping("/sts")
    public ResponseEntity<BaseResponse<AssumeRoleResponse>> sts() {
        return ResponseEntity.ok(BaseResponse.ok(aliOssService.sts()));
    }

    @ApiOperation(
            value = "单个上传",
            notes = "文件上传至临时目录，待业务提交表单后。再业务进行拷贝文件、删除临时文件操作。"
    )
    @PostMapping("/{bucketName}/put-object")
    public ResponseEntity<BaseResponse<FileResponse>> putObject(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName, @RequestPart(name = "file", required = false) MultipartFile file, HttpServletRequest request) throws Exception {
        if (file == null || file.getSize() == 0) {
            throw new Exception("上传文件不能为空");
        }

        //创建上传Object的Metadata
        ObjectMetadata metadata = new ObjectMetadata();
        metadata.setContentLength(file.getInputStream().available());
        metadata.setCacheControl("no-cache");
        metadata.setHeader("Pragma", "no-cache");
        metadata.setContentEncoding("utf-8");
        metadata.setContentType(aliOssService.getContentType(file.getOriginalFilename()));
        metadata.setContentDisposition("filename/filesize=" + file.getOriginalFilename() + "/" + file.getSize() + "Byte.");

        aliOssService.putObject(bucketName, "", file, metadata);

        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "下载")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "分桶名称", required = true, paramType = "path", defaultValue = ""),
            @ApiImplicitParam(name = "filename", value = "文件", required = false, paramType = "query", defaultValue = "")
    })
    @GetMapping("/{bucketName}/download")
    public void download(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName, @RequestParam("filename") String filename, HttpServletRequest request, HttpServletResponse response) {
        aliOssService.download(bucketName, filename, response);
    }

    @ApiOperation(value = "删除文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "分桶名称", required = true, paramType = "path", defaultValue = ""),
            @ApiImplicitParam(name = "filename", value = "文件", required = false, paramType = "query", defaultValue = "")
    })
//    @DeleteMapping("/{bucketName}/delete-object")
    public ResponseEntity<BaseResponse<?>> deleteObject(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName, @RequestParam("filename") String filename) {
        aliOssService.deleteFile(bucketName, filename);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "清空分桶（注意分桶内不能有文件）")
//    @DeleteMapping("/{bucketName}/delete")
    public ResponseEntity<BaseResponse<?>> deleteBucket(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName) {
        aliOssService.deleteBucket(bucketName);
        return ResponseEntity.ok(BaseResponse.ok());
    }


}
