package com.studio.nx.oss.local.controller;

import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.local.service.LocalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

@Api(tags = {"本地"})
@Slf4j
@RestController
@RequestMapping("/service/oss/local")
public class LocalController {

    @Autowired
    private LocalService service;

    @ApiOperation(value = "上传图片")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "prefix", value = "文件存储路径", required = false, paramType = "query", defaultValue = "./tmp")
    )
    @PostMapping("/upload")
    public ResponseEntity<BaseResponse<FileResponse>> upload(@RequestParam("prefix") String prefix, @RequestPart("file") MultipartFile file) {
        if (null == file || file.getSize() <= 0) {
            return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "文件为空，请重试！"));
        }

        /* prefix上传目录不传，则上传至临时目录 */
        FileResponse dto = service.upload(file, prefix);

        return ResponseEntity.ok(dto == null ?
                BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "文件上传失败！") : BaseResponse.ok(dto));
    }

}
