package com.studio.nx.oss.minio.config;

import io.minio.MinioClient;
import lombok.Data;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 功能描述：
 * http://docs.minio.org.cn/docs/master/java-client-quickstart-guide
 *
 * @Author 梁展鹏
 * @Date 2021/6/9 14:58
 */
@Data
@Configuration
@ConfigurationProperties("nx-framework.nx-oss.minio")
public class MinioYaml {

    @Value("${endpoint:https://play.min.io}")
    private String endpoint;

    @Value("${port:9000}")
    private Integer port;

    @Value("${bucket-name:tmp}")
    private String bucketName;

    /* 如果是true，则用的是https而不是http,默认值是true */
    @Value("${secure:true}")
    private Boolean secure;

    /* Access key就像用户ID，可以唯一标识你的账户 */
    @Value("${access-key:Q3AM3UQ867SPQQA43P2F}")
    private String accessKey;

    /* Secret key是你账户的密码 */
    @Value("${secret-key:zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG}")
    private String secretKey;

    @Value("${spring.profiles.active}")
    private String active;

    @Bean
    public MinioClient localMinioClient() {
        String endpointTmp = endpoint;
        if ("prod".equals(active) || "test".equals(active)) {
            endpointTmp = "http://127.0.0.1";
        }

        /* 使用MinIO服务的URL，端口，Access key和Secret key创建一个MinioClient对象 */
        return MinioClient.builder()
                .endpoint(endpointTmp, this.port, this.secure)
                .credentials(this.accessKey, this.secretKey)
                .build();
    }

    @Bean
    public MinioClient minioClient() {
        /* 使用MinIO服务的URL，端口，Access key和Secret key创建一个MinioClient对象 */
        return MinioClient.builder()
                .endpoint(this.endpoint, this.port, this.secure)
                .credentials(this.accessKey, this.secretKey)
                .build();
    }

}
