package com.studio.nx.oss.minio.controller;

import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.minio.config.MinioYaml;
import com.studio.nx.oss.minio.service.MinioService;
import io.minio.GetObjectResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.io.IOException;
import java.util.HashMap;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/9 14:36
 */
@Api(tags = {"Minio"})
@Slf4j
@RestController
@RequestMapping("/framework/oss/minio")
public class MinioController {

    @Autowired
    private MinioYaml minioYaml;

    @Autowired
    private MinioService service;

    @ApiOperation(
            value = "单个上传",
            notes = "文件上传至临时目录，待业务提交表单后。再业务进行拷贝文件、删除临时文件操作。"
    )
    @PostMapping("/upload")
    public ResponseEntity<BaseResponse<FileResponse>> upload(@RequestPart(name = "file", required = false) MultipartFile file, HttpServletRequest request) throws Exception {
        if (file == null || file.getSize() == 0) {
            throw new Exception("上传文件不能为空");
        }

        FileResponse _response = service.putObject(minioYaml.getBucketName(), "", file, new HashMap<>(), new HashMap<>());

        return ResponseEntity.ok(BaseResponse.ok(_response));
    }

    @ApiOperation(value = "下载")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "分桶名称", required = true, paramType = "path", defaultValue = ""),
            @ApiImplicitParam(name = "filename", value = "文件", required = false, paramType = "query", defaultValue = "")
    })
//    @GetMapping("/{bucketName}/download")
    public void download(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName, @RequestParam("filename") String filename, HttpServletRequest request, HttpServletResponse response) {

        GetObjectResponse _response = service.download(bucketName, filename);

        response.setHeader("Content-Disposition", "attachment;filename=" + filename.substring(filename.lastIndexOf("/") + 1, filename.length()));
        response.setContentType("application/force-download");
        response.setCharacterEncoding("UTF-8");
        try {
            IOUtils.copy(_response, response.getOutputStream());
        } catch (IOException e) {
            log.error("[Minio下载异常] 异常：[{}]， [{}]", e.getMessage(), e);
        }
    }

    @ApiOperation(value = "删除文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "分桶名称", required = true, paramType = "path", defaultValue = ""),
            @ApiImplicitParam(name = "filename", value = "文件", required = false, paramType = "query", defaultValue = "")
    })
//    @DeleteMapping("/{bucketName}/remove")
    public ResponseEntity<BaseResponse<?>> remove(@Valid @NotBlank(message = "分桶名称为必填") @PathVariable("bucketName") String bucketName, @RequestParam("filename") String filename) {

        service.removeObject(bucketName, filename);

        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "清空默认分桶")
//    @DeleteMapping("/remove")
    public ResponseEntity<BaseResponse<?>> removeBucket() {
        service.removeBucket();
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
