package com.studio.nx.oss.qiniu.controller;

import com.alibaba.fastjson.JSONObject;
import com.qiniu.http.Response;
import com.qiniu.storage.Configuration;
import com.qiniu.storage.Region;
import com.qiniu.storage.UploadManager;
import com.qiniu.storage.model.DefaultPutRet;
import com.qiniu.util.Auth;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.oss.qiniu.config.QiniuYaml;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.UUID;

/**
 * 官方SDK文档
 * <p>
 * https://developer.qiniu.com/kodo/sdk/1239/java
 * <p>
 * 密钥配置
 * https://portal.qiniu.com/user/key
 */
//@Api(tags = {"七牛云"})
@Slf4j
@RestController
@RequestMapping("/service/oss/qiniu")
public class QiniuCloudController {

    private final String GROUP = "public";

    @Autowired
    private QiniuYaml qiniuYaml;

    private String upToken;

    private UploadManager uploadManager;

    // todo add by 梁展鹏 20200530 暂时不初始化七牛云接口
//    @PostConstruct
    private void init() {
        uploadManager = new UploadManager(new Configuration(Region.huanan()));
        Auth auth = Auth.create(qiniuYaml.getAccessKey(), qiniuYaml.getSecretKey());
        upToken = auth.uploadToken(qiniuYaml.getBucket());
    }

    /**
     * 上传文件
     *
     * @param group
     * @param file
     * @return
     */
    @PostMapping("/{group}/upload")
    public BaseResponse upload(@PathVariable("group") String group, @RequestParam("file") MultipartFile file) {
        if (StringUtils.isEmpty(group)) {
            group = GROUP;
        }
        if (file == null || file.getSize() <= 0) {
            return BaseResponse.failure(500, "未找到上传文件，请重试！");
        }

        try {
            String key = filename(group, file.getOriginalFilename());
            Response response = uploadManager.put(file.getBytes(), key, upToken);
            if (response.isOK()) {
                //解析上传成功的结果
                DefaultPutRet putRet = JSONObject.parseObject(response.bodyString(), DefaultPutRet.class);
                return BaseResponse.ok(key);
            } else {
                return BaseResponse.failure(500, "上传文件失败，请重试或联系管理员！");
            }
        } catch (IOException e) {
            return BaseResponse.failure(500, "上传文件异常，请联系管理员！");
        }
    }

    private String filename(String group, String originalFilename) {
        String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf("."), originalFilename.length());
        String newFileName = UUID.randomUUID().toString().replaceAll("-", "") + fileSuffix;
        return group + "/" + newFileName;
    }

}
