package com.studio.nx.oss.ueditor.controller;

import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.oss.ueditor.UEditorConfig;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Api(tags = {"百度富文本编译器"})
@Slf4j
@RestController
@RequestMapping("/service/oss/ueditor")
public class UEditorController {

    @ApiOperation(value = "获取上传配置")
    @IgnoreAuth
    @GetMapping(value = "/ueditor-action")
    public Map<String, Object> actionControl(@RequestParam("action") String action, HttpServletRequest request) {
        if ("config".equals(action)) {
            return this.getConfig(request);
        }

        return Collections.EMPTY_MAP;
    }

    @ApiOperation(value = "单图片上传")
    @IgnoreAuth
    @PostMapping(value = "ueditor-action")
    public Map<String, Object> actionControl(@RequestParam("action") String action,
                                             @RequestParam(value = "callback", required = false) String callback,
                                             @RequestParam(value = "upfile", required = false) MultipartFile upfile,
                                             HttpServletRequest request,
                                             @RequestParam(value = "start", required = false) Integer start,
                                             @RequestParam(value = "size", required = false) Integer size) {
        if ("uploadimage".equals(action)) {
            if (upfile != null)
                return this.upload_image(upfile);
        }
        return Collections.EMPTY_MAP;
    }

    private Map<String, Object> getConfig(HttpServletRequest request) {
//        String rootPath = request.getSession().getServletContext().getRealPath("/");
//        String contextPath = request.getContextPath();
//        String uri = request.getRequestURI();
//        ConfigManager configManager = ConfigManager.getInstance(rootPath, contextPath, uri);

        /* JSONObject to Map */
//        Map<String, Object> map = configManager.getAllConfig().toMap();

        /* add by 梁展鹏 20210622 直接获取底层配置 */
        Map<String, Object> map = UEditorConfig.setting;
        map.put("state", "SUCCESS");
        return map;
    }

    private String getExtensionType(String originalFilename) {
        if (StringUtils.isEmpty(originalFilename)) {
            return "";
        }

        Pattern pattern = Pattern.compile("\\.([^\\.]*?)$");
        Matcher matcher = pattern.matcher(originalFilename);
        if (matcher.find()) {
            return matcher.group(1).toLowerCase();
        }

        return "";
    }

    private Map<String, Object> upload_image(MultipartFile upfile) {
        String fileType = this.getExtensionType(upfile.getOriginalFilename());
        if ("".equals(fileType)) {
            return Collections.EMPTY_MAP;
        }
        String base64Str = "data:" + fileType + ";base64,";
        String url = "";
        try {
            url = Base64.encodeBase64String(upfile.getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        }

        Map<String, Object> result = new HashMap();
        result.put("state", "SUCCESS");
        result.put("url", base64Str + url);
        result.put("title", upfile.getOriginalFilename());
        result.put("original", upfile.getOriginalFilename());
        return result;
    }
}
