package com.studio.nx.oss.util;

import lombok.extern.slf4j.Slf4j;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/5/28 14:58
 */
@Slf4j
public class WatermarkUtil {

//    public static void main(String[] args) {
//        System.out.println("开始水印：");
//        WatermarkUtil.addStringWatermark("/Users/liangzp/Downloads/yxl/2.scale.jpg", "/Users/liangzp/Downloads/yxl/2.scale.jpg", "jpg", "幼享乐", 80, Color.gray, 0.8F);
//        System.out.println("水印完成。");
//    }

    /**
     * 功能描述：增加文字水印（-45度）斜体
     *
     * @param originImgPath     源图片路径
     * @param targetImgPath     保存的图片路径
     * @param fileExt           图片格式
     * @param watermarkContent  水印文本内容
     * @param watermarkFontSize 水印文本字体大小
     * @param watermarkColor    水印文本颜色
     * @param watermarkAlpha    水印文本透明度（默认为1.0  值越小颜色越浅）
     * @return void
     * @description
     */
    public static void addStringWatermark(String originImgPath, String targetImgPath, String fileExt, String watermarkContent, Integer watermarkFontSize, Color watermarkColor, Float watermarkAlpha) {
        Font font = new Font("宋体", Font.BOLD, watermarkFontSize); // 水印字体，大小
        OutputStream outImgStream = null;

        try {
            Image srcImg = ImageIO.read(new File(originImgPath)); // 读取文件并转为图片
            int srcImgWidth = srcImg.getWidth(null); // 获取图片的宽
            int srcImgHeight = srcImg.getHeight(null); // 获取图片的高

            // 加水印
            BufferedImage bufImg = new BufferedImage(srcImgWidth, srcImgHeight, BufferedImage.TYPE_INT_RGB);
            Graphics2D g = bufImg.createGraphics(); // 得到画笔
            g.drawImage(srcImg, 0, 0, srcImgWidth, srcImgHeight, null);
            g.setColor(watermarkColor); // 设置水印颜色
            g.setFont(font); // 设置字体
            g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, watermarkAlpha)); // 设置水印文字透明度
            if (bufImg.getWidth() > bufImg.getHeight()) { // 横向
                g.rotate(Math.toRadians(-45), bufImg.getWidth(), bufImg.getHeight() + 100); // 设置水印文字的旋转角度
            } else { // 纵向
                g.rotate(Math.toRadians(-45), bufImg.getHeight() + 100, bufImg.getWidth() + 50); // 设置水印文字的旋转角度
            }

            JLabel label = new JLabel(watermarkContent);
            FontMetrics metrics = label.getFontMetrics(font);
            int width = metrics.stringWidth(label.getText()); // 文字水印的宽
            int rowsNumber = (srcImgHeight / width) + 4; // 图片的高  除以  文字水印的宽 ——> 打印的行数(以文字水印的宽为间隔)
            int colsNumber = (srcImgWidth / width); //图片的宽 除以 文字水印的宽 ——> 每行打印的列数(以文字水印的宽为间隔)
            // 防止图片太小而文字水印太长，所以至少打印一次
            if (rowsNumber < 1) {
                rowsNumber = 1;
            }
            if (colsNumber < 1) {
                colsNumber = 1;
            }
            for (int j = 0; j < rowsNumber; j++) {
                for (int i = 0; i < colsNumber; i++) {
                    g.drawString(watermarkContent, i * width + j * width, -i * width + j * width); // 画出水印,并设置水印位置
                }
            }
            g.dispose(); // 释放资源

            // 输出图片
            outImgStream = new FileOutputStream(targetImgPath);
            ImageIO.write(bufImg, fileExt, outImgStream);
        } catch (Exception e) {
            e.printStackTrace();
            e.getMessage();
        } finally {
            try {
                if (outImgStream != null) {
                    outImgStream.flush();
                    outImgStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
                e.getMessage();
            }
        }
    }

}
