package com.studio.nx.pay.weixin.dto;

import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * 功能描述：
 * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_5.shtml
 *
 * @Author 梁展鹏
 * @Date 2021/7/1 00:30
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class PaySuccessNotifyResponse {

    private String appid; // 应用ID
    private String mchid; // 商户号
    private String out_trade_no; // 商户订单号
    private String transaction_id; // 微信支付订单号
    private String trade_type; // 交易类型
    private String trade_state; // 交易状态
    private String trade_state_desc; // 交易状态描述
    private String bank_type; // 付款银行
    private String attach; // 附加数据
    // ZonedDateTime zonedDateTime = ZonedDateTime.parse(payResponse.getSuccess_time());
    private String success_time; // 支付完成时间

    private Payer payer;
    private Amount amount;
    private SceneInfo scene_info;
    private List<PromotionDetail> promotion_detail;

    @ApiModel
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class Payer {
        private String openid; // 用户标识
    }

    @ApiModel
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class Amount {
        private Integer total; // 总金额
        private Integer payer_total; // 用户支付金额
        private String currency; // 货币类型
        private String payer_currency; // 用户支付币种
    }

    @ApiModel
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class SceneInfo {
        private String device_id; // 商户端设备号
    }

    @ApiModel
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class PromotionDetail {
        private String coupon_id; // 券ID
        private String name; // 优惠名称
        private String scope; // 优惠范围
        private String type; // 优惠类型
        private Integer amount; // 优惠券面额
        private String stock_id; // 活动ID
        private Integer wechatpay_contribute; // 微信出资
        private Integer merchant_contribute; // 商户出资
        private Integer other_contribute; // 其他出资
        private String currency; // 优惠币种
        private List<GoodsDetail> goods_details; // 单品列表
    }

    @ApiModel
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class GoodsDetail {
        private String goods_id; // 商品编码
        private Integer quantity; // 商品数量
        private Integer unit_price; // 商品单价
        private Integer discount_amount; // 商品优惠金额
        private String goods_remark; // 商品备注
    }

}
