package com.studio.nx.pay.weixin.dto;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

/**
 * 功能描述：
 * 官方文档
 * https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=7_7&index=5
 * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_4.shtml#menu1
 *
 * @Author 梁展鹏
 * @Date 2021/6/29 10:55
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class WeixinPrePayResponse {

    /**
     * 预支付交易会话标识。用于后续接口调用中使用，该值有效期为2小时
     * 示例值：wx201410272009395522657a690389285100
     */
    @ApiModelProperty("预支付交易会话ID")
    private String prepayid;

    @ApiModelProperty("商户号")
    private String partnerid;

    /**
     * 微信开放平台审核通过的移动应用appid 。
     * 示例值：wx8888888888888888
     */
    @ApiModelProperty("应用id")
    private String appid;

    /**
     * 时间戳
     * 时间戳从1970年1月1日00:00:00至今的秒数,即当前的时间
     * 示例值：1412000000
     */
    @ApiModelProperty("时间戳")
    private String timestamp;

    /**
     * 随机字符串，不长于32位。推荐随机数生成算法。
     * 示例值： 5K8264ILTKCH16CQ2502SI8ZNMTM67VS
     */
    @ApiModelProperty("随机字符串")
    private String noncestr;

    /**
     * 签名，使用字段appId、timeStamp、nonceStr、prepayid计算得出的签名值
     * 注意：取值RSA格式
     */
    @ApiModelProperty("签名")
    private String sign;

}
