package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.yxl.controller.admin.search.ActivitySearchDto;
import com.studio.nx.yxl.dto.ActivityDto;
import com.studio.nx.yxl.entity.Activity;
import com.studio.nx.yxl.service.ActivityService;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * @author Loki
 * @date 2021/6/24 15:24
 */
@Api(tags = "活动")
@Slf4j
@Validated
@RestController("AdminActivityController")
@RequestMapping("/admin/activity")
public class ActivityController {

    @Autowired
    private ActivityService service;


    /**
     * 功能描述：查询[活动]详情
     *
     * @param id 标识
     * @return
     */
    @ApiOperation(value = "查询[活动]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findById(id)));
    }

    /**
     * 功能描述：查询[活动]列表(分页)
     *
     * @return
     */
    @ApiOperation(value = "查询[活动]列表(分页)")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<?>>> page(@Valid @RequestBody BasePageRequest<ActivitySearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.page(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：查询[活动]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[活动]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<List<?>>> list(@Valid @RequestBody BasePageRequest<ActivitySearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.list(request.getData())));
    }

    /**
     * 功能描述：新增[活动]
     *
     * @param request
     * @return
     */
    @OperationLog("新增[活动]")
    @ApiOperation(value = "新增[活动]")
    @PostMapping("/save")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<ActivityDto> request) {
        service.createItem(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }
    /**
     * 功能描述：更新[活动]
     *
     * @param request
     * @return
     */
    @OperationLog("更新[活动]")
    @ApiOperation(value = "更新[活动]")
    @PutMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> save(@NotBlank @PathVariable("id") String id, @Valid @RequestBody BaseRequest<ActivityDto> request) {
        service.updateItem(id, request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[活动]显示/隐藏状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[活动]显示/隐藏状态")
    @ApiOperation(value = "替换[活动]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> show(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[活动]删除状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[活动]删除状态")
    @ApiOperation(value = "替换[活动]删除状态")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
