package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.yxl.controller.admin.request.BannerSaveRequest;
import com.studio.nx.yxl.controller.admin.search.BannerSearchDto;
import com.studio.nx.yxl.entity.Banner;
import com.studio.nx.yxl.service.BannerService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/5/27 13:56
 */
@Api(tags = "横幅")
@Slf4j
@Validated
@RestController("AdminBannerController")
@RequestMapping("/admin/banner")
public class BannerController {

    @Autowired
    private BannerService service;

    /**
     * 功能描述：查询[横幅]详情
     *
     * @param id 标识
     * @return
     */
    @ApiOperation(value = "查询[横幅]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findById(id)));
    }

    /**
     * 功能描述：查询[横幅]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[横幅]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<Banner>>> page(@Valid @RequestBody BasePageRequest<BannerSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.page(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：保存[横幅]
     *
     * @param request
     * @return
     */
    @OperationLog("保存[横幅]")
    @ApiOperation(value = "保存[横幅]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<BannerSaveRequest> request) {
        service.save(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：更新[横幅]
     *
     * @param request
     * @return
     */
    @OperationLog("更新[横幅]")
    @ApiOperation(value = "更新[横幅]")
    @PutMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> update(@NotBlank @PathVariable("id") String id,@Valid @RequestBody BaseRequest<BannerSaveRequest> request) {
        service.update(id,request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[横幅]显示/隐藏状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[横幅]显示/隐藏状态")
    @ApiOperation(value = "替换[横幅]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> show(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[横幅]删除状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[横幅]删除状态")
    @ApiOperation(value = "替换[横幅]删除状态")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
