package com.studio.nx.yxl.controller.admin;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.uiac.dto.RoleDto;
import com.studio.nx.uiac.service.AccountRoleService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.admin.response.DownloadCountResponse;
import com.studio.nx.yxl.controller.admin.response.DownloadPageResponse;
import com.studio.nx.yxl.controller.admin.response.UploadCountResponse;
import com.studio.nx.yxl.controller.admin.response.UploadPageResponse;
import com.studio.nx.yxl.controller.admin.search.DownloadPageSearch;
import com.studio.nx.yxl.mapper.DataStatisticsMapper;
import com.studio.nx.yxl.mapper.OrderMapper;
import com.studio.nx.yxl.service.PhotoAccountRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Api(tags = "数据统计")
@Slf4j
@Validated
@RestController("AdminDataStatisticsController")
@RequestMapping("/admin/data/statistics")
public class DataStatisticsController {

    @Autowired
    private AccountRoleService accountRoleService;

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private PhotoAccountRecordService photoAccountRecordService;

    @Autowired
    private DataStatisticsMapper mapper;

    @Autowired
    private OrderMapper orderMapper;

    @ApiOperation(value = "下载照片")
    @PostMapping("/download/page")
    public ResponseEntity<BaseResponse<BasePageResponse<DownloadPageResponse>>> downloadPage(@Valid @RequestBody BasePageRequest<DownloadPageSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        DownloadPageSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        IPage<DownloadPageResponse> _page = mapper.downloadPage(search, new Page(request.getPage(), request.getLimit()));
        for (DownloadPageResponse response : _page.getRecords()) {
            response.setListPreview(localService.fullPath(response.getListPreview()));
        }

        return ResponseEntity.ok(BaseResponse.ok(new BasePageResponse().setTotal(_page.getTotal()).setData(_page.getRecords())));
    }

    @ApiOperation(value = "下载照片统计")
    @PostMapping("/download/count")
    public ResponseEntity<BaseResponse<DownloadCountResponse>> countDownloadPage(@Valid @RequestBody BasePageRequest<DownloadPageSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        DownloadPageSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        DownloadCountResponse response = new DownloadCountResponse();

        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(photoAccountRecordService.countByIsHdLogoPhoto(search) + photoAccountRecordService.countByIsOriginPhoto(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(photoAccountRecordService.countByIsHdLogoPhoto(search) + photoAccountRecordService.countByIsOriginPhoto(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(photoAccountRecordService.countByIsHdLogoPhoto(search) + photoAccountRecordService.countByIsOriginPhoto(search));

        response.setSumPaidPrice(orderMapper.sumByCash());
        response.setSumPoints(orderMapper.sumByPoint().multiply(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_CASH_OUT))).intValue());

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "上传照片")
    @PostMapping("/upload/page")
    public ResponseEntity<BaseResponse<BasePageResponse<UploadPageResponse>>> uploadPage(@Valid @RequestBody BasePageRequest<DownloadPageSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        DownloadPageSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        IPage<UploadPageResponse> _page = mapper.uploadPage(search, new Page(request.getPage(), request.getLimit()));

        return ResponseEntity.ok(BaseResponse.ok(new BasePageResponse().setTotal(_page.getTotal()).setData(_page.getRecords())));
    }

    @ApiOperation(value = "上传照片统计")
    @PostMapping("/upload/count")
    public ResponseEntity<BaseResponse<UploadCountResponse>> uploadcount(@Valid @RequestBody BasePageRequest<DownloadPageSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        DownloadPageSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        UploadCountResponse response = new UploadCountResponse();
        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(mapper.uploadCount(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(mapper.uploadCount(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(mapper.uploadCount(search));

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

}
