package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.yxl.controller.admin.response.IntegralCashRecordAdminPageResponse;
import com.studio.nx.yxl.controller.admin.search.IntegralCashRecordSearch;
import com.studio.nx.yxl.entity.IntegralCashRecord;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Api(tags = "积分记录")
@Slf4j
@Validated
@RestController("AdminIntegralRecordController")
@RequestMapping("/admin/record")
public class IntegralCashRecordController {

    @Autowired
    private IntegralCashRecordService service;

    /**
     * 功能描述：查询[积分记录]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[积分记录]列表")
    @PostMapping("/integral/page")
    public ResponseEntity<BaseResponse<BasePageResponse<IntegralCashRecordAdminPageResponse>>> integralPage(@Valid @RequestBody BasePageRequest<IntegralCashRecordSearch> request) {
        IntegralCashRecordSearch search = request.getData();
        search.setType(IntegralCashRecord.Type.INTEGRAL);
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：查询[现金记录]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[现金记录]列表")
    @PostMapping("/cash/page")
    public ResponseEntity<BaseResponse<BasePageResponse<IntegralCashRecordAdminPageResponse>>> cashPage(@Valid @RequestBody BasePageRequest<IntegralCashRecordSearch> request) {
        IntegralCashRecordSearch search = request.getData();
        search.setType(IntegralCashRecord.Type.CASH);
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(search, request.getPage(), request.getLimit())));
    }

}
