package com.studio.nx.yxl.controller.admin;

import cn.hutool.core.date.DateUtil;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.admin.request.KindergartenItemRequestDto;
import com.studio.nx.yxl.controller.admin.request.VerifyRequest;
import com.studio.nx.yxl.controller.admin.response.KindergartenCountResponse;
import com.studio.nx.yxl.controller.admin.response.KindergartenListResponse;
import com.studio.nx.yxl.controller.admin.search.KindergartenSearch;
import com.studio.nx.yxl.controller.app.request.my.apply.KindergartenApplySaveRequest;
import com.studio.nx.yxl.entity.KindergartenInfo;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import com.studio.nx.yxl.service.KindergartenApplyService;
import com.studio.nx.yxl.service.KindergartenInfoService;
import com.studio.nx.yxl.service.KindergartenService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Api(tags = "幼儿园")
@Slf4j
@Validated
@RestController("AdminKindergartenController")
@RequestMapping("/admin/kindergarten")
public class KindergartenController {

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private KindergartenService service;

    @Autowired
    private KindergartenInfoService kindergartenInfoService;

    @Autowired
    private KindergartenApplyService kindergartenApplyService;

    /**
     * 功能描述：查询[幼儿园]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[幼儿园]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<KindergartenListResponse>>> page(@Valid @RequestBody BasePageRequest<KindergartenSearch> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "查询[幼儿园]统计")
    @PostMapping("/count")
    public ResponseEntity<BaseResponse<?>> count(@Valid @RequestBody BasePageRequest<KindergartenSearch> request) {
        KindergartenSearch search = request.getData();

        KindergartenCountResponse response = new KindergartenCountResponse();

        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(kindergartenMapper.adminCount(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(kindergartenMapper.adminCount(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(kindergartenMapper.adminCount(search));

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    /**
     * 功能描述：查询[幼儿园]列表
     * todo add by 梁展鹏 20210901 让直接使用adminPage接口
     * @return
     */
    @Deprecated
    @ApiOperation(value = "查询[幼儿园]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<BasePageResponse<KindergartenListResponse>>> list(@Valid @RequestBody BasePageRequest<KindergartenSearch> request) {
        KindergartenSearch search = request.getData();
        search.setAuditStatus(KindergartenInfo.AuditStatus.PASS);
        Integer page = 1;
        Integer size = 99999;
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(search, page, size)));
    }

    /**
     * 功能描述：查询[幼儿园]详情
     *
     * @param id 标识
     * @return
     */
    @ApiOperation(value = "查询[幼儿园]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findByInfo(id)));
    }

    @OperationLog("新增[幼儿园]")
    @ApiOperation(value = "新增[幼儿园]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<KindergartenApplySaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.adminKindergartenSave(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("更新[幼儿园]")
    @ApiOperation(value = "更新[幼儿园]")
    @PutMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> update(@Valid @ApiParam("标识") @PathVariable String id, @Valid @RequestBody BaseRequest<KindergartenItemRequestDto> request) {
        service.update(id, request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换[幼儿园]显示/隐藏状态")
    @ApiOperation(value = "替换[幼儿园]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> show(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("发送[幼儿园]签到二维码邮件")
    @ApiOperation(value = "发送[幼儿园]签到二维码邮件")
    @PutMapping("/{id:\\d+}/send-qrcode")
    public ResponseEntity<BaseResponse<?>> sendQRCode(@Valid @ApiParam("标识") @PathVariable String id) {
        kindergartenApplyService.sendKindergartenQrCodeEmailBykindergartenId(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换[幼儿园]删除状态")
    @ApiOperation(value = "替换[幼儿园]删除状态")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换[幼儿园]审核状态")
    @ApiOperation(value = "替换[幼儿园]审核状态")
    @PutMapping("/{id:\\d+}/audit")
    public ResponseEntity<BaseResponse<?>> audit(@Valid @ApiParam("标识") @PathVariable String id, @Valid @RequestBody BaseRequest<VerifyRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        kindergartenApplyService.handleIsAudit(id, request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
