package com.studio.nx.yxl.controller.admin;

import cn.hutool.core.date.DateUtil;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.alioss.service.AliOssService;
import com.studio.nx.uiac.dto.RoleDto;
import com.studio.nx.uiac.service.AccountRoleService;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseAuditMultiRequest;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseAuditRequest;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseSaveRequest;
import com.studio.nx.yxl.controller.admin.response.PhotoWarehouseCountResponse;
import com.studio.nx.yxl.controller.admin.response.PhotoWarehouseResponse;
import com.studio.nx.yxl.controller.admin.search.PhotoWarehouseSearch;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import com.studio.nx.yxl.service.PhotoWarehouseProcessingService;
import com.studio.nx.yxl.service.PhotoWarehouseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/11 17:02
 */
@Api(tags = "照片库")
@Slf4j
@Validated
@RestController("AdminPhotoWarehouseController")
@RequestMapping("/admin/photo-warehouse")
public class PhotoWarehouseController {

    @Autowired
    private AccountRoleService accountRoleService;

    @Autowired
    private AliOssService aliOssService;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private PhotoWarehouseService service;

    @Autowired
    private PhotoWarehouseProcessingService processingService;

    @ApiOperation(value = "保存[照片库]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<PhotoWarehouseSaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.adminPhotoerSave(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "审核[照片库]")
    @PostMapping("/audit")
    public ResponseEntity<BaseResponse<?>> audit(@Valid @RequestBody BaseRequest<PhotoWarehouseAuditRequest> request) {
        service.audit(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "批量审核[照片库]")
    @PostMapping("/audit/multi")
    public ResponseEntity<BaseResponse<?>> auditMulti(@Valid @RequestBody BaseRequest<PhotoWarehouseAuditMultiRequest> request) {
        service.auditMulti(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * curl -X GET "https://youxiangle.cn/yxl-back-end/admin/photo-warehouse/processing" -H "accept: *//*" -H "token: "
     * // 需要替换为为 /
     * @return
     */
    @Deprecated
    @ApiOperation(value = "触发裂变[照片库]")
    @GetMapping("/processing")
    public ResponseEntity<BaseResponse<?>> processing() {
        processingService.start();
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "[照片库]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoWarehouseResponse>>> page(@Valid @RequestBody BasePageRequest<PhotoWarehouseSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        PhotoWarehouseSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        return ResponseEntity.ok(BaseResponse.ok(service.page(search, request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "[照片库]统计")
    @PostMapping("/count")
    public ResponseEntity<BaseResponse<?>> count(@Valid @RequestBody BasePageRequest<PhotoWarehouseSearch> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        PhotoWarehouseSearch search = request.getData();

        /* 如果是认证摄影师，增加数据权限 */
        List<RoleDto> roleDtoList = accountRoleService.listByAccountId(accountId);
        if (roleDtoList.stream().filter(o -> "photoer".equalsIgnoreCase(o.getCode())).collect(Collectors.toList()).size() > 0) {
            search.setAccountId(accountId);
        }

        PhotoWarehouseCountResponse response = new PhotoWarehouseCountResponse();

        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(photoWarehouseMapper.adminCount(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(photoWarehouseMapper.adminCount(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(photoWarehouseMapper.adminCount(search));

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "查询[照片库]详情")
    @GetMapping("/{photoWarehouseId:\\d+}")
    public ResponseEntity<BaseResponse<PhotoWarehouseResponse>> getItem(@NotBlank @PathVariable("photoWarehouseId") String photoWarehouseId) {
        return ResponseEntity.ok(BaseResponse.ok(service.findAdminById(photoWarehouseId)));
    }

    /**
     * 后台专用于图片更新价格等信息的
     *
     * @param id
     * @param request
     * @return
     */
    @ApiOperation(value = "更新[照片库]")
    @PutMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> updateItem(@NotBlank @PathVariable("id") String id, @Valid @RequestBody BaseRequest<PhotoWarehouseResponse> request) {
        service.updateItem(id, request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "删除[照片库]")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> handlerIsDel(@NotBlank @PathVariable("id") String id) {
        service.handlerIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
