package com.studio.nx.yxl.controller.admin;

import cn.hutool.core.date.DateUtil;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.admin.request.VerifyRequest;
import com.studio.nx.yxl.controller.admin.response.PhotoerAuditAdminPageResponse;
import com.studio.nx.yxl.controller.admin.response.PhotoerCountResponse;
import com.studio.nx.yxl.controller.admin.search.PhotoerSearch;
import com.studio.nx.yxl.mapper.PhotoerAuditMapper;
import com.studio.nx.yxl.service.PhotoerAuditService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;

/**
 * 功能描述：
 *
 * @Author loki
 * @Date 2021/6/18 13:56
 */
@Api(tags = "摄影师")
@Slf4j
@Validated
@RestController("AdminPhotoerController")
@RequestMapping("/admin/photoer")
public class PhotoerController {

    @Autowired
    private PhotoerAuditMapper photoerAuditMapper;

    @Autowired
    private PhotoerAuditService service;

    /**
     * 功能描述：查询[摄影师]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[摄影师]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoerAuditAdminPageResponse>>> page(@Valid @RequestBody BasePageRequest<PhotoerSearch> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.page(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "查询[摄影师]统计")
    @PostMapping("/count")
    public ResponseEntity<BaseResponse<?>> count(@Valid @RequestBody BasePageRequest<PhotoerSearch> request) {
        PhotoerSearch search = request.getData();

        PhotoerCountResponse response = new PhotoerCountResponse();

        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(photoerAuditMapper.adminCount(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(photoerAuditMapper.adminCount(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(photoerAuditMapper.adminCount(search));

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    /**
     * 功能描述：查询[摄影师]详情
     *
     * @param id 标识
     * @return
     */
    @ApiOperation(value = "查询[摄影师]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findById(id)));
    }

    /**
     * 功能描述：替换[摄影师]审核状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[摄影师]审核状态")
    @ApiOperation(value = "替换[摄影师]审核状态")
    @PutMapping("/{id:\\d+}/audit")
    public ResponseEntity<BaseResponse<?>> audit(@Valid @ApiParam("标识") @PathVariable String id,@Valid @RequestBody BaseRequest<VerifyRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.handleIsAudit(id, request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
