package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.yxl.controller.admin.request.TopSearchKeywordsSaveRequest;
import com.studio.nx.yxl.controller.admin.search.TopSearchKeywordsSearchRequest;
import com.studio.nx.yxl.service.TopSearchKeywordsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/15 21:27
 */
@Api(tags = "热门搜索词")
@Slf4j
@Validated
@RestController("AdminTopSearchKeywordsController")
@RequestMapping("/admin/top-search-keywords")
public class TopSearchKeywordsController {

    @Autowired
    private TopSearchKeywordsService service;

    /**
     * 功能描述：查询[热门搜索词]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[热门搜索词]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<?>>> page(@Valid @RequestBody BasePageRequest<TopSearchKeywordsSearchRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：保存[热门搜索词]
     *
     * @param request
     * @return
     */
    @OperationLog("保存[热门搜索词]")
    @ApiOperation(value = "保存[热门搜索词]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<TopSearchKeywordsSaveRequest> request) {
        service.save(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：更新[热门搜索词]
     *
     * @param id
     * @param request
     * @return
     */
    @OperationLog("更新[热门搜索词]")
    @ApiOperation(value = "更新[热门搜索词]")
    @PostMapping("{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> update(@Valid @ApiParam("标识") @PathVariable String id, @Valid @RequestBody BaseRequest<TopSearchKeywordsSaveRequest> request) {
        service.update(id,request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[热门搜索词]显示/隐藏状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[热门搜索词]显示/隐藏状态")
    @ApiOperation(value = "替换[热门搜索词]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> show(@Valid @ApiParam("标识") @PathVariable String id) {
        service.isShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[热门搜索词]删除状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[热门搜索词]删除状态")
    @ApiOperation(value = "替换[热门搜索词]删除状态")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.isDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
