package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.yxl.controller.admin.search.TweetSearchDto;
import com.studio.nx.yxl.dto.TweetDto;
import com.studio.nx.yxl.service.TweetService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Api(tags = "资讯")
@Slf4j
@Validated
@RestController("AdminTweetController")
@RequestMapping("/admin/tweet")
public class TweetController {

    @Autowired
    private TweetService service;

    /**
     * 功能描述：查询[资讯]详情
     *
     * @param id 标识
     * @return
     */
    @ApiOperation(value = "查询[资讯]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findByIdInAdmin(id)));
    }

    /**
     * 功能描述：查询[资讯]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[资讯]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<?>>> page(@Valid @RequestBody BasePageRequest<TweetSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.page(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：保存[资讯]
     *
     * @param request
     * @return
     */
    @OperationLog("保存[资讯]")
    @ApiOperation(value = "保存[资讯]")
    @PostMapping("")
    public ResponseEntity<BaseResponse<?>> saveItem(@Valid @RequestBody BaseRequest<TweetDto> request) {
        service.saveItem(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：更新[资讯]
     *
     * @param id
     * @param request
     * @return
     */
    @OperationLog("更新[资讯]")
    @ApiOperation(value = "更新[资讯]")
    @PutMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> updateItem(@Valid @ApiParam("标识") @PathVariable String id, @Valid @RequestBody BaseRequest<TweetDto> request) {
        service.updateItem(id, request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[资讯]显示/隐藏状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[资讯]显示/隐藏状态")
    @ApiOperation(value = "替换[资讯]显示/隐藏状态")
    @PutMapping("/{id:\\d+}/show")
    public ResponseEntity<BaseResponse<?>> show(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsShow(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：替换[资讯]删除状态
     *
     * @param id
     * @return
     */
    @OperationLog("替换[资讯]删除状态")
    @ApiOperation(value = "替换[资讯]删除状态")
    @DeleteMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> del(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
