package com.studio.nx.yxl.controller.admin;

import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.admin.request.WithdrawalRecordVerifyRequest;
import com.studio.nx.yxl.controller.admin.response.WithdrawalRecordCountResponse;
import com.studio.nx.yxl.controller.admin.search.WithdrawalRecordSearch;
import com.studio.nx.yxl.dto.WithdrawalRecordDto;
import com.studio.nx.yxl.mapper.OrderMapper;
import com.studio.nx.yxl.mapper.WithdrawalRecordMapper;
import com.studio.nx.yxl.service.WithdrawalRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Api(tags = "提现记录")
@Slf4j
@Validated
@RestController("AdminWithdrawalRecordController")
@RequestMapping("/admin/withdrawal/record")
public class WithdrawalRecordController {

    @Autowired
    private WithdrawalRecordService service;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private WithdrawalRecordMapper withdrawalRecordMapper;

    @ApiOperation(value = "查询[提现记录]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{id:\\d+}")
    public ResponseEntity<BaseResponse<WithdrawalRecordDto>> findById(@NotBlank @PathVariable("id") String id) {
        return ResponseEntity.ok(BaseResponse.ok(service.findByInfo(id)));
    }

    @ApiOperation(value = "查询[提现记录]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<WithdrawalRecordDto>>> page(@Valid @RequestBody BasePageRequest<WithdrawalRecordSearch> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "查询[提现记录]统计")
    @PostMapping("/count")
    public ResponseEntity<BaseResponse<WithdrawalRecordCountResponse>> count(@Valid @RequestBody BasePageRequest<WithdrawalRecordSearch> request) {

        // todo 分账号权限
        BigDecimal sumPaidPrice = orderMapper.sumPaidPrice(); // 收入金额
        BigDecimal withdrawalAmount = withdrawalRecordMapper.sumBySuccess(request.getData()); // 成功提现金额
        BigDecimal undrawnAmount = sumPaidPrice.subtract(withdrawalAmount); // 未提现金额

        WithdrawalRecordCountResponse response = new WithdrawalRecordCountResponse();
        response.setSumPaidPrice(sumPaidPrice);
        response.setWithdrawalAmount(withdrawalAmount);
        response.setUndrawnAmount(undrawnAmount);

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @OperationLog("审核[提现记录]")
    @ApiOperation(value = "审核[提现记录]")
    @PutMapping("/{id:\\d+}/audit")
    public ResponseEntity<BaseResponse<?>> audit(@Valid @ApiParam("标识") @PathVariable String id,@Valid @RequestBody BaseRequest<WithdrawalRecordVerifyRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.handleAudit(id, request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
