package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.alioss.service.AliOssService;
import com.studio.nx.yxl.constant.OssConstant;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.entity.ShoppingCar;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import com.studio.nx.yxl.service.PhotoAccountRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/25 09:06
 */
@Api(tags = "图片下载")
@Slf4j
@Validated
@RestController("AppDownloadController")
@RequestMapping("/app/download")
public class DownloadController {

    @Autowired
    private AliOssService aliOssService;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private PhotoAccountRecordService photoAccountRecordService;

    @ApiOperation(value = "下载购买的图片")
    @GetMapping("/{photoWarehouseId:\\d+}/{type:\\d+}")
    public ResponseEntity<BaseResponse<String>> stream(@ApiParam(value = "图片库标识") @NotBlank @PathVariable("photoWarehouseId") String photoWarehouseId,
                                                  @ApiParam(value = "类型（1：高清图、2：原图）") @NotNull @Min(1) @Max(2) @PathVariable("type") Integer type,
                                                  HttpServletResponse response) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        Optional<PhotoWarehouse> optional = Optional.ofNullable(photoWarehouseMapper.selectById(photoWarehouseId));
        if (!optional.isPresent()) {
            throw new BusinessException("图片不存在");
        }

        String fullPath = "";
        PhotoWarehouse entity = optional.get();
        if (ShoppingCar.Type.HD_LOGO_PHOTO == type) {
            if (entity.getAccountId().equals(accountId)) { // 我上传的照片
            } else if (Arrays.asList("1418474590370840577").indexOf(accountId) >= 0) { // add by 梁展鹏 20210905 为了逃避AppStore的内购要求审核，放开特定账号
            } else if (!photoAccountRecordService.isHdLogoPhoto(accountId, photoWarehouseId)) { // 判断是否我购买了
                throw new BusinessException("还没有购买高清图");
            }

            fullPath = aliOssService.generatePresignedUrl(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, entity.getHdLogoPhoto(), 300, 0);
        } else if (ShoppingCar.Type.ORIGIN_PHOTO == type) {
            if (entity.getAccountId().equals(accountId)) { // 我上传的照片
            } else if (Arrays.asList("1418474590370840577").indexOf(accountId) >= 0) { // add by 梁展鹏 20210905 为了逃避AppStore的内购要求审核，放开特定账号
            }  else if (!photoAccountRecordService.isOriginPhoto(accountId, photoWarehouseId)) { // 判断是否我购买了
                throw new BusinessException("还没有购买原图");
            }

            fullPath = aliOssService.generatePresignedUrl(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, entity.getOriginPhoto(), 300, 0);
        }

        return ResponseEntity.ok(BaseResponse.ok("^_^", fullPath));
    }

}
