package com.studio.nx.yxl.controller.app;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.app.request.index.CollectionLikeRequest;
import com.studio.nx.yxl.controller.app.request.index.KindergartenSearchRequest;
import com.studio.nx.yxl.controller.app.response.KindergartenInfoResponse;
import com.studio.nx.yxl.controller.app.response.index.AgreementResponse;
import com.studio.nx.yxl.controller.app.response.index.BannerResponse;
import com.studio.nx.yxl.controller.app.response.index.IndexKindergartenResponse;
import com.studio.nx.yxl.controller.app.response.index.ParamsObjResponse;
import com.studio.nx.yxl.entity.Banner;
import com.studio.nx.yxl.entity.BannerMapStruct;
import com.studio.nx.yxl.service.BannerService;
import com.studio.nx.yxl.service.IndexService;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/15 22:24
 */
@Api(tags = "首页")
@Slf4j
@Validated
@RestController("AppIndexController")
@RequestMapping("/app/index")
public class IndexController {

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private LocalService localService;

    @Autowired
    private IndexService service;

    @Autowired
    private BannerService bannerService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    @IgnoreAuth
    @ApiOperation(
            value = "查询[横幅]列表",
            notes = "字段描述：<br/>" +
                    "type：0：首页（多个）、1：我的 - 开通会员专享权益、2：我的 - 积分签到、3：邀请拉新 - 主页、4：邀请拉新 - 注册页、5：会员中心 - 广告、6：会员中心 - 会员权益、7：会员中心 - 会员福利、8：启动应用广告、9：登录页背景图/视频、10：邀请注册海报"
    )
    @PostMapping("/banner")
    public ResponseEntity<BaseResponse<List<BannerResponse>>> banner(@Valid @RequestBody BaseRequest request) {
        List<Banner> list = bannerService.cacheList();

        /* 过滤多余的Banner数据，只有Banner场景返回多个，其余场景返回Top1 */
        List<Banner> result = new ArrayList<>();
        list.stream().collect(Collectors.groupingBy(Banner::getType))
                .forEach((key, value) -> {
                    if (Arrays.asList("0").indexOf(key) >= 0) { // 首页Banner场景，会有多个
                        result.addAll(value);
                    } else if (value.size() > 0) { // 其余场景只取第一个
                        result.add(value.get(RandomUtil.randomInt(value.size())));
                    }
                });

        return ResponseEntity.ok(BaseResponse.ok(
                result.stream()
                        .map(o -> BannerMapStruct.INSTANCES.toResponse(o))
                        .map(o -> o.setImgUrl(localService.fullPath(o.getImgUrl())))
                        .map(o -> o.setParamsObj(JSONObject.parseObject(o.getParams(), ParamsObjResponse.class)))
                        .collect(Collectors.toList())
        ));
    }

    /**
     * 功能描述：查询[首页场景]列表
     *
     * @return
     */
    @IgnoreAuth
    @ApiOperation(value = "查询[首页场景]列表")
    @PostMapping("/kindergarten/list")
    public ResponseEntity<BaseResponse<BasePageResponse<IndexKindergartenResponse>>> list(@Valid @RequestBody BasePageRequest<KindergartenSearchRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.appPage(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：查询[场景]详情
     *
     * @param kindergartenId 标识
     * @return
     */
    @ApiOperation(value = "查询[场景]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "kindergartenId", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/kindergarten/{kindergartenId:\\d+}")
    public ResponseEntity<BaseResponse<KindergartenInfoResponse>> findAppKindergartenInfo(@NotBlank @PathVariable("kindergartenId") String kindergartenId) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.clientDetail(kindergartenId, accountId)));
    }

    /**
     * 功能描述：点赞/收藏
     *
     * @return
     */
    @ApiOperation(value = "点赞/收藏")
    @PostMapping("/collection")
    public ResponseEntity<BaseResponse<?>> collection(@Valid @RequestBody BaseRequest<CollectionLikeRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.collection(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：取消点赞/收藏
     *
     * @return
     */
    @ApiOperation(value = "取消点赞/收藏")
    @PostMapping("/un-collection")
    public ResponseEntity<BaseResponse<?>> unCollection(@Valid @RequestBody BaseRequest<CollectionLikeRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.unCollection(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @IgnoreAuth
    @ApiOperation(value = "协议说明")
    @GetMapping("/agreement")
    public ResponseEntity<BaseResponse<AgreementResponse>> agreement() {
        String h5WebDomain = businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN);

        AgreementResponse response = new AgreementResponse();
        response.setLaunchAppAgreement(h5WebDomain + "/app/launch-app?backgroundColor=white" + getT()); // 儿童隐私保护声明
        response.setLoginLicenseAgreement(h5WebDomain + "/app/login/license_agreement?" + getT()); // 用户授权协议

        response.setMineCheckInHelp(h5WebDomain + "/app/mine/check_in_help?" + getT()); // 积分签到 - 帮忙说明
        response.setMineWalletHelp(h5WebDomain + "/app/mine/wallet_help?" + getT()); // 我的钱包 - 帮忙说明
        response.setPhotoDownloadAgreement(h5WebDomain + "/app/mine/photo-download?" + getT()); // 照片下载使用协议

        response.setSystemServiceAgreement(h5WebDomain + "/app/system/service_agreement?" + getT()); // 系统设置 - 服务协议
        response.setSystemPrivacyPolicy(h5WebDomain + "/app/system/privacy_policy?" + getT()); // 系统设置 - 隐私政策
        response.setSystemCommunityNorms(h5WebDomain + "/app/system/community_norms?" + getT()); // 系统设置 - 社区规范
        response.setSystemHelpIndex(h5WebDomain + "/app/system/help/info?" + getT()); // 系统设置 - 帮助中心
        response.setSystemContact(h5WebDomain + "/app/system/contact?" + getT()); // 系统设置 - 联系我们

        response.setApplyKindergarten(h5WebDomain + "/app/system/apply-kindergarten?" + getT()); // 申请场景认证协议
        response.setSubmitKindergarten(h5WebDomain + "/app/system/service_agreement?" + getT()); // 提交申请场景认证协议
        response.setApplyPhotoer(h5WebDomain + "/app/system/notice?" + getT()); // 申请摄影师认证协议
        response.setSubmitPhotoer(h5WebDomain + "/app/system/service_agreement?" + getT()); // 提交申请摄影师认证协议

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    private String getT() {
        return "&t=" + System.currentTimeMillis();
    }

    @ApiOperation(value = "分享奖励")
    @PostMapping("/share-points")
    public ResponseEntity<BaseResponse<?>> sharePoints() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        integralCashRecordService.saveIntegralSharePoints(accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
