package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.my.apply.KindergartenApplySaveRequest;
import com.studio.nx.yxl.controller.app.request.my.apply.PhotoerApplySaveRequest;
import com.studio.nx.yxl.controller.app.response.my.applys.KindergartenApplySaveResponse;
import com.studio.nx.yxl.controller.app.response.my.applys.PhotoerApplySaveResponse;
import com.studio.nx.yxl.service.KindergartenApplyService;
import com.studio.nx.yxl.service.PhotoerApplyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/16 14:52
 */
@Api(tags = "我的-申请认证")
@Slf4j
@Validated
@RestController("AppMyApplyController")
@RequestMapping("/app/my/apply")
public class MyApplyController {

    @Autowired
    private KindergartenApplyService kindergartenApplyService;

    @Autowired
    private PhotoerApplyService photoerApplyService;

    @ApiOperation(value = "获取[幼儿园认证]")
    @GetMapping("/kindergarten/get")
    public ResponseEntity<BaseResponse<KindergartenApplySaveResponse>> kindergartenGet() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(kindergartenApplyService.kindergartenGet(accountId)));
    }

    @ApiOperation(value = "提交[幼儿园认证]")
    @PostMapping("/kindergarten/save")
    public ResponseEntity<BaseResponse<?>> kindergartenSave(@Valid @RequestBody BaseRequest<KindergartenApplySaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        kindergartenApplyService.appKindergartenSave(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "更新[幼儿园认证]")
    @PostMapping("/kindergarten/update")
    public ResponseEntity<BaseResponse<?>> kindergartenUpdate(@Valid @RequestBody BaseRequest<KindergartenApplySaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        kindergartenApplyService.kindergartenUpdate(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "发送[幼儿园]签到二维码邮件")
    @PostMapping("/kindergarten/send-qrcode")
    public ResponseEntity<BaseResponse<?>> sendQRCode() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        kindergartenApplyService.sendKindergartenQrCodeEmailByAccountId(accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "获取[摄影师认证]")
    @GetMapping("/photoer/get")
    public ResponseEntity<BaseResponse<PhotoerApplySaveResponse>> photoerGet() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(photoerApplyService.photoerGet(accountId)));
    }

    @ApiOperation(value = "提交[摄影师认证]")
    @PostMapping("/photoer/save")
    public ResponseEntity<BaseResponse<?>> photoerSave(@Valid @RequestBody BaseRequest<PhotoerApplySaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        photoerApplyService.photoerSave(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "更新[摄影师认证]")
    @PostMapping("/photoer/update")
    public ResponseEntity<BaseResponse<?>> photoerUpdate(@Valid @RequestBody BaseRequest<PhotoerApplySaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        photoerApplyService.photoerUpdate(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
