package com.studio.nx.yxl.controller.app;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.ResourcesConstant;
import com.studio.nx.yxl.controller.app.response.my.collection.KindergartenCollectionResponse;
import com.studio.nx.yxl.controller.app.response.my.collection.TweetCollectionResponse;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityResponse;
import com.studio.nx.yxl.mapper.CollectionLikeMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/22 10:40
 */
@Api(tags = "我的-我的收藏")
@Slf4j
@Validated
@RestController("AppMyCollectionController")
@RequestMapping("/app/my/collection")
public class MyCollectionController {

    @Autowired
    private LocalService localService;

    @Autowired
    private ResourcesConstant resourcesStatic;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private CollectionLikeMapper collectionLikeMapper;

    @ApiOperation(value = "获取[幼儿园收藏]列表")
    @PostMapping("/kindergartens")
    public ResponseEntity<BaseResponse<BasePageResponse<KindergartenCollectionResponse>>> kindergartens(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<KindergartenCollectionResponse> p = collectionLikeMapper.findMyKindergartenCollection(accountId, new Page(request.getPage(), request.getLimit()));
        for (KindergartenCollectionResponse response : p.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "获取[活动收藏]列表")
    @PostMapping("/activitys")
    public ResponseEntity<BaseResponse<BasePageResponse<ActivityResponse>>> activitys(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<ActivityResponse> p = collectionLikeMapper.findMyActivityCollection(accountId, new Page(request.getPage(), request.getLimit()));
        for (ActivityResponse response : p.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }


    @ApiOperation(value = "获取[资讯收藏]列表")
    @PostMapping("/tweets")
    public ResponseEntity<BaseResponse<BasePageResponse<TweetCollectionResponse>>> tweets(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<TweetCollectionResponse> p = collectionLikeMapper.findMyTweetCollection(accountId, new Page(request.getPage(), request.getLimit()));
        for (TweetCollectionResponse response : p.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }

            /* add by 梁展鹏 20210830 增加参数使app内访问隐藏引导下载浮动框 */
            response.setLink(String.format("%s/h5/tweet?id=%s&hidden=1", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 访问地址

            response.setShareTitle(String.format("%s", response.getTitle())); // 分享-标题
            response.setShareDesc(String.format("%s", response.getBrief())); // 分享-简介
            response.setShareLink(String.format("%s/h5/tweet?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 分享-链接
            response.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图
        }

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

}
