package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.uiac.controller.request.my.PersonInfoRequest;
import com.studio.nx.uiac.dto.AccountInfoDto;
import com.studio.nx.uiac.service.AccountService;
import com.studio.nx.uiac.service.AuthService;
import com.studio.nx.yxl.service.CollectionLikeService;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import com.studio.nx.yxl.service.PhotoGetLikeRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/16 14:52
 */
@Api(tags = "我的-个人信息")
@Slf4j
@Validated
@RestController("AppMyInfoController")
@RequestMapping("/app/my/info")
public class MyInfoController {

    @Autowired
    private AccountService accountService;

    @Autowired
    private AuthService authService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    @Autowired
    private CollectionLikeService collectionLikeService;

    @Autowired
    private PhotoGetLikeRecordService photoGetLikeRecordService;

    /**
     * 功能描述：更新用户信息
     *
     * @return
     */
    @ApiOperation(
            value = "更新用户信息",
            notes = "avatarUrl字段更新说明：使用/framework/oss/local/upload?prefix=接口，并且接口参数prefix='/avatar-url' <br/>"
    )
    @PostMapping("")
    public ResponseEntity<BaseResponse> updateMine(@Valid @RequestBody BaseRequest<PersonInfoRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        accountService.appUpdateMine(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "当前登录人信息")
    @GetMapping("/myself")
    public ResponseEntity<BaseResponse<AccountInfoDto>> myself() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        AccountInfoDto accountInfoDto = authService.appMyself(accountId, nickname);
        accountInfoDto.setIntegral(integralCashRecordService.countMyIntegral(accountId).longValue()); // 积分
        accountInfoDto.setCollections(collectionLikeService.countMyCollection(accountId).longValue()); // 收藏
        accountInfoDto.setLikes(photoGetLikeRecordService.countMyphotoGetLike(accountId).longValue()); // 获赞
        return ResponseEntity.ok(BaseResponse.ok(accountInfoDto));
    }

}
