package com.studio.nx.yxl.controller.app;

import cn.hutool.core.util.RandomUtil;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.config.LocalYaml;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.app.response.invite.InviteResponse;
import com.studio.nx.yxl.controller.app.response.my.IntegralCashRecordResponse;
import com.studio.nx.yxl.entity.Banner;
import com.studio.nx.yxl.service.BannerService;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/16 14:52
 */
@Api(tags = "我的-邀请好友")
@Slf4j
@Validated
@RestController("AppMyInviteController")
@RequestMapping("/app/my/invite")
public class MyInviteController {

    @Autowired
    private LocalYaml localYaml;

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    @Autowired
    private BannerService bannerService;

    @ApiOperation(value = "获取[邀请拉新]生成海报")
    @GetMapping("/get")
    public ResponseEntity<BaseResponse<?>> get() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        /* 获取Banner类型为：邀请好友注册 */
        List<Banner> bannerResponse = bannerService.cacheList().stream().filter(o -> o.getType().equals("10")).collect(Collectors.toList());
        if (bannerResponse.size() == 0) {
            return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "生成海报失败"));
        }

        InviteResponse response = new InviteResponse();
        response.setPosterUrl(localService.fullPath(bannerResponse.get(RandomUtil.randomInt(bannerResponse.size())).getImgUrl()));
        response.setDisplayText("邀请你加入幼享乐，一起'记录 ·分享·幼儿快乐'。");
        response.setLogoUrl(String.format("%s/static/yxl/invite-poster-logo.png", localYaml.getHost()));
        response.setQrText(String.format("%s/h5/invite?invitationCode=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), accountId));

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "获取[邀请记录]")
    @PostMapping("/record")
    public ResponseEntity<BaseResponse<BasePageResponse<IntegralCashRecordResponse>>> record(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(integralCashRecordService.pageInvite(accountId, request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "统计[邀请记录]")
    @GetMapping("/count")
    public ResponseEntity<BaseResponse<Integer>> record() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(integralCashRecordService.countInvite(accountId)));
    }

}
