package com.studio.nx.yxl.controller.app;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.controller.app.response.photowall.PhotogalleryResponse;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/22 15:21
 */
@Api(tags = "我的-我的图片库")
@Slf4j
@Validated
@RestController("AppMyPhotoGalleryController")
@RequestMapping("/app/my/photo-gallery")
public class MyPhotoGalleryController {

    @Autowired
    private LocalService localService;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @ApiOperation(value = "获取[已购买]列表")
    @PostMapping("/purchases")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotogalleryResponse>>> purchases(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<PhotogalleryResponse> p = photoWarehouseMapper.findMyPurchasePhotoList(accountId, new Page(request.getPage(), request.getLimit()));
        p.getRecords().stream().forEach(e -> e.setListPreview(localService.fullPath(e.getListPreview())));

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "获取[已收藏]列表")
    @PostMapping("/collectionss")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotogalleryResponse>>> collectionss(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<PhotogalleryResponse> p = photoWarehouseMapper.findMyCollectionsPhotoList(accountId, new Page(request.getPage(), request.getLimit()));
        p.getRecords().stream().forEach(e -> e.setListPreview(localService.fullPath(e.getListPreview())));

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "获取[已上传]列表")
    @PostMapping("/takes")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotogalleryResponse>>> takes(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<PhotogalleryResponse> p = photoWarehouseMapper.findMyPhotoList(accountId, new Page(request.getPage(), request.getLimit()));
        p.getRecords().stream().forEach(e -> e.setListPreview(localService.fullPath(e.getListPreview())));

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

}
