package com.studio.nx.yxl.controller.app;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.app.response.my.PhotoGetLikeRecordDetailResponse;
import com.studio.nx.yxl.controller.app.response.my.PhotoGetLikeRecordResponse;
import com.studio.nx.yxl.entity.PhotoGetLikeRecord;
import com.studio.nx.yxl.entity.PhotoGetLikeRecordMapStruct;
import com.studio.nx.yxl.mapper.PhotoGetLikeRecordMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/24 21:19
 */
@Api(tags = "我的-我的获赞")
@Slf4j
@Validated
@RestController("AppMyPhotoGetLikeRecordController")
@RequestMapping("/app/my/photo-get-like")
public class MyPhotoGetLikeRecordController {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private PhotoGetLikeRecordMapper mapper;

    @ApiOperation(value = "获取[我的获赞]列表")
    @PostMapping("/records")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoGetLikeRecordResponse>>> records(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<PhotoGetLikeRecordResponse> p = mapper.appPage(accountId, new Page(request.getPage(), request.getLimit()));
        p.getRecords().stream().forEach(e -> e.setListPreview(localService.fullPath(e.getListPreview())));

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "获取[获赞明细]列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "photoWarehouseId", value = "照片库标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @PostMapping("/{photoWarehouseId:\\d+}/detail")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoGetLikeRecordDetailResponse>>> record(@NotBlank @PathVariable("photoWarehouseId") String photoWarehouseId, @Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        IPage<PhotoGetLikeRecord> p = mapper.selectPage(new Page(request.getPage(), request.getLimit()), new LambdaQueryWrapper<PhotoGetLikeRecord>()
                .eq(PhotoGetLikeRecord::getPhotoWarehouseId, photoWarehouseId)
                .eq(PhotoGetLikeRecord::getOriginAccountId, accountId)
                .orderByDesc(PhotoGetLikeRecord::getCreateAt)
        );

        List<PhotoGetLikeRecordDetailResponse> list = p.getRecords().stream()
                .map(o -> PhotoGetLikeRecordMapStruct.INSTANCES.toResponse(o))
                .collect(Collectors.toList());
        list.stream().forEach(e -> e.setShowValue(String.format("+%s 积分", businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_POINTS_BY_LIKE))));

        BasePageResponse response = new BasePageResponse().setTotal(p.getTotal()).setData(list);
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

}
