package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.entity.PhotoerSignInRecord;
import com.studio.nx.yxl.service.PhotoerSignInRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/7/10 10:32
 */
@Api(tags = "我的-扫一扫")
@Slf4j
@Validated
@RestController("AppMyPhotoerSignInController")
@RequestMapping("/app/my/photoer/sign-in")
public class MyPhotoerSignInRecordController {

    @Autowired
    private PhotoerSignInRecordService service;

    @ApiOperation(
            value = "打卡",
            notes = "幼儿园认证通过会生成二维码并发邮件到申请人邮箱上。二维码组成为幼儿园的h5详情页地址：/h5/kindergarten?id=。客户端扫码解析出文本，截取URL的参数id，调用本接口即可"
    )
    @GetMapping("/{kindergartenId:\\d+}")
    public ResponseEntity<BaseResponse> signIn(@NotBlank @PathVariable("kindergartenId") String kindergartenId) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.signIn(kindergartenId, accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "我的打卡记录")
    @PostMapping("/record")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoerSignInRecord>>> record(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.record(accountId, request.getPage(), request.getLimit())));
    }

}
