package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.my.wallet.WalletPageRequest;
import com.studio.nx.yxl.controller.app.response.my.IntegralCashRecordResponse;
import com.studio.nx.yxl.controller.app.response.my.MyIntegralCashRecordResponse;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author loki
 * @Date 2021/7/1 14:52
 */
@Api(tags = "我的-钱包")
@Slf4j
@Validated
@RestController("AppMyWalletController")
@RequestMapping("/app/my/wallet")
public class MyWalletController {

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    @ApiOperation(value = "查询[积分剩余 + 余额剩余]")
    @GetMapping("/total")
    public ResponseEntity<BaseResponse<MyIntegralCashRecordResponse>> getTotal() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        MyIntegralCashRecordResponse response = new MyIntegralCashRecordResponse();
        response.setIntegral(integralCashRecordService.countMyIntegral(accountId));
        response.setCash(integralCashRecordService.countMyCash(accountId));
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "查询[积分记录]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<IntegralCashRecordResponse>>> page(@Valid @RequestBody BasePageRequest<WalletPageRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(integralCashRecordService.appPage(request.getData(), accountId, request.getPage(), request.getLimit())));
    }

}
