package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.my.withdrawal.WithdrawalRecordApplyRequest;
import com.studio.nx.yxl.entity.WithdrawalRecord;
import com.studio.nx.yxl.service.WithdrawalRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/21 16:40
 */
@Api(tags = "我的-提现记录")
@Slf4j
@Validated
@RestController("AppMyWithdrawalRecordController")
@RequestMapping("/app/my/withdrawal/record")
public class MyWithdrawalRecordController {

    @Autowired
    private WithdrawalRecordService service;

    /**
     * 功能描述：提交[提现记录]
     *
     * @return
     */
    @ApiOperation(value = "提交[提现记录]")
    @PostMapping("/apply")
    public ResponseEntity<BaseResponse<?>> apply(@Valid @RequestBody BaseRequest<WithdrawalRecordApplyRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.apply(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    /**
     * 功能描述：查询[提现记录]列表
     * @return
     */
    @ApiOperation(value = "查询[提现记录]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<WithdrawalRecord>>> page(@Valid @RequestBody BasePageRequest request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.appPage(accountId, request.getPage(), request.getLimit())));
    }

}
