package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ShoppingCarAddRequest;
import com.studio.nx.yxl.controller.app.response.order.OrderResponse;
import com.studio.nx.yxl.service.OrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "订单服务")
@Slf4j
@Validated
@RestController("AppOrderController")
@RequestMapping("/app/order")
public class OrderController {

    @Autowired
    private OrderService orderService;

    @ApiOperation(
            value = "生成订单（购物车id）",
            notes = "根据购物车id集合，生成订单"
    )
    @PostMapping("/generate")
    public ResponseEntity<BaseResponse<OrderResponse>> generateByShoppingCar(@Valid @RequestBody BaseRequest<List<String>> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(orderService.generateByShoppingCarId(request.getData(), accountId)));
    }

    @ApiOperation(
            value = "生成订单（照片id）",
            notes = "根据照片id集合，生成订单"
    )
    @PostMapping("/generate-by-photo")
    public ResponseEntity<BaseResponse<OrderResponse>> generateByPhoto(@Valid @RequestBody BaseRequest<List<ShoppingCarAddRequest>> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(orderService.generateByPhoto(request.getData(), accountId)));
    }

}
