package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseSaveRequest;
import com.studio.nx.yxl.controller.app.request.photowall.PhotoCollectionLikeRequest;
import com.studio.nx.yxl.controller.app.request.photowall.activity.PhotoWallActivityRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.ChoicePhotoRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.PhotoWallKindergartenInfoRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.PhotoWallKindergartenListRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.PhotoWallKindergartenRequest;
import com.studio.nx.yxl.controller.app.response.photowall.PhotoInfoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.ChoicePhotoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.PhotoWallKindergartenInfoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.PhotoWallKindergartenResponse;
import com.studio.nx.yxl.service.PhotoWallActivityService;
import com.studio.nx.yxl.service.PhotoWallKindergartenService;
import com.studio.nx.yxl.service.PhotoWarehouseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/24 14:44
 */
@Api(tags = "照片墙")
@Slf4j
@Validated
@RestController("AppPhotoWallController")
@RequestMapping("/app/photo-wall")
public class PhotoWallController {

    @Autowired
    private PhotoWallActivityService photoWallActivityService;

    @Autowired
    private PhotoWallKindergartenService photoWallKindergartenService;

    @Autowired
    private PhotoWarehouseService photoWarehouseService;

    @ApiOperation(value = "照片墙 - 幼儿园列表")
    @PostMapping("/kindergarten/list")
    public ResponseEntity<BaseResponse<BasePageResponse<PhotoWallKindergartenResponse>>> kindergartenList(@Valid @RequestBody BasePageRequest<PhotoWallKindergartenRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.list(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "照片墙 - 幼儿园 - 摄影师拍摄/随手拍")
    @PostMapping("/kindergarten/{kindergartenId:\\d+}")
    public ResponseEntity<BaseResponse<PhotoWallKindergartenInfoResponse>> photoInfo(@NotBlank @PathVariable("kindergartenId") String kindergartenId, @Valid @RequestBody BaseRequest<PhotoWallKindergartenInfoRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.photoInfo(kindergartenId, request.getData())));
    }

    @Deprecated
    @ApiOperation(value = "照片墙 - 幼儿园 - 摄影师拍摄/随手拍 - 日期照片列表")
    @PostMapping("/kindergarten/dayList")
    public ResponseEntity<BaseResponse<List<ChoicePhotoResponse>>> photoListByDay(@Valid @RequestBody BaseRequest<PhotoWallKindergartenListRequest> request) {
        String kindergartenId = request.getData().getKindergartenId();
        String photoTime = request.getData().getCreateAt();
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.photoListByDay(kindergartenId, photoTime)));
    }

    @ApiOperation(value = "照片墙 - 幼儿园 - 摄影师拍摄/随手拍 - 日期照片列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "kindergartenId", value = "幼儿园标识", dataType = "String", paramType = "path", required = true, defaultValue = ""),
            @ApiImplicitParam(name = "photoTime", value = "拍摄时间（yyyy-MM-dd）", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @PostMapping("/kindergarten/{kindergartenId:\\d+}/{photoTime}/list")
    public ResponseEntity<BaseResponse<List<ChoicePhotoResponse>>> photoListByDay(@NotBlank @PathVariable("kindergartenId") String kindergartenId,
                                                                                  @NotBlank @PathVariable("photoTime") String photoTime,
                                                                                  @Valid @RequestBody BaseRequest request) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.photoListByDay(kindergartenId, photoTime)));
    }

    @ApiOperation(value = "照片详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "照片标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/photo/info/{id:\\d+}")
    public ResponseEntity<BaseResponse<PhotoInfoResponse>> photoInfoById(@NotBlank @PathVariable("id") String id) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.photoInfoById(id, accountId)));
    }

    @ApiOperation(value = "照片墙 - 精选/照片墙 - 幼儿园 - 精选")
    @PostMapping("/beautiful")
    public ResponseEntity<BaseResponse<BasePageResponse<ChoicePhotoResponse>>> beautiful(@Valid @RequestBody BasePageRequest<ChoicePhotoRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallKindergartenService.beautifulList(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "照片墙 - 幼儿园 - 随手拍(上传)")
    @PostMapping("/take")
    public ResponseEntity<BaseResponse> take(@Valid @RequestBody BaseRequest<PhotoWarehouseSaveRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        photoWarehouseService.appPhotoerSave(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "照片墙 - 活动列表")
    @PostMapping("/activity/list")
    public ResponseEntity<BaseResponse<BasePageResponse<ActivityResponse>>> activityList(@Valid @RequestBody BasePageRequest<PhotoWallActivityRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(photoWallActivityService.list(request.getData(), accountId, request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "照片墙 - 活动 - 详情（日期列表）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "activityId", value = "活动标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/activity/{activityId:\\d+}")
    public ResponseEntity<BaseResponse<ActivityResponse>> appDetail(@NotBlank @PathVariable("activityId") String activityId) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(photoWallActivityService.clientDetail(activityId, accountId)));
    }

    @ApiOperation(value = "照片墙 - 活动 - 详情（照片列表）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "activityId", value = "活动标识", dataType = "String", paramType = "path", required = true, defaultValue = ""),
            @ApiImplicitParam(name = "photoTime", value = "拍摄时间（yyyy-MM-dd）", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/activity/{activityId:\\d+}/{photoTime}")
    public ResponseEntity<BaseResponse<ActivityResponse>> appDetailByPhotoTime(@NotBlank @PathVariable("activityId") String activityId,
                                                                               @NotBlank @PathVariable("photoTime") String photoTime) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(photoWallActivityService.clientDetailByPhotoTime(activityId, photoTime, accountId)));
    }

    @ApiOperation(value = "照片 - 点赞/收藏")
    @PostMapping("/collection")
    public ResponseEntity<BaseResponse<?>> collection(@Valid @RequestBody BaseRequest<PhotoCollectionLikeRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        photoWallActivityService.collection(request.getData(), accountId, nickname);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(value = "照片 - 取消点赞/收藏")
    @PostMapping("/un-collection")
    public ResponseEntity<BaseResponse<?>> unCollection(@Valid @RequestBody BaseRequest<PhotoCollectionLikeRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        photoWallActivityService.unCollection(request.getData(), accountId, nickname);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
