package com.studio.nx.yxl.controller.app;

import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.ResourcesConstant;
import com.studio.nx.yxl.controller.app.response.search.IndexSearchInfoResponse;
import com.studio.nx.yxl.controller.app.response.search.IndexSearchResponse;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import com.studio.nx.yxl.service.IndexService;
import com.studio.nx.yxl.service.TopSearchKeywordsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/16 15:30
 */
@Api(tags = "搜索")
@Slf4j
@Validated
@RestController("AppSearchController")
@RequestMapping("/app/search")
public class SearchController {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private ResourcesConstant resourcesStatic;

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private IndexService indexService;

    @Autowired
    private TopSearchKeywordsService topSearchKeywordsService;

    /**
     * 功能描述：查询[热门搜索词]列表
     *
     * @return
     */
    @ApiOperation(value = "查询[热门搜索词]列表")
    @PostMapping("/top-search-keywords")
    public ResponseEntity<BaseResponse<List<String>>> topSearchKeywords() {
        List<String> list = topSearchKeywordsService.list().stream()
                .map(o -> o.getName())
                .collect(Collectors.toList());

        return ResponseEntity.ok(BaseResponse.ok(list));
    }

    /**
     * 功能描述：搜索
     *
     * @return
     */
    @IgnoreAuth
    @ApiOperation(value = "搜索")
    @GetMapping("/keyword")
    public ResponseEntity<BaseResponse<IndexSearchResponse>> search(@RequestParam @ApiParam("关键词") String request) {
        List<IndexSearchInfoResponse> indexSearchList = kindergartenMapper.findIndexSearchList(request);
        for (IndexSearchInfoResponse response : indexSearchList) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        IndexSearchResponse response = new IndexSearchResponse();

        /* 幼儿园 */
        List<IndexSearchInfoResponse> kindergartenList = indexSearchList.stream().filter(o -> o.getType() == 1).collect(Collectors.toList());
        kindergartenList.stream().forEach(o -> {
            o.setShareTitle(String.format("%s", o.getName())); // 分享-标题
            o.setShareDesc(String.format("%s", o.getBrief())); // 分享-简介
            o.setShareLink(String.format("%s/h5/kindergarten?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), o.getId())); // 分享-链接
            o.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图
        });
        response.setKindergartenList(kindergartenList);

        /* 活动 */
        List<IndexSearchInfoResponse> activityList = indexSearchList.stream().filter(o -> o.getType() == 3).collect(Collectors.toList());
        activityList.stream().forEach(o -> {
            o.setShareTitle(String.format("%s", o.getName())); // 分享-标题
            o.setShareDesc(String.format("%s", o.getBrief())); // 分享-简介
            o.setShareLink(String.format("%s/h5/activity?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), o.getId())); // 分享-链接
            o.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图
        });
        response.setActivityList(activityList);

        /* 资讯 */
        List<IndexSearchInfoResponse> tweetList = indexSearchList.stream().filter(o -> o.getType() == 2).collect(Collectors.toList());
        tweetList.stream().forEach(o -> {
            /* add by 梁展鹏 20210830 增加参数使app内访问隐藏引导下载浮动框 */
            o.setLink(String.format("%s/h5/tweet?id=%s&hidden=1", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), o.getId())); // 访问地址

            o.setShareTitle(String.format("%s", o.getName())); // 分享-标题
            o.setShareDesc(String.format("%s", o.getBrief())); // 分享-简介
            o.setShareLink(String.format("%s/h5/tweet?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), o.getId())); // 分享-链接
            o.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图
        });
        response.setTweetList(tweetList);

        return ResponseEntity.ok(BaseResponse.ok(response));
    }

}
