package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ShoppingCarAddRequest;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ToCollectionRequest;
import com.studio.nx.yxl.controller.app.response.shoppingcar.ShoppingCarResponse;
import com.studio.nx.yxl.service.ShoppingCarService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/26 17:20
 */
@Api(tags = "购物车")
@Slf4j
@Validated
@RestController("AppShoppingCarController")
@RequestMapping("/app/shopping-car")
public class ShoppingCarController {

    @Autowired
    private ShoppingCarService service;

    @ApiOperation(value = "获取[购物车数量]")
    @GetMapping("/number")
    public ResponseEntity<BaseResponse<Integer>> carNumber() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.carNumber(accountId)));
    }

    @ApiOperation(value = "获取[购物车列表]")
    @GetMapping("/list")
    public ResponseEntity<BaseResponse<List<ShoppingCarResponse>>> carList() {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.carList(accountId)));
    }

    @ApiOperation(value = "提交[加入购物车]")
    @PostMapping("/add")
    public ResponseEntity<BaseResponse<ShoppingCarResponse>> addShoppingCar(@Valid @RequestBody BaseRequest<ShoppingCarAddRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();

        /* add by 梁展鹏 20210910 为了逃避AppStore的内购要求审核，这里hardcode为特殊账号时,直接返回。 */
        if (Arrays.asList("1418474590370840577").indexOf(accountId) >= 0) {
            throw new BusinessException("您已经购买该照片，可直接下载！～");
        }

        return ResponseEntity.ok(BaseResponse.ok(service.addShoppingCar2Response(request.getData(), accountId)));
    }

    @ApiOperation(value = "提交[移入收藏夹]")
    @PostMapping("/to-collection")
    public ResponseEntity<BaseResponse<?>> toCollection(@Valid @RequestBody BaseRequest<ToCollectionRequest> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        String nickname = JWTUtil.getLocalUser().get().getNickname();
        service.toCollection(request.getData(), accountId, nickname);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @ApiOperation(
            value = "删除[购物车项]",
            notes = "传入为shoppingCarId集合"
    )
    @PostMapping("/delete")
    public ResponseEntity<BaseResponse<?>> delete(@Valid @RequestBody BaseRequest<List<String>> request) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        service.deleteByShoppingCarId(request.getData(), accountId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
