package com.studio.nx.yxl.controller.app;

import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.yxl.controller.app.request.tweet.TweetRequest;
import com.studio.nx.yxl.controller.app.response.tweet.TweetListResponse;
import com.studio.nx.yxl.service.TweetService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/17 13:50
 */
@Api(tags = "资讯")
@Slf4j
@Validated
@RestController("AppTweetController")
@RequestMapping("/app/tweet")
public class TweetController {

    @Autowired
    private TweetService service;

    /**
     * 功能描述：查询[APP-资讯]列表
     *
     * @return
     */
    @IgnoreAuth
    @ApiOperation(value = "查询[APP-资讯]列表")
    @PostMapping("/list")
    public ResponseEntity<BaseResponse<BasePageResponse<TweetListResponse>>> list(@Valid @RequestBody BasePageRequest<TweetRequest> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.appList(request.getData(), request.getPage(), request.getLimit())));
    }

    /**
     * 功能描述：查询[APP-资讯]点赞收藏情况
     *
     * @param tweetId 标识
     * @return
     */
    @ApiOperation(value = "查询[APP-资讯]点赞收藏情况")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tweetId", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/collection/{tweetId:\\d+}")
    public ResponseEntity<BaseResponse<?>> appTweetInfoCollection(@NotBlank @PathVariable("tweetId") String tweetId) {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        return ResponseEntity.ok(BaseResponse.ok(service.appTweetInfoCollection(tweetId, accountId)));
    }

}
