package com.studio.nx.yxl.controller.h5;

import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityResponse;
import com.studio.nx.yxl.service.PhotoWallActivityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author loki
 * @Date 2021/6/19 21:27
 */
@Api(tags = "活动")
@Slf4j
@Validated
@RestController("H5ActivityController")
@RequestMapping("/h5/activity")
public class ActivityController {

    @Autowired
    private PhotoWallActivityService photoWallActivityService;

    @IgnoreAuth
    @ApiOperation(value = "照片墙 - 活动 - 详情（日期列表）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "activityId", value = "活动标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{activityId:\\d+}")
    public ResponseEntity<BaseResponse<ActivityResponse>> appDetail(@NotBlank @PathVariable("activityId") String activityId) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallActivityService.clientDetail(activityId, null)));
    }

    @IgnoreAuth
    @ApiOperation(value = "照片墙 - 活动 - 详情（照片列表）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "activityId", value = "活动标识", dataType = "String", paramType = "path", required = true, defaultValue = ""),
            @ApiImplicitParam(name = "photoTime", value = "拍摄时间（yyyy-MM-dd）", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{activityId:\\d+}/{photoTime}")
    public ResponseEntity<BaseResponse<ActivityResponse>> appDetailByPhotoTime(@NotBlank @PathVariable("activityId") String activityId,
                                                                               @NotBlank @PathVariable("photoTime") String photoTime) {
        return ResponseEntity.ok(BaseResponse.ok(photoWallActivityService.clientDetailByPhotoTime(activityId, photoTime, null)));
    }

}
