package com.studio.nx.yxl.controller.h5;

import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.yxl.controller.app.response.KindergartenInfoResponse;
import com.studio.nx.yxl.service.IndexService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotBlank;

/**
 * 功能描述：
 *
 * @Author loki
 * @Date 2021/6/19 21:27
 */
@Api(tags = "幼儿园")
@Slf4j
@Validated
@RestController("H5KindergartenController")
@RequestMapping("/h5/kindergarten")
public class KindergartenController {

    @Autowired
    private IndexService service;

    /**
     * 功能描述：查询[幼儿园]详情
     *
     * @param kindergartenId 标识
     * @return
     */
    @IgnoreAuth
    @ApiOperation(value = "查询[幼儿园]详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "kindergartenId", value = "标识", dataType = "String", paramType = "path", required = true, defaultValue = "")
    })
    @GetMapping("/{kindergartenId:\\d+}")
    public ResponseEntity<BaseResponse<KindergartenInfoResponse>> findAppKindergartenInfo(@NotBlank @PathVariable("kindergartenId") String kindergartenId) {
        return ResponseEntity.ok(BaseResponse.ok(service.clientDetail(kindergartenId, null)));
    }

}
