package com.studio.nx.yxl.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.studio.nx.framework.entity.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDateTime;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@TableName("t_kindergarten_info")
public class KindergartenInfo extends BaseEntity {

    @ApiModelProperty("幼儿园标识")
    @NotEmpty
    @TableField(value = "kindergarten_id")
    private String kindergartenId;

    @ApiModelProperty("账号标识")
    @NotEmpty
    @TableField(value = "account_id")
    private String accountId;

    @ApiModelProperty("申请人真实姓名")
    @TableField(value = "real_name")
    private String realName;

    @ApiModelProperty("营业执照（Minio上传）")
    @TableField(value = "business_license")
    private String businessLicense;

    @ApiModelProperty("申请人手机号")
    @TableField(value = "phonenumber")
    private String phonenumber;

    @ApiModelProperty("申请人身份证")
    @TableField(value = "id_card_number")
    private String idCardNumber;

    @ApiModelProperty("申请人证件正面（Minio上传）")
    @TableField(value = "id_card_front")
    private String idCardFront;

    @ApiModelProperty("申请人证件背面（Minio上传）")
    @TableField(value = "id_card_back")
    private String idCardBack;

    @ApiModelProperty("申请人半身照（Minio上传）")
    @TableField(value = "body_photo")
    private String bodyPhoto;

    @ApiModelProperty("申请人邮箱")
    @NotEmpty
    private String email;

    @ApiModelProperty("审核账号标识")
    @TableField(value = "audit_account_id")
    private String auditAccountId;

    @ApiModelProperty("审核状态（1：审核通过、0：未审核、-1:审核失败）")
    @TableField(value = "audit_status")
    private Integer auditStatus;

    @ApiModelProperty("审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "audit_date")
    private LocalDateTime auditDate;

    @ApiModelProperty("审核备注")
    @TableField(value = "audit_msg")
    private String auditMsg;

    public static class AuditStatus {
        public static final Integer PASS = 1; // 审核通过
        public static final Integer WAIT = 0; // 未审核
        public static final Integer FAIL = -1; // 审核失败
    }

}

