package com.studio.nx.yxl.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.studio.nx.framework.config.json.BigDecimalSerialize;
import com.studio.nx.framework.entity.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/28 15:06
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@TableName("t_order")
public class Order extends BaseEntity {

    @ApiModelProperty("账号标识")
    @NotEmpty
    @TableField(value = "account_id")
    private String accountId;

    @ApiModelProperty("订单原价")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "origin_price")
    private BigDecimal originPrice;

    @ApiModelProperty("订单应付")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "payable_price")
    private BigDecimal payablePrice;

    @ApiModelProperty("订单实付")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "paid_price")
    private BigDecimal paidPrice;

    @ApiModelProperty("支付渠道（积分、微信、支付宝）")
    @NotEmpty
    @TableField(value = "pay_channels")
    private String payChannels;

    @ApiModelProperty("订单状态")
    @NotNull
    @TableField(value = "state")
    private Integer state;

    @ApiModelProperty("支付时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "pay_end_time")
    private LocalDateTime payEndTime;

    @ApiModelProperty("第三方平台支付标识")
    @NotNull
    @TableField(value = "transaction_id")
    private String transactionId;

    public static class PayChannels {
        public static final String INTEGRAL = "积分";
        public static final String WEIXIN = "微信";
        public static final String ALIPAY = "支付宝";
    }

    public static class State {
        public static final Integer DELETED = -1; // 取消支付（30分钟后，自动关单）
        public static final Integer WAIT_PAY = 0; // 待支付或支付中...
        public static final Integer SUCCESS_PAY = 1; // 已支付
    }

}
