package com.studio.nx.yxl.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.studio.nx.framework.config.json.BigDecimalSerialize;
import com.studio.nx.framework.entity.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * 功能描述：照片库
 *
 * @Author 梁展鹏
 * @Date 2021/6/11 16:00
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@TableName("t_photo_warehouse")
public class PhotoWarehouse extends BaseEntity {

    @ApiModelProperty("幼儿园标识")
    @NotEmpty
    @TableField(value = "kindergarten_id")
    private String kindergartenId;

    @ApiModelProperty("活动标识")
    @TableField(value = "activity_id")
    private String activityId;

    @ApiModelProperty("账号标识")
    @NotEmpty
    @TableField(value = "account_id")
    private String accountId;

    @ApiModelProperty("原始文件名称")
    @NotEmpty
    @TableField(value = "origin_filename")
    private String originFilename;

    @ApiModelProperty("拍照时间（读取图片信息）")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "photo_time")
    private LocalDateTime photoTime;

    @ApiModelProperty("是否横图")
    @TableField(value = "is_horizontal")
    private Boolean isHorizontal;

    @ApiModelProperty("列表页预览图（幼儿园logo、品牌logo）")
    @TableField(value = "list_preview")
    private String listPreview;

    @ApiModelProperty("详情页预览图（水印、幼儿园logo、品牌logo）")
    @TableField(value = "detail_preview")
    private String detailPreview;

    @ApiModelProperty("高清带logo大小")
    @TableField(value = "hd_logo_photo_size")
    private Long hdLogoPhotoSize;

    @ApiModelProperty("高清带logo原价格")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "hd_logo_photo_origin_price")
    private BigDecimal hdLogoPhotoOriginPrice;

    @ApiModelProperty("高清带logo价格")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "hd_logo_photo_price")
    private BigDecimal hdLogoPhotoPrice;

    @ApiModelProperty("高清带logo像素")
    @TableField(value = "hd_logo_photo_pixel")
    private String hdLogoPhotoPixel;

    @ApiModelProperty("高清带logo")
    @TableField(value = "hd_logo_photo")
    private String hdLogoPhoto;

    @ApiModelProperty("原图大小")
    @TableField(value = "origin_photo_size")
    private Long originPhotoSize;

    @ApiModelProperty("原图原价格")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @TableField(value = "origin_photo_origin_price")
    private BigDecimal originPhotoOriginPrice;

    @ApiModelProperty("原图价格")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @TableField(value = "origin_photo_price")
    private BigDecimal originPhotoPrice;

    @ApiModelProperty("原图像素")
    @TableField(value = "origin_photo_pixel")
    private String originPhotoPixel;

    @ApiModelProperty("原图")
    @TableField(value = "origin_photo")
    private String originPhoto;

    @ApiModelProperty("点赞数")
    @NotNull
    @TableField(value = "likes")
    private Integer likes;

    @ApiModelProperty("收藏数")
    @NotNull
    @TableField(value = "collections")
    private Integer collections;

    @ApiModelProperty("是否为随手拍")
    @NotNull
    @TableField(value = "is_take")
    private Boolean isTake;

    @ApiModelProperty("是否为精选")
    @NotNull
    @TableField(value = "is_beautiful")
    private Boolean isBeautiful;

    @ApiModelProperty("是否处理成功（图片裂变）")
    @NotNull
    @TableField(value = "is_ready")
    private Boolean isReady;

    @ApiModelProperty("是否上架")
    @NotNull
    @TableField(value = "is_show")
    private Boolean isShow;

    @ApiModelProperty("审核状态（1：审核通过、0：未审核、-1:审核失败）")
    @NotNull
    @TableField(value = "audit_status")
    private Integer auditStatus;

    @ApiModelProperty("审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "audit_date")
    private LocalDateTime auditDate;

    @ApiModelProperty("审核备注")
    @TableField(value = "audit_msg")
    private String auditMsg;

    public static class AuditStatus {
        public static final Integer PASS = 1; // 审核通过
        public static final Integer WAIT = 0; // 未审核
        public static final Integer FAIL = -1; // 审核失败
    }

}
