package com.studio.nx.yxl.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.studio.nx.framework.entity.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDateTime;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@TableName("t_photoer_audit")
public class PhotoerAudit extends BaseEntity {

    @ApiModelProperty("申请人标识")
    @NotEmpty
    @TableField(value = "account_id")
    private String accountId;

    @ApiModelProperty("真实姓名")
    @NotEmpty
    @TableField(value = "real_name")
    private String realName;

    @ApiModelProperty("申请人手机号码")
    @NotEmpty
    @TableField(value = "phonenumber")
    private String phonenumber;

    /* add by 梁展鹏 20210621 由于目前微信、支付宝还没能注册满90天，故不能使用平台转账到个人。目前暂时使用银联线下处理的方式 */
    @ApiModelProperty("申请人银行卡号")
    @NotEmpty
    @TableField(value = "bank_card_number")
    private String bankCardNumber;

    @ApiModelProperty("申请人身份证证件号码")
    @NotEmpty
    @TableField(value = "id_card_number")
    private String idCardNumber;

    @ApiModelProperty("申请人身份证正面（Minio上传）")
    @NotEmpty
    @TableField(value = "id_card_front")
    private String idCardFront;

    @ApiModelProperty("申请人身份证背面（Minio上传）")
    @NotEmpty
    @TableField(value = "id_card_back")
    private String idCardBack;

    @ApiModelProperty("申请人半身照（Minio上传）")
    @TableField(value = "body_photo")
    private String bodyPhoto;

    @ApiModelProperty("申请人邮箱")
    @NotEmpty
    @TableField(value = "email")
    private String email;

    @ApiModelProperty("常用相机型号")
    @NotEmpty(message = "常用相机型号不能为空")
    @TableField(value = "camera_model")
    private String cameraModel;

    @ApiModelProperty("经常合作场景")
    @NotEmpty(message = "经常合作场景不能为空")
    @TableField(value = "cooperation_scenario")
    private String cooperationScenario;

    @ApiModelProperty("每月可拍摄天数")
    @NotEmpty(message = "每月可拍摄天数不能为空")
    @TableField(value = "shooting_days_per_month")
    private String shootingDaysPerMonth;

    @ApiModelProperty("可能拍摄时间")
    @NotEmpty(message = "可能拍摄时间不能为空")
    @TableField(value = "possible_shooting_time")
    private String possibleShootingTime;

    @ApiModelProperty("个人拍摄经验")
    @NotEmpty(message = "个人拍摄经验不能为空")
    @TableField(value = "personal_shooting_experience")
    private String personalShootingExperience;

    @ApiModelProperty("作品集（Minio上传）")
    @NotEmpty(message = "作品集不能为空")
    @TableField(value = "sample_reels")
    private String sampleReels;

    @ApiModelProperty("审核账号标识")
    @TableField(value = "audit_account_id")
    private String auditAccountId;

    @ApiModelProperty("审核状态（1：审核通过、0：未审核、-1:审核失败）")
    @TableField(value = "audit_status")
    private Integer auditStatus;

    @ApiModelProperty("审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "audit_date")
    private LocalDateTime auditDate;

    @ApiModelProperty("审核备注")
    @TableField(value = "audit_msg")
    private String auditMsg;

    public static class AuditStatus {
        public static final Integer PASS = 1; // 审核通过
        public static final Integer WAIT = 0; // 未审核
        public static final Integer FAIL = -1; // 审核失败
    }
}
