package com.studio.nx.yxl.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.studio.nx.framework.config.json.BigDecimalSerialize;
import com.studio.nx.framework.entity.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@ApiModel
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@TableName("t_withdrawal_record")
public class WithdrawalRecord extends BaseEntity {

    @ApiModelProperty("账号标识")
    @NotEmpty
    @TableField(value = "account_id")
    private String accountId;

    @ApiModelProperty("申请人手机号码")
    @NotEmpty
    @TableField(value = "phonenumber")
    private String phonenumber;

    @ApiModelProperty("真实姓名")
    @NotEmpty
    @TableField(value = "real_name")
    private String realName;

    @ApiModelProperty("第三方平台标识")
    @NotEmpty
    @TableField(value = "transaction_account_id")
    private String transactionAccountId;

    @ApiModelProperty("渠道（微信、支付宝、银行卡）")
    @NotEmpty
    @TableField(value = "channels")
    private String channels;

    @ApiModelProperty("数值（可正可负）")
    @JsonSerialize(using = BigDecimalSerialize.class)
    @NotNull
    @TableField(value = "value")
    private BigDecimal value;

    @ApiModelProperty("第三方平台支付标识")
    @TableField(value = "transaction_id")
    private String transactionId;

    @ApiModelProperty("审核人标识")
    @TableField(value = "audit_account_id")
    private String auditAccountId;

    @ApiModelProperty("审核状态（1：审核通过、0：未审核、-1：审核失败）")
    @TableField(value = "audit_status")
    private Integer auditStatus;

    @ApiModelProperty("审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "audit_date")
    private LocalDateTime auditDate;

    @ApiModelProperty("审核备注")
    @TableField(value = "audit_msg")
    private String auditMsg;

    public static class AuditStatus {
        public static final Integer PASS = 1; // 审核通过
        public static final Integer WAIT = 0; // 未审核
        public static final Integer FAIL = -1; // 审核失败
    }
}
