package com.studio.nx.yxl.job;

import cn.hutool.json.JSONUtil;
import com.studio.nx.uiac.entity.AccountInvite;
import com.studio.nx.uiac.service.AccountInviteService;
import com.studio.nx.yxl.service.IntegralCashRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/20 14:40
 */
@Slf4j
@Component
public class AccountInviteRewardJob {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private AccountInviteService accountInviteService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    /**
     * 功能描述：登录奖励邀请人积分
     * 每5秒执行
     */
    @Scheduled(cron = "0/5 * * * * *")
    private void integralInviteToSignUp() {
        String json = stringRedisTemplate.opsForList().rightPop("yxl:integral-cash:integral_invite_to_sign_up");
        if (!StringUtils.hasText(json)) {
            return;
        }
        AccountInvite accountInvite = JSONUtil.toBean(json, AccountInvite.class);
        if (!Optional.ofNullable(accountInvite).isPresent()) {
            return;
        }

        integralCashRecordService.saveInvite(accountInvite.getAccountId(), accountInvite.getInviteAccountNickname());
        accountInviteService.setReward(accountInvite.getId());
    }

    /**
     * 功能描述：注册奖励积分
     * 每5秒执行
     */
    @Scheduled(cron = "0/5 * * * * *")
    public void integralToSignUp() {
        /* 注册奖励积分 */
        String accountId = stringRedisTemplate.opsForList().rightPop("yxl:integral-cash:integral_to_sign_up");
        if (StringUtils.hasText(accountId)) {
            integralCashRecordService.saveIntegralToSignUp(accountId);
        }
    }

}
