package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.controller.admin.search.ActivitySearchDto;
import com.studio.nx.yxl.dto.ActivityDto;
import com.studio.nx.yxl.entity.Activity;
import com.studio.nx.yxl.mapper.ActivityMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author loki
 * @Date 2021/6/24 10:01
 */
@Service
public class ActivityService {

    @Autowired
    private LocalService localService;

    @Autowired
    private ActivityMapper mapper;

    /**
     * 活动详情
     * @param id
     * @return
     */
    public Optional<Activity> findById(String id) {
        Activity _entity = mapper.selectById(id);
        if (!Optional.ofNullable(_entity).isPresent()) {
            return Optional.empty();
        }

        if (StringUtils.hasText(_entity.getCover())) {
            _entity.setCover(localService.fullPath(_entity.getCover()));
        }
        if (StringUtils.hasText(_entity.getPreviewLogo())) {
            _entity.setPreviewLogo(localService.fullPath(_entity.getPreviewLogo()));
        }
        if (StringUtils.hasText(_entity.getHdLogo())) {
            _entity.setHdLogo(localService.fullPath(_entity.getHdLogo()));
        }

        return Optional.ofNullable(_entity);
    }

    /**
     * 功能描述：查询[活动]列表
     */
    public BasePageResponse<ActivityDto> page(ActivitySearchDto dto, Integer page, Integer size) {
        IPage<ActivityDto> p = mapper.adminPage(dto,new Page(page, size));
        p.getRecords().stream().forEach(o -> {
            if (StringUtils.hasText(o.getCover())) {
                o.setCover(localService.fullPath(o.getCover()));
            }
            if (StringUtils.hasText(o.getPreviewLogo())) {
                o.setPreviewLogo(localService.fullPath(o.getPreviewLogo()));
            }
            if (StringUtils.hasText(o.getHdLogo())) {
                o.setHdLogo(localService.fullPath(o.getHdLogo()));
            }
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    /**
     * 功能描述：新增数据[活动]
     *
     * @param activityDto
     */
    public Activity createItem(ActivityDto activityDto) {
        Activity _activity = new Activity();
        _activity.setKindergartenId(activityDto.getKindergartenId());
        _activity.setName(activityDto.getName());
        _activity.setBrief(activityDto.getBrief());
        _activity.setEventStartDate(activityDto.getEventStartDate());
        _activity.setEventEndDate(activityDto.getEventEndDate());

        if (StringUtils.hasText(activityDto.getCover())) {
            _activity.setCover(localService.cutPath(activityDto.getCover()));
        }
        if (StringUtils.hasText(activityDto.getPreviewLogo())) {
            _activity.setPreviewLogo(localService.cutPath(activityDto.getPreviewLogo()));
        }
        if (StringUtils.hasText(activityDto.getHdLogo())) {
            _activity.setHdLogo(localService.cutPath(activityDto.getHdLogo()));
        }

        _activity.setSort(activityDto.getSort());
        _activity.setIsShow(activityDto.getIsShow());

        mapper.insert(_activity);
        return _activity;
    }

    /**
     * 功能描述：新增数据[活动]
     * @param id
     * @param activityDto
     * @return
     */
    public Activity updateItem(String id, ActivityDto activityDto) {
        Activity _activity = mapper.selectById(id);
        if (!Optional.ofNullable(_activity).isPresent()) {
            return null;
        }

        _activity.setKindergartenId(activityDto.getKindergartenId());
        _activity.setName(activityDto.getName());
        _activity.setBrief(activityDto.getBrief());
        _activity.setEventStartDate(activityDto.getEventStartDate());
        _activity.setEventEndDate(activityDto.getEventEndDate());

        if (StringUtils.hasText(activityDto.getCover())) {
            _activity.setCover(localService.cutPath(activityDto.getCover()));
        }
        if (StringUtils.hasText(activityDto.getPreviewLogo())) {
            _activity.setPreviewLogo(localService.cutPath(activityDto.getPreviewLogo()));
        }
        if (StringUtils.hasText(activityDto.getHdLogo())) {
            _activity.setHdLogo(localService.cutPath(activityDto.getHdLogo()));
        }

        _activity.setSort(activityDto.getSort());
        _activity.setIsShow(activityDto.getIsShow());

        mapper.updateById(_activity);
        return _activity;
    }

    /**
     * 功能描述：替换[活动]显示状态
     *
     * @param id
     */
    public void handleIsShow(String id) {
        Optional<Activity> optional = findById(id);
        if (optional.isPresent()) {
            Activity _activity = optional.get();
            _activity.setIsShow(!_activity.getIsShow());
            mapper.updateById(_activity);
        }
    }

    /**
     * 功能描述：替换[活动]显示状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        Optional<Activity> optional = findById(id);
        if (optional.isPresent()) {
            Activity _activity = optional.get();
            _activity.setIsDel(!_activity.getIsDel());
            mapper.updateById(_activity);
        }
    }

    public List<Activity> list(ActivitySearchDto dto) {
        LambdaQueryWrapper<Activity> wrapper = new LambdaQueryWrapper<Activity>()
                .eq(Activity::getIsShow, Boolean.TRUE)
                .eq(Activity::getIsDel, Boolean.FALSE)
                .orderByDesc(Activity::getSort)
                .orderByDesc(Activity::getCreateAt);

        if (dto.getKindergartenId() != null && dto.getKindergartenId() != "") {
            wrapper.eq(Activity::getKindergartenId, dto.getKindergartenId());
        }

        return mapper.selectList(wrapper);

    }
}
