package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.controller.admin.request.BannerSaveRequest;
import com.studio.nx.yxl.controller.admin.search.BannerSearchDto;
import com.studio.nx.yxl.entity.Banner;
import com.studio.nx.yxl.mapper.BannerMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/5/27 13:55
 */
@Slf4j
@Service
public class BannerService {

    @Autowired
    private LocalService localService;

    @Autowired
    private BannerMapper mapper;

    public Banner findById(String id) {
        Banner _entity = mapper.selectById(id);
        _entity.setImgUrl(localService.fullPath(_entity.getImgUrl()));
        return _entity;
    }

    @Cacheable(cacheNames = "yxl:banner", keyGenerator = "redisDefaultKeyGenerator")
    public List<Banner> cacheList() {
        return mapper.selectList(new LambdaQueryWrapper<Banner>()
                .eq(Banner::getIsShow, Boolean.TRUE)
                .eq(Banner::getIsDel, Boolean.FALSE)
                .orderByAsc(Banner::getType)
                .orderByDesc(Banner::getSort)
        );
    }

    public BasePageResponse<Banner> page(BannerSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Banner> wrapper = new LambdaQueryWrapper();
        wrapper.eq(Banner::getIsDel, Boolean.FALSE);
        if (null != dto.getIsShow()) {
            wrapper.eq(Banner::getIsShow, dto.getIsShow());
        }
        if (dto.getType() != null && dto.getType() != "") {
            wrapper.eq(Banner::getType, dto.getType());
        }

        wrapper.orderByAsc(Banner::getType);
        wrapper.orderByDesc(Banner::getSort);

        Page<Banner> p = mapper.selectPage(new Page(page, size), wrapper);
        List<Banner> list = p.getRecords().stream().map(o -> o.setImgUrl(localService.fullPath(o.getImgUrl()))).collect(Collectors.toList());

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    @CacheEvict(cacheNames = "yxl:banner", keyGenerator = "redisDefaultKeyGenerator")
    public void save(BannerSaveRequest request) {
        Banner _entity = new Banner();
        _entity.setName(request.getName());
        _entity.setType(request.getType());
        _entity.setImgUrl(localService.cutPath(request.getImgUrl()));
        _entity.setLink(request.getLink());
        _entity.setParams(request.getParams());
        _entity.setSort(request.getSort());
        _entity.setIsShow(request.getIsShow());
        mapper.insert(_entity);
    }

    @CacheEvict(cacheNames = "yxl:banner", keyGenerator = "redisDefaultKeyGenerator")
    public void update(String id, BannerSaveRequest request) {
        Optional<Banner> optional = Optional.ofNullable(findById(id));
        if (optional.isPresent()) {
            Banner _entity = optional.get();
            _entity.setName(request.getName());
            _entity.setType(request.getType());
            _entity.setImgUrl(localService.cutPath(request.getImgUrl()));
            _entity.setLink(request.getLink());
            _entity.setParams(request.getParams());
            _entity.setSort(request.getSort());
            _entity.setIsShow(request.getIsShow());
            mapper.updateById(_entity);
        }
    }

    @CacheEvict(cacheNames = "yxl:banner", keyGenerator = "redisDefaultKeyGenerator")
    public void handleIsShow(String id) {
        Optional<Banner> optional = Optional.ofNullable(findById(id));
        if (optional.isPresent()) {
            Banner _entity = optional.get();
            _entity.setIsShow(!_entity.getIsShow());
            mapper.updateById(_entity);
        }
    }

    @CacheEvict(cacheNames = "yxl:banner", keyGenerator = "redisDefaultKeyGenerator")
    public void handleIsDel(String id) {
        Optional<Banner> optional = Optional.ofNullable(findById(id));
        if (optional.isPresent()) {
            Banner _entity = optional.get();
            _entity.setIsDel(!_entity.getIsDel());
            mapper.updateById(_entity);
        }
    }

}
