package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.ResourcesConstant;
import com.studio.nx.yxl.controller.app.request.index.CollectionLikeMapStruct;
import com.studio.nx.yxl.controller.app.request.index.CollectionLikeRequest;
import com.studio.nx.yxl.controller.app.request.index.KindergartenSearchRequest;
import com.studio.nx.yxl.controller.app.response.KindergartenInfoResponse;
import com.studio.nx.yxl.controller.app.response.index.IndexKindergartenResponse;
import com.studio.nx.yxl.controller.app.response.index.PhotoWarehouseResponse;
import com.studio.nx.yxl.entity.Activity;
import com.studio.nx.yxl.entity.CollectionLike;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.entity.PhotoWarehouseMapStruct;
import com.studio.nx.yxl.mapper.ActivityMapper;
import com.studio.nx.yxl.mapper.CollectionLikeMapper;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/15 22:44
 */
@Service
public class IndexService {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private ResourcesConstant resourcesStatic;

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private CollectionLikeMapper collectionLikeMapper;

    @Autowired
    private ActivityMapper activityMapper;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private TweetService tweetService;

    /**
     * 赤道半径(单位m)
     */
    private static double EARTH_RADIUS = 6378137;


    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    public BasePageResponse<IndexKindergartenResponse> appPage(KindergartenSearchRequest request, Integer page, Integer size) {
        List<IndexKindergartenResponse> indexList = kindergartenMapper.appPage(request);

        //按照最新认证排序
        if (KindergartenSearchRequest.SortType.AUTH.equals(request.getSort())) {
            Collections.sort(indexList, (a, b) -> b.getCreateAt().compareTo(a.getCreateAt()));
        }

        //按照最多图片排序
        if (KindergartenSearchRequest.SortType.PIC.equals(request.getSort())) {
            Collections.sort(indexList, (a, b) -> b.getPicSum().compareTo(a.getPicSum()));
        }

        //按照最多活动排序
        if (KindergartenSearchRequest.SortType.ACT.equals(request.getSort())) {
            Collections.sort(indexList, (a, b) -> b.getActivitySum().compareTo(a.getActivitySum()));
        }

        //获取x公里范围内的数据
        if (ObjectUtils.isNotEmpty(request.getDistance()) && request.getDistance() > 0) {
            List<IndexKindergartenResponse> indexKindergartenResponses = new LinkedList<>();
            for (IndexKindergartenResponse _dto : indexList) {
                IndexKindergartenResponse indexKindergartenResponse = new IndexKindergartenResponse();
                //计算两个坐标距离
                double place = LantitudeLongitudeDist(request.getLng(), request.getLat(), _dto.getLng(), _dto.getLat());
                BeanUtils.copyProperties(_dto, indexKindergartenResponse);
                if (place / 1000 <= request.getDistance()) {
                    indexKindergartenResponses.add(_dto);
                }
            }
            indexList = indexKindergartenResponses;
        }

        // 代码过滤最终数据，使用lambda分页
        List<IndexKindergartenResponse> result = indexList.stream()
                .skip(size * (page >= 1 ? page - 1 : page))
                .limit(size)
                .collect(Collectors.toList());

        for (IndexKindergartenResponse response : result) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }

            String address = response.getProvince() + response.getCity() + response.getArea() + response.getAddress();
            response.setAddress(address);
        }

        return new BasePageResponse().setTotal(new Long((long) indexList.size())).setData(result);
    }

    /**
     * 计算两个坐标距离
     *
     * @param lon1
     * @param lat1
     * @param lon2
     * @param lat2
     * @return
     */
    private static double LantitudeLongitudeDist(double lon1, double lat1, double lon2, double lat2) {
        double radLat1 = rad(lat1);
        double radLat2 = rad(lat2);

        double radLon1 = rad(lon1);
        double radLon2 = rad(lon2);

        if (radLat1 < 0) {
            // south
            radLat1 = Math.PI / 2 + Math.abs(radLat1);
        }
        if (radLat1 > 0) {
            // north
            radLat1 = Math.PI / 2 - Math.abs(radLat1);
        }
        if (radLon1 < 0) {
            // west
            radLon1 = Math.PI * 2 - Math.abs(radLon1);
        }
        if (radLat2 < 0) {
            // south
            radLat2 = Math.PI / 2 + Math.abs(radLat2);
        }
        if (radLat2 > 0) {
            // north
            radLat2 = Math.PI / 2 - Math.abs(radLat2);
        }
        if (radLon2 < 0) {
            // west
            radLon2 = Math.PI * 2 - Math.abs(radLon2);
        }
        double x1 = EARTH_RADIUS * Math.cos(radLon1) * Math.sin(radLat1);
        double y1 = EARTH_RADIUS * Math.sin(radLon1) * Math.sin(radLat1);
        double z1 = EARTH_RADIUS * Math.cos(radLat1);

        double x2 = EARTH_RADIUS * Math.cos(radLon2) * Math.sin(radLat2);
        double y2 = EARTH_RADIUS * Math.sin(radLon2) * Math.sin(radLat2);
        double z2 = EARTH_RADIUS * Math.cos(radLat2);

        double d = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2) + (z1 - z2) * (z1 - z2));
        //余弦定理求夹角
        double theta = Math.acos((EARTH_RADIUS * EARTH_RADIUS + EARTH_RADIUS * EARTH_RADIUS - d * d) / (2 * EARTH_RADIUS * EARTH_RADIUS));
        double dist = theta * EARTH_RADIUS;
        return dist;
    }

    /**
     * 获取客户端幼儿园详情
     *
     * @param kindergartenId
     * @return
     */
    public KindergartenInfoResponse clientDetail(String kindergartenId, String accountId) {
        KindergartenInfoResponse response = null;
        if (StringUtils.hasText(accountId)) {
            response = kindergartenMapper.appDetail(kindergartenId, accountId);
        } else {
            response = kindergartenMapper.h5Detail(kindergartenId);
        }
        if (!Optional.ofNullable(response).isPresent()) {
            throw new BusinessException("数据异常");
        }

        // 获取最新的活动
        Activity activity = activityMapper.selectOne(new LambdaQueryWrapper<Activity>()
                .eq(Activity::getKindergartenId, kindergartenId)
                .eq(Activity::getIsShow, Boolean.TRUE)
                .eq(Activity::getIsDel, Boolean.FALSE)
                .orderByDesc(Activity::getSort)
                .orderByDesc(Activity::getCreateAt)
                .last("limit 1")
        );
        if (Optional.ofNullable(activity).isPresent()) {
            activity.setCover(localService.fullPath(activity.getCover()));
            response.setActivity(activity);
        }

        // 获取最新的10张图片
        List<PhotoWarehouse> photoWarehouses = photoWarehouseMapper.selectList(new LambdaQueryWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getKindergartenId, kindergartenId)
                .and(wrapper -> wrapper.eq(PhotoWarehouse::getActivityId, "").or().isNull(PhotoWarehouse::getActivityId))
                .eq(PhotoWarehouse::getIsReady, Boolean.TRUE)
                .eq(PhotoWarehouse::getIsShow, Boolean.TRUE)
                .eq(PhotoWarehouse::getAuditStatus, PhotoWarehouse.AuditStatus.PASS)
                .eq(PhotoWarehouse::getIsDel, Boolean.FALSE)
                .last("ORDER BY RAND() limit 10")
        );
        List<PhotoWarehouseResponse> list = new ArrayList<>();
        photoWarehouses.stream().forEach(o -> {
            PhotoWarehouseResponse photoWarehouseResponse = PhotoWarehouseMapStruct.INSTANCES.toListResponse(o);

            photoWarehouseResponse.setListPreview(localService.fullPath(photoWarehouseResponse.getListPreview()));
            photoWarehouseResponse.setDetailPreview(localService.fullPath(photoWarehouseResponse.getDetailPreview()));
            list.add(photoWarehouseResponse);
        });
        response.setPhotoWarehouses(list);

        List<String> coverList = new ArrayList<>();
        for (String cover : response.getCover().split(",")) {
            coverList.add(localService.fullPath(cover));
        }
        if (coverList.size() > 0) {
            response.setCover(coverList.get(0)); // 封面
        }
        response.setCoverList(coverList);

        response.setShareTitle(String.format("%s", response.getName())); // 分享-标题
        response.setShareDesc(String.format("%s", response.getBrief())); // 分享-简介
        response.setShareLink(String.format("%s/h5/kindergarten?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 分享-链接
        response.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图

        return response;
    }

    /**
     * 点赞/收藏
     */
    @Transactional
    public void collection(CollectionLikeRequest dto, String accountId) {
        List<CollectionLike> list = collectionLikeMapper.selectList(new LambdaQueryWrapper<CollectionLike>()
                .eq(CollectionLike::getAccountId, accountId)
                .eq(CollectionLike::getChannel, dto.getChannel())
                .eq(CollectionLike::getType, dto.getType())
                .eq(CollectionLike::getObjectId, dto.getObjectId())
        );
        if (list.size() > 0) {
            return;
        }

        CollectionLike entity = CollectionLikeMapStruct.INSTANCES.toEntity(dto);
        entity.setAccountId(accountId);
        collectionLikeMapper.insert(entity);

        /* 处理业务表 */
        if (dto.getType() == 0) { // 收藏
            switch (dto.getChannel()) {
                case 0: // 幼儿园
                    break;
                case 1: // 活动
                    break;
                case 2: // 资讯
                    tweetService.collectionsNum(dto.getObjectId(), Boolean.TRUE);
                    break;
            }
        } else if (dto.getType() == 1) { // 点赞
            switch (dto.getChannel()) {
                case 0: // 幼儿园
                    break;
                case 1: // 活动
                    break;
                case 2: // 资讯
                    tweetService.likesNum(dto.getObjectId(), Boolean.TRUE);
                    break;
            }
        }
    }

    /**
     * 取消点赞/收藏
     */
    @Transactional
    public void unCollection(CollectionLikeRequest dto, String accountId) {
        // 查询是否已有记录,有则直接更新状态
        collectionLikeMapper.delete(new LambdaQueryWrapper<CollectionLike>()
                .eq(CollectionLike::getAccountId, accountId)
                .eq(CollectionLike::getChannel, dto.getChannel())
                .eq(CollectionLike::getType, dto.getType())
                .eq(CollectionLike::getObjectId, dto.getObjectId())
        );

        /* 处理业务表 */
        if (dto.getType() == 0) { // 收藏
            switch (dto.getChannel()) {
                case 0: // 幼儿园
                    break;
                case 1: // 活动
                    break;
                case 2: // 资讯
                    tweetService.collectionsNum(dto.getObjectId(), Boolean.FALSE);
                    break;
            }
        } else if (dto.getType() == 1) { // 点赞
            switch (dto.getChannel()) {
                case 0: // 幼儿园
                    break;
                case 1: // 活动
                    break;
                case 2: // 资讯
                    tweetService.likesNum(dto.getObjectId(), Boolean.FALSE);
                    break;
            }
        }
    }

}
