package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.admin.response.IntegralCashRecordAdminPageResponse;
import com.studio.nx.yxl.controller.admin.search.IntegralCashRecordSearch;
import com.studio.nx.yxl.controller.app.request.my.wallet.WalletPageRequest;
import com.studio.nx.yxl.controller.app.response.my.IntegralCashRecordResponse;
import com.studio.nx.yxl.entity.IntegralCashRecord;
import com.studio.nx.yxl.entity.IntegralCashRecordMapStruct;
import com.studio.nx.yxl.mapper.IntegralCashRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/22 10:01
 */
@Service
public class IntegralCashRecordService {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private IntegralCashRecordMapper mapper;

    /**
     * 功能描述：注册获得积分
     *
     * @param accountId
     */
    public void saveIntegralToSignUp(String accountId) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_TO_SIGN_UP);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_TO_SIGN_UP); // 文案
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_TO_SIGN_UP))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：邀请拉新奖励
     *
     * @param accountId 邀请人
     * @param nickname  被邀请人
     */
    public void saveInvite(String accountId, String nickname) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_INVITE_TO_SIGN_UP);
        _entity.setDisplayText(nickname); // 文案
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_INVITE_TO_SIGN_UP))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：分享获得积分
     *
     * @param accountId 邀请人
     */
    public void saveIntegralSharePoints(String accountId) {
        /* 业务配置：分享获得积分每天限额次数 */
        Integer dayCount = Integer.valueOf(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SHARE_TIME_BY_DAY));

        String redisKey = String.format("yxl:integral:integral_share_time_by_day:%s", accountId);
        String redisCount = stringRedisTemplate.opsForValue().get(redisKey);
        if (!StringUtils.hasText(redisCount)) {
            redisCount = "0";
        }
        if (StringUtils.hasText(redisCount) && Integer.parseInt(redisCount) >= dayCount) {
            return;
        }

        // 计算当前离零时还有多少秒
        Long tomorrowSeconds = ChronoUnit.SECONDS.between(LocalDateTime.now(), LocalDateTime.of(LocalDateTime.now().plusDays(1).toLocalDate(), LocalTime.MIN));
        stringRedisTemplate.opsForValue().set(redisKey, String.valueOf(Integer.parseInt(redisCount) + 1), tomorrowSeconds, TimeUnit.SECONDS);

        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_SHARE_POINTS);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_SHARE_POINTS); // 文案
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SHARE_POINTS))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：点赞照片获得积分
     *
     * @param accountId 邀请人
     */
    public void saveLike(String accountId) {
        /* 业务配置：点赞照片获得积分每天限额次数 */
        Integer dayCount = Integer.valueOf(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_LIKE_TO_POINTS_TIME_BY_DAY));

        String redisKey = String.format("yxl:integral:integral_like_to_points_time_by_day:%s", accountId);
        String redisCount = stringRedisTemplate.opsForValue().get(redisKey);
        if (!StringUtils.hasText(redisCount)) {
            redisCount = "0";
        }
        if (StringUtils.hasText(redisCount) && Integer.parseInt(redisCount) >= dayCount) {
            return;
        }

        // 计算当前离零时还有多少秒
        Long tomorrowSeconds = ChronoUnit.SECONDS.between(LocalDateTime.now(), LocalDateTime.of(LocalDateTime.now().plusDays(1).toLocalDate(), LocalTime.MIN));
        stringRedisTemplate.opsForValue().set(redisKey, String.valueOf(Integer.parseInt(redisCount) + 1), tomorrowSeconds, TimeUnit.SECONDS);

        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_LIKE_TO_POINTS);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_LIKE_TO_POINTS);
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_LIKE_TO_POINTS))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：被点赞照片获得积分
     *
     * @param accountId 邀请人
     */
    public void saveLiked(String accountId) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_POINTS_BY_LIKE);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_POINTS_BY_LIKE); // 文案
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_POINTS_BY_LIKE))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：每日签到获得积分
     *
     * @param accountId 邀请人
     */
    public void saveIntegralSignInByDay(String accountId) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_SIGN_IN_BY_DAY);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_SIGN_IN_BY_DAY);
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SIGN_IN_BY_DAY))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：连续签到获得积分
     *
     * @param accountId 邀请人
     */
    public void saveIntegralSignInByEveryDayPoints(String accountId) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(IntegralCashRecord.Scene.INTEGRAL_SIGN_IN_BY_EVERY_DAY_POINTS);
        _entity.setDisplayText(IntegralCashRecord.Scene.INTEGRAL_SIGN_IN_BY_EVERY_DAY_POINTS);
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SIGN_IN_BY_EVERY_DAY_POINTS))); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：积分支付
     */
    public void savePayInvite(String accountId, String scene, String displayText, BigDecimal value) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(scene);
        _entity.setDisplayText(displayText); // 文案
        _entity.setType(IntegralCashRecord.Type.INTEGRAL); // 类型（积分/现金）
        _entity.setValue(value); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：提现现金
     */
    public void savePayCash(String accountId, String scene, String displayText, BigDecimal value) {
        IntegralCashRecord _entity = new IntegralCashRecord();
        _entity.setAccountId(accountId);
        _entity.setScene(scene);
        _entity.setDisplayText(displayText); // 文案
        _entity.setType(IntegralCashRecord.Type.CASH); // 类型（积分/现金）
        _entity.setValue(value); // 数值（可正可负）
        mapper.insert(_entity);
    }

    /**
     * 功能描述：邀请拉新奖励记录
     *
     * @param accountId
     * @param page
     * @param size
     * @return
     */
    public BasePageResponse<IntegralCashRecordResponse> pageInvite(String accountId, Integer page, Integer size) {
        Page<IntegralCashRecord> p = mapper.selectPage(new Page(page, size), new LambdaQueryWrapper<IntegralCashRecord>()
                .eq(IntegralCashRecord::getAccountId, accountId)
                .eq(IntegralCashRecord::getScene, IntegralCashRecord.Scene.INTEGRAL_INVITE_TO_SIGN_UP)
                .eq(IntegralCashRecord::getIsDel, Boolean.FALSE)
        );

        List<IntegralCashRecordResponse> list = new ArrayList();
        p.getRecords().stream().forEach(o -> {
            IntegralCashRecordResponse response = IntegralCashRecordMapStruct.INSTANCES.toResponse(o);

            if (IntegralCashRecord.Type.CASH.equals(o.getType())) { // 现金
                response.setShowValue(String.format("%s%s %s", o.getValue().intValue() > 0 ? "+" : "", o.getValue(), o.getType()));
            } else if (IntegralCashRecord.Type.INTEGRAL.equals(o.getType())) { // 积分（取整）
                response.setShowValue(String.format("%s%s %s", o.getValue().intValue() > 0 ? "+" : "", o.getValue().intValue(), o.getType()));
            }

            // todo
            response.setNickname("");
            response.setAvatarUrl("");
            list.add(response);
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    /**
     * 功能描述：统计邀请拉新个数
     *
     * @param accountId
     * @return
     */
    public Integer countInvite(String accountId) {
        return mapper.selectCount(new LambdaQueryWrapper<IntegralCashRecord>()
                .eq(IntegralCashRecord::getAccountId, accountId)
                .eq(IntegralCashRecord::getScene, IntegralCashRecord.Scene.INTEGRAL_INVITE_TO_SIGN_UP)
                .eq(IntegralCashRecord::getIsDel, Boolean.FALSE)
        );
    }

    public BasePageResponse<IntegralCashRecordAdminPageResponse> adminPage(IntegralCashRecordSearch search, Integer page, Integer size) {
        IPage<IntegralCashRecordAdminPageResponse> p = mapper.adminPage(search, new Page(page, size));

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    /**
     * 功能描述：获取我的可用积分
     *
     * @param accountId
     * @return
     */
    public Integer countMyIntegral(String accountId) {
        return mapper.getMyIntegral(accountId);
    }

    /**
     * 功能描述：获取我的可用现金
     *
     * @param accountId
     * @return
     */
    public BigDecimal countMyCash(String accountId) {
        return mapper.getMyCash(accountId);
    }

    /**
     * 功能描述：app查询[流水记录]列表
     */
    public BasePageResponse<IntegralCashRecordResponse> appPage(WalletPageRequest dto, String accountId, Integer page, Integer size) {
        LambdaQueryWrapper<IntegralCashRecord> lambdaQueryWrapper = new QueryWrapper().lambda();
        lambdaQueryWrapper.eq(IntegralCashRecord::getAccountId, accountId);
        if (null != dto.getType() && dto.getType() == 1) {
            lambdaQueryWrapper.eq(IntegralCashRecord::getType, IntegralCashRecord.Type.INTEGRAL);
        } else if (null != dto.getType() && dto.getType() == 2) {
            lambdaQueryWrapper.eq(IntegralCashRecord::getType, IntegralCashRecord.Type.CASH);
        }
        lambdaQueryWrapper.eq(IntegralCashRecord::getIsDel, false);
        lambdaQueryWrapper.orderByDesc(IntegralCashRecord::getCreateAt);

        List<IntegralCashRecordResponse> list = new ArrayList();
        Page<IntegralCashRecord> p = mapper.selectPage(new Page(page, size), lambdaQueryWrapper);
        p.getRecords().stream().forEach(o -> {
            IntegralCashRecordResponse response = IntegralCashRecordMapStruct.INSTANCES.toResponse(o);

            if (IntegralCashRecord.Type.CASH.equals(o.getType())) { // 现金
                response.setShowValue(String.format("%s%s %s", o.getValue().intValue() > 0 ? "+" : "", o.getValue(), o.getType()));
            } else if (IntegralCashRecord.Type.INTEGRAL.equals(o.getType())) { // 积分（取整）
                response.setShowValue(String.format("%s%s %s", o.getValue().intValue() > 0 ? "+" : "", o.getValue().intValue(), o.getType()));
            }

            // todo
            response.setNickname("");
            response.setAvatarUrl("");
            list.add(response);
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }
}
