package com.studio.nx.yxl.service;

import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.yxl.controller.app.response.order.OrderResponse;
import com.studio.nx.yxl.entity.IntegralCashRecord;
import com.studio.nx.yxl.entity.Order;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/29 09:10
 */
@Service
public class IntegralPayService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    public OrderResponse pay(String orderId, String accountId) {
        /* 判断该用户是否有这笔订单 */
        OrderResponse orderResponse = orderService.findByMyOrder(orderId, accountId);

        /* 查询用户是否盈余积分支付 */
        if (integralCashRecordService.countMyIntegral(accountId) < orderResponse.getIntegralPay()) {
            throw new BusinessException("您的积分余额不够支付。可以完成每日任务获取积分奖励。");
        }

        /* 扣减用户积分 */
        integralCashRecordService.savePayInvite(accountId, IntegralCashRecord.Scene.PAY_INTEGRAL, "购买照片", new BigDecimal(orderResponse.getIntegralPay()).negate());

        /* 更新订单状态以及订单项 */
        orderService.payToEnd(orderId, Order.PayChannels.INTEGRAL, "", orderResponse.getPayablePrice(), LocalDateTime.now());

        return orderService.findByMyOrder(orderId, accountId);
    }

}
