package com.studio.nx.yxl.service;

import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.extra.qrcode.QrConfig;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.notice.service.EmailService;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.oss.minio.service.MinioService;
import com.studio.nx.uiac.controller.response.RoleResponse;
import com.studio.nx.uiac.service.AccountRoleService;
import com.studio.nx.uiac.service.RoleService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.admin.request.KindergartenItemRequestDto;
import com.studio.nx.yxl.controller.admin.request.VerifyRequest;
import com.studio.nx.yxl.controller.app.request.my.apply.KindergartenApplySaveRequest;
import com.studio.nx.yxl.controller.app.response.my.applys.KindergartenApplySaveResponse;
import com.studio.nx.yxl.entity.Kindergarten;
import com.studio.nx.yxl.entity.KindergartenInfo;
import com.studio.nx.yxl.entity.KindergartenInfoMapStruct;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.mapper.KindergartenInfoMapper;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import javax.transaction.Transactional;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：幼儿园认证
 *
 * @Author 梁展鹏
 * @Date 2021/6/28 14:53
 */
@Slf4j
@Service
public class KindergartenApplyService {

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private LocalService localService;

    @Autowired
    private MinioService minioService;

    @Autowired
    private EmailService emailService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private AccountRoleService accountRoleService;

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private KindergartenService kindergartenService;

    @Autowired
    private KindergartenInfoMapper kindergartenInfoMapper;

    @Autowired
    private KindergartenInfoService kindergartenInfoService;

    public static final String KINDERGARTEN_APPLY_BUCKET_NAME = "kindergarten-apply";

    public KindergartenApplySaveResponse kindergartenGet(String accountId) {
        List<KindergartenInfo> kindergartenInfoList = kindergartenInfoService.findListByAccountId(accountId);

        /* 不存在时返回空 */
        if (kindergartenInfoList.size() == 0) {
            return null;
        }

        KindergartenApplySaveResponse dto = new KindergartenApplySaveResponse();

        KindergartenInfo _kindergartenInfoEntity = kindergartenInfoList.get(kindergartenInfoList.size() - 1);
//        dto.setKindergartenId(_kindergartenInfoEntity.getKindergartenId()); // 幼儿园标识
//        dto.setRealName(_kindergartenInfoEntity.getRealName()); // 申请人真实姓名
//        dto.setBusinessLicense(_kindergartenInfoEntity.getBusinessLicense()); // 营业执照
//        dto.setPersonPhonenumber(_kindergartenInfoEntity.getPhonenumber()); // 申请人手机号
//        dto.setIdCardNumber(_kindergartenInfoEntity.getIdCardNumber()); // 申请人身份证
//        dto.setIdCardFront(_kindergartenInfoEntity.getIdCardFront()); // 申请人证件正面
//        dto.setIdCardBack(_kindergartenInfoEntity.getIdCardBack()); // 申请人证件背面
//        dto.setBodyPhoto(_kindergartenInfoEntity.getBodyPhoto()); // 申请人半身照
//        dto.setAuditStatus(_kindergartenInfoEntity.getAuditStatus()); // 审核状态（1：审核通过、0：未审核、-1:审核失败）
//        dto.setAuditDate(_kindergartenInfoEntity.getAuditDate()); // 审核时间
//        dto.setAuditMsg(_kindergartenInfoEntity.getAuditMsg()); // 审核备注

        Optional<Kindergarten> kindergartenOptional = kindergartenService.findById(_kindergartenInfoEntity.getKindergartenId());
        if (kindergartenOptional.isPresent()) {
            Kindergarten _kindergartenEntity = kindergartenOptional.get();
//            dto.setName(_kindergartenEntity.getName()); // 幼儿园名称
//            dto.setLogo(_kindergartenEntity.getLogo()); // Logo
//            dto.setPhonenumber(_kindergartenEntity.getPhonenumber()); // 办公电话
//            dto.setBrief(_kindergartenEntity.getBrief()); // 简介
//            dto.setFrontDoor(_kindergartenEntity.getFrontDoor()); // 封面
//            dto.setType(_kindergartenEntity.getType()); // 类型(公办/民办)
//            dto.setProvince(_kindergartenEntity.getProvince()); // 省
//            dto.setCity(_kindergartenEntity.getCity()); // 市
//            dto.setArea(_kindergartenEntity.getArea()); // 区
//            dto.setAddress(_kindergartenEntity.getAddress()); // 详细地址

            dto = KindergartenInfoMapStruct.INSTANCES.toResponse(_kindergartenEntity, _kindergartenInfoEntity);
            dto.setLogo(localService.fullPath(_kindergartenEntity.getLogo())); // Logo
            List<String> coverList = new ArrayList<>();
            for (String cover : _kindergartenEntity.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                dto.setCover(coverList.get(0)); // 封面
            }
            dto.setCoverList(coverList); // 封面
            dto.setBusinessLicense(minioService.getPresignedObjectUrl(KINDERGARTEN_APPLY_BUCKET_NAME, minioService.cutFilename(KINDERGARTEN_APPLY_BUCKET_NAME, dto.getBusinessLicense()), 5, TimeUnit.MINUTES)); // 营业执照
            dto.setIdCardFront(minioService.getPresignedObjectUrl(KINDERGARTEN_APPLY_BUCKET_NAME, minioService.cutFilename(KINDERGARTEN_APPLY_BUCKET_NAME, dto.getIdCardFront()), 5, TimeUnit.MINUTES)); // 申请人证件正面
            dto.setIdCardBack(minioService.getPresignedObjectUrl(KINDERGARTEN_APPLY_BUCKET_NAME, minioService.cutFilename(KINDERGARTEN_APPLY_BUCKET_NAME, dto.getIdCardBack()), 5, TimeUnit.MINUTES)); // 申请人证件背面
            dto.setBodyPhoto(minioService.getPresignedObjectUrl(KINDERGARTEN_APPLY_BUCKET_NAME, minioService.cutFilename(KINDERGARTEN_APPLY_BUCKET_NAME, dto.getBodyPhoto()), 5, TimeUnit.MINUTES)); // 申请人半身照
        }

        return dto;
    }

    @Transactional
    public void appKindergartenSave(KindergartenApplySaveRequest dto, String accountId) {
        Optional<KindergartenInfo> optional = kindergartenInfoService.findInfoByAccountId(accountId);

        if (!optional.isPresent()) {
            kindergartenService.adminKindergartenSave(dto, accountId);
        }
    }

    @Transactional
    public void kindergartenUpdate(KindergartenApplySaveRequest dto, String accountId) {
        Optional<KindergartenInfo> optional = kindergartenInfoService.findInfoByAccountId(accountId);

        if (optional.isPresent()) {
            KindergartenInfo _kindergartenInfoEntity = optional.get();
            /* 如果是审核通过了，不允许更改资料 */
            if (PhotoerAudit.AuditStatus.PASS == _kindergartenInfoEntity.getAuditStatus()) {
                return;
            }

            KindergartenItemRequestDto requestDto = new KindergartenItemRequestDto();
            BeanUtils.copyProperties(dto, requestDto);
            kindergartenService.update(_kindergartenInfoEntity.getKindergartenId(), requestDto);
        }
    }

    /**
     * 功能描述：替换[幼儿园]审核状态
     *
     * @param id
     */
    @Transactional
    public void handleIsAudit(String id, VerifyRequest request, String accountId) {
        Optional<KindergartenInfo> optional = kindergartenInfoService.findById(id);
        if (!optional.isPresent()) {
            throw new BusinessException("幼儿园申请信息不存在");
        }

        KindergartenInfo _kindergartenInfo = optional.get();
        if (KindergartenInfo.AuditStatus.PASS == _kindergartenInfo.getAuditStatus()) {
            throw new BusinessException("幼儿园 已通过 审核。");
        }
        if (KindergartenInfo.AuditStatus.FAIL == _kindergartenInfo.getAuditStatus()) {
            throw new BusinessException("幼儿园 未能通过 审核，原因：" + _kindergartenInfo.getAuditMsg());
        }

        /* 一键审核后自动上架幼儿园 */
        kindergartenMapper.update(null, new LambdaUpdateWrapper<Kindergarten>()
                .set(Kindergarten::getIsShow, Boolean.TRUE)
                .eq(Kindergarten::getId, _kindergartenInfo.getKindergartenId())
        );
        kindergartenInfoMapper.update(null, new LambdaUpdateWrapper<KindergartenInfo>()
                .set(KindergartenInfo::getAuditAccountId, accountId)
                .set(KindergartenInfo::getAuditStatus, request.getAuditStatus())
                .set(KindergartenInfo::getAuditDate, LocalDateTime.now())
                .set(KindergartenInfo::getAuditMsg, request.getAuditMsg())
                .eq(KindergartenInfo::getId, id)
        );

        /* add by 梁展鹏 20210909 只有在审批通过才进行 */
        if (KindergartenInfo.AuditStatus.PASS == request.getAuditStatus()) {
            /* 幼儿园审核通过增加幼儿园角色 */
            Optional<RoleResponse> roleResponseDtoOptional = roleService.findByCode("kindergarten");
            if (roleResponseDtoOptional.isPresent()) {
                String photoerAccountId = _kindergartenInfo.getAccountId();
                String photoerRoleId = roleResponseDtoOptional.get().getId();
                accountRoleService.save(photoerAccountId, photoerRoleId);
            }

            /* 发送二维码邮件到申请人邮箱 */
            Kindergarten kindergarten = kindergartenMapper.selectById(_kindergartenInfo.getKindergartenId());
            sendKindergartenQrCodeEmail(kindergarten.getId(), kindergarten.getName(), _kindergartenInfo.getEmail());
        }
    }

    public void sendKindergartenQrCodeEmailByAccountId(String accountId) {
        List<KindergartenInfo> kindergartenInfoList = kindergartenInfoService.findListByAccountId(accountId);
        if (kindergartenInfoList.size() == 0) {
            return;
        }

        KindergartenInfo _kindergartenInfoEntity = kindergartenInfoList.get(kindergartenInfoList.size() - 1);
        /* 未能审核通过不发送二维码邮件 */
        if (PhotoerAudit.AuditStatus.PASS != _kindergartenInfoEntity.getAuditStatus()) {
            return;
        }
        Optional<Kindergarten> kindergartenOptional = kindergartenService.findById(_kindergartenInfoEntity.getKindergartenId());
        if (!kindergartenOptional.isPresent()) {
            return;
        }

        Kindergarten kindergarten = kindergartenOptional.get();

        /* 发送邮件 */
        sendKindergartenQrCodeEmail(kindergarten.getId(), kindergarten.getName(), _kindergartenInfoEntity.getEmail());
    }

    public void sendKindergartenQrCodeEmailBykindergartenId(String kindergartenId) {
        Optional<KindergartenInfo> optional = kindergartenInfoService.findInfoByKindergartenId(kindergartenId);
        if (!optional.isPresent()) {
            return;
        }
        KindergartenInfo _kindergartenInfoEntity = optional.get();
        /* 未能审核通过不发送二维码邮件 */
        if (PhotoerAudit.AuditStatus.PASS != _kindergartenInfoEntity.getAuditStatus()) {
            return;
        }
        Optional<Kindergarten> kindergartenOptional = kindergartenService.findById(_kindergartenInfoEntity.getKindergartenId());
        if (!kindergartenOptional.isPresent()) {
            return;
        }

        Kindergarten kindergarten = kindergartenOptional.get();

        /* 发送邮件 */
        sendKindergartenQrCodeEmail(kindergarten.getId(), kindergarten.getName(), _kindergartenInfoEntity.getEmail());
    }

    @SneakyThrows
    public void sendKindergartenQrCodeEmail(String kindergartenId, String kindergartenName, String email) {
        /* 生成幼儿园h5详情页的二维码 */
        String qrcodeBase64 = QrCodeUtil.generateAsBase64(String.format("%s/h5/kindergarten?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), kindergartenId),
                new QrConfig(800, 800)
                        .setImg(ImageIO.read(new ClassPathResource("static/yxl/kindergarten-qr-code-yxl-logo.png").getInputStream())),
                "jpg");

        /* 发送邮件 */
        emailService.sendTemplateMail(String.format("%s - 摄影师签到二维码", kindergartenName), new String[]{email}, new HashMap<String, Object>() {{
            put("qrCodeBase64", qrcodeBase64);
        }}, "mail/kindergarten-qr-code.html");
    }

}
