package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.oss.minio.service.MinioService;
import com.studio.nx.yxl.controller.admin.request.KindergartenItemRequestDto;
import com.studio.nx.yxl.controller.admin.response.KindergartenItemResponse;
import com.studio.nx.yxl.controller.admin.response.KindergartenListResponse;
import com.studio.nx.yxl.controller.admin.search.KindergartenSearch;
import com.studio.nx.yxl.controller.app.request.my.apply.KindergartenApplySaveRequest;
import com.studio.nx.yxl.entity.Kindergarten;
import com.studio.nx.yxl.entity.KindergartenInfo;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.mapper.KindergartenInfoMapper;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class KindergartenService {

    @Autowired
    private LocalService localService;

    @Autowired
    private MinioService minioService;

    @Autowired
    private KindergartenMapper mapper;

    @Autowired
    private KindergartenInfoMapper kindergartenInfoMapper;

    @Autowired
    private KindergartenInfoService kindergartenInfoService;

    /**
     * 幼儿园主表查询
     *
     * @param id
     * @return
     */
    public Optional<Kindergarten> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    /**
     * 功能描述：查询[幼儿园]详情
     *
     * @param id 标识
     * @return
     */
    public Optional<KindergartenItemResponse> findByInfo(String id) {
        KindergartenItemResponse infoDto = mapper.findByInfo(id);
        if (!Optional.ofNullable(infoDto).isPresent()) {
            return Optional.empty();
        }

        infoDto.setLogo(localService.fullPath(infoDto.getLogo()));
        infoDto.setPreviewLogo(localService.fullPath(infoDto.getPreviewLogo()));
        infoDto.setHdLogo(localService.fullPath(infoDto.getHdLogo()));
        List<String> coverList = new ArrayList<>();
        for (String cover : infoDto.getCover().split(",")) {
            coverList.add(localService.fullPath(cover));
        }
        if (coverList.size() > 0) {
            infoDto.setCover(coverList.get(0)); // 封面
        }
        infoDto.setCoverList(coverList);

        String bucketName = minioService.cutBucketName(infoDto.getBusinessLicense());
        infoDto.setBusinessLicense(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, infoDto.getBusinessLicense()), 5, TimeUnit.MINUTES));
        infoDto.setBodyPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, infoDto.getBodyPhoto()), 5, TimeUnit.MINUTES));
        infoDto.setIdCardFront(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, infoDto.getIdCardFront()), 5, TimeUnit.MINUTES));
        infoDto.setIdCardBack(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, infoDto.getIdCardBack()), 5, TimeUnit.MINUTES));

        return Optional.ofNullable(infoDto);
    }

    /**
     * 功能描述：查询[幼儿园]列表
     */
    public BasePageResponse<KindergartenListResponse> adminPage(KindergartenSearch dto, Integer page, Integer size) {
        IPage<KindergartenListResponse> p = mapper.adminPage(dto, new Page(page, size));
        p.getRecords().forEach(o -> {
            o.setLogo(localService.fullPath(o.getLogo()));
            o.setPreviewLogo(localService.fullPath(o.getPreviewLogo()));
            o.setHdLogo(localService.fullPath(o.getHdLogo()));
            List<String> coverList = new ArrayList<>();
            for (String cover : o.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                o.setCover(coverList.get(0)); // 封面
            }
            o.setCoverList(coverList);

            String bucketName = minioService.cutBucketName(o.getBusinessLicense());
            o.setBusinessLicense(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getBusinessLicense()), 5, TimeUnit.MINUTES));
            o.setBodyPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getBodyPhoto()), 5, TimeUnit.MINUTES));
            o.setIdCardFront(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardFront()), 5, TimeUnit.MINUTES));
            o.setIdCardBack(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardBack()), 5, TimeUnit.MINUTES));
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public List<KindergartenListResponse> list(KindergartenSearch search) {
        search.setAuditStatus(1);
        IPage<KindergartenListResponse> p = mapper.adminPage(search, new Page(1, 99999));
        p.getRecords().forEach(o -> {
            o.setLogo(localService.fullPath(o.getLogo()));
            o.setPreviewLogo(localService.fullPath(o.getPreviewLogo()));
            o.setHdLogo(localService.fullPath(o.getHdLogo()));
            List<String> coverList = new ArrayList<>();
            for (String cover : o.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                o.setCover(coverList.get(0)); // 封面
            }
            o.setCoverList(coverList);

            String bucketName = minioService.cutBucketName(o.getBusinessLicense());
            o.setBusinessLicense(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getBusinessLicense()), 5, TimeUnit.MINUTES));
            o.setBodyPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getBodyPhoto()), 5, TimeUnit.MINUTES));
            o.setIdCardFront(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardFront()), 5, TimeUnit.MINUTES));
            o.setIdCardBack(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardBack()), 5, TimeUnit.MINUTES));
        });

        return p.getRecords();
    }

    @Transactional
    public void adminKindergartenSave(KindergartenApplySaveRequest dto, String accountId) {
        Kindergarten _kindergartenEntity = new Kindergarten();
        _kindergartenEntity.setName(dto.getName()); // 幼儿园名称
        _kindergartenEntity.setLogo(localService.cutPath(dto.getLogo())); // Logo
        if (StringUtils.hasText(dto.getPreviewLogo())) {
            _kindergartenEntity.setPreviewLogo(localService.cutPath(dto.getPreviewLogo())); // 预览图Logo
        }
        if (StringUtils.hasText(dto.getHdLogo())) {
            _kindergartenEntity.setHdLogo(localService.cutPath(dto.getHdLogo())); // 高清图Logo
        }
        _kindergartenEntity.setPhonenumber(dto.getPhonenumber()); // 办公电话
        _kindergartenEntity.setBrief(dto.getBrief()); // 简介
        List<String> coverList = new ArrayList<>();
        for (String cover : dto.getCover().split(",")) {
            coverList.add(localService.cutPath(cover));
        }
        _kindergartenEntity.setCover(coverList.stream().collect(Collectors.joining(","))); // 封面
        _kindergartenEntity.setType(dto.getType()); // 类型(公办/民办)
        _kindergartenEntity.setProvince(dto.getProvince()); // 省
        _kindergartenEntity.setCity(dto.getCity()); // 市
        _kindergartenEntity.setArea(dto.getArea()); // 区
        _kindergartenEntity.setAddress(dto.getAddress()); // 详细地址
        _kindergartenEntity.setLng(0L);
        _kindergartenEntity.setLat(0L);
        _kindergartenEntity.setSort(0); // 排序（倒序）
        _kindergartenEntity.setIsShow(Boolean.FALSE); // 显示/隐藏
        mapper.insert(_kindergartenEntity);

        KindergartenInfo _kindergartenInfoEntity = new KindergartenInfo();
        _kindergartenInfoEntity.setKindergartenId(_kindergartenEntity.getId());
        _kindergartenInfoEntity.setAccountId(accountId);
        _kindergartenInfoEntity.setRealName(dto.getRealName()); // 真实姓名
        _kindergartenInfoEntity.setBusinessLicense(minioService.cutPath(kindergartenCopyFilename(dto.getBusinessLicense(), "营业执照", accountId))); // 营业执照
        _kindergartenInfoEntity.setPhonenumber(dto.getPersonPhonenumber()); // 申请人手机号码
        _kindergartenInfoEntity.setIdCardNumber(dto.getIdCardNumber()); // 申请人身份证证件号码
        _kindergartenInfoEntity.setIdCardFront(minioService.cutPath(kindergartenCopyFilename(dto.getIdCardFront(), "申请人身份证正面", accountId))); // 申请人身份证正面
        _kindergartenInfoEntity.setIdCardBack(minioService.cutPath(kindergartenCopyFilename(dto.getIdCardBack(), "申请人身份证背面", accountId))); // 申请人身份证背面
        _kindergartenInfoEntity.setBodyPhoto(minioService.cutPath(kindergartenCopyFilename(dto.getBodyPhoto(), "申请人半身照", accountId))); // 申请人半身照

        /* add by 梁展鹏 20210807 开发调试临时处理 */
//        _kindergartenInfoEntity.setBusinessLicense(minioService.cutPath(dto.getBusinessLicense())); // 营业执照
//        _kindergartenInfoEntity.setIdCardFront(minioService.cutPath(dto.getIdCardFront())); // 申请人身份证正面
//        _kindergartenInfoEntity.setIdCardBack(minioService.cutPath(dto.getIdCardBack())); // 申请人身份证背面
//        _kindergartenInfoEntity.setBodyPhoto(minioService.cutPath(dto.getBodyPhoto())); // 申请人半身照

        _kindergartenInfoEntity.setEmail(dto.getEmail()); // 申请人邮箱
        _kindergartenInfoEntity.setAuditStatus(PhotoerAudit.AuditStatus.WAIT); // 审核状态（1：审核通过、0：未审核、-1:审核失败）
        kindergartenInfoMapper.insert(_kindergartenInfoEntity);
    }

    private String kindergartenCopyFilename(String fullPath, String text, String accountId) {
        /* 1. 文件复制到指定业务目录 */
        String relativePath = minioService.cutPath(fullPath);
        String bucketName = minioService.cutBucketName(relativePath);
        String filename = minioService.cutFilename(bucketName, relativePath);
        String newBucketName = KindergartenApplyService.KINDERGARTEN_APPLY_BUCKET_NAME;
        String newFilename = String.format("%s/%s", accountId, filename);

        if (!minioService.isExists(bucketName, filename)) {
            throw new BusinessException(String.format("%s 文件不存在，请重新上传", text));
        }

        minioService.copyObject(bucketName, filename, newBucketName, newFilename);

        /* 2. 删除临时目录的文件 */
        if (minioService.isExists(newBucketName, newFilename)) {
            minioService.removeObject(bucketName, filename);
        }

        return minioService.getPresignedObjectUrl(newBucketName, newFilename, 5, TimeUnit.MINUTES);
    }

    @Transactional
    public void update(String kindergartenId, KindergartenItemRequestDto dto) {
        Optional<Kindergarten> kindergartenOptional = findById(kindergartenId);
        if (kindergartenOptional.isPresent()) {
            Kindergarten _entity = kindergartenOptional.get();
            _entity.setName(dto.getName());
            if (StringUtils.hasText(dto.getLogo())) {
                _entity.setLogo(localService.cutPath(dto.getLogo()));
            }
            if (StringUtils.hasText(dto.getPreviewLogo())) {
                _entity.setPreviewLogo(localService.cutPath(dto.getPreviewLogo())); // 预览图Logo
            }
            if (StringUtils.hasText(dto.getHdLogo())) {
                _entity.setHdLogo(localService.cutPath(dto.getHdLogo())); // 高清图Logo
            }

            _entity.setPhonenumber(dto.getPhonenumber());
            _entity.setBrief(dto.getBrief());
            List<String> coverList = new ArrayList<>();
            for (String cover : dto.getCover().split(",")) {
                coverList.add(localService.cutPath(cover));
            }
            _entity.setCover(coverList.stream().collect(Collectors.joining(","))); // 封面
            _entity.setType(dto.getType());
            _entity.setProvince(dto.getProvince());
            _entity.setCity(dto.getCity());
            _entity.setArea(dto.getArea());
            _entity.setAddress(dto.getAddress());
            _entity.setSort(dto.getSort());
            _entity.setIsShow(dto.getIsShow());
            mapper.updateById(_entity);
        }

        Optional<KindergartenInfo> kindergartenInfoOptional = kindergartenInfoService.findByKindergartenId(kindergartenId);
        if (kindergartenInfoOptional.isPresent()) {
            KindergartenInfo _entity = kindergartenInfoOptional.get();
            _entity.setRealName(dto.getRealName());
            _entity.setBusinessLicense(minioService.cutPath(dto.getBusinessLicense()));
            _entity.setPhonenumber(dto.getPersonPhonenumber());
            _entity.setIdCardNumber(dto.getIdCardNumber());
            _entity.setIdCardFront(minioService.cutPath(dto.getIdCardFront()));
            _entity.setIdCardBack(minioService.cutPath(dto.getIdCardBack()));
            _entity.setBodyPhoto(minioService.cutPath(dto.getBodyPhoto()));
            _entity.setEmail(dto.getEmail());
            kindergartenInfoMapper.updateById(_entity);
        }
    }

    /**
     * 功能描述：替换[幼儿园]显示状态
     *
     * @param id
     */
    public void handleIsShow(String id) {
        Optional<Kindergarten> optional = findById(id);
        if (optional.isPresent()) {
            Kindergarten _kindergarten = optional.get();
            _kindergarten.setIsShow(!_kindergarten.getIsShow());
            mapper.updateById(_kindergarten);
        }
    }

    /**
     * 功能描述：替换[幼儿园]删除状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        Optional<Kindergarten> optional = findById(id);
        if (optional.isPresent()) {
            Kindergarten _kindergarten = optional.get();
            _kindergarten.setIsDel(!_kindergarten.getIsDel());
            mapper.updateById(_kindergarten);
        }
    }

}
