package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.pay.weixin.dto.PaySuccessNotifyResponse;
import com.studio.nx.pay.weixin.service.WeixinPayAppService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.admin.response.OrderPageResponse;
import com.studio.nx.yxl.controller.admin.search.OrderSearch;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ShoppingCarAddRequest;
import com.studio.nx.yxl.controller.app.response.order.OrderResponse;
import com.studio.nx.yxl.entity.*;
import com.studio.nx.yxl.mapper.OrderItemMapper;
import com.studio.nx.yxl.mapper.OrderMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import com.studio.nx.yxl.mapper.ShoppingCarMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZonedDateTime;
import java.util.*;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/28 15:17
 */
@Slf4j
@Service
public class OrderService {

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private ShoppingCarMapper shoppingCarMapper;

    @Autowired
    private ShoppingCarService shoppingCarService;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderItemMapper orderItemMapper;

    @Autowired
    private WeixinPayAppService weixinPayAppService;

    @Autowired
    private PhotoAccountRecordService photoAccountRecordService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    /**
     * 后台查询订单列表分页
     *
     * @param dto
     * @param page
     * @param size
     * @return
     */
    public BasePageResponse<OrderPageResponse> page(OrderSearch dto, Integer page, Integer size) {
        IPage<OrderPageResponse> p = orderMapper.adminPage(dto, new Page(page, size));

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public OrderResponse generateByPhoto(List<ShoppingCarAddRequest> list, String accountId) {
        List<ShoppingCar> shoppingCarList = new ArrayList<>();
        list.stream().forEach(o -> {
            shoppingCarList.add(shoppingCarService.addShoppingCar(o, accountId));
        });

        /* 根据购物车对象，生成订单 */
        return generateByShoppingCar(shoppingCarList, accountId);
    }

    public OrderResponse generateByShoppingCarId(List<String> shoppingCarIdList, String accountId) {
        List<ShoppingCar> shoppingCarList = new ArrayList<>();
        shoppingCarIdList.stream().forEach(o -> {
            ShoppingCar _shoppingCar = shoppingCarMapper.selectOne(new LambdaQueryWrapper<ShoppingCar>()
                    .eq(ShoppingCar::getId, o)
                    .eq(ShoppingCar::getAccountId, accountId)
            );

            shoppingCarList.add(_shoppingCar);
        });

        return generateByShoppingCar(shoppingCarList, accountId);
    }

    @Transactional
    private OrderResponse generateByShoppingCar(List<ShoppingCar> shoppingCarList, String accountId) {
        BigDecimal originPriceOrder = new BigDecimal("0"); // 订单原价
        BigDecimal payablePriceOrder = new BigDecimal("0"); // 订单应付 todo 目前没有考虑会员情况

        /**
         * 1. 查询照片对象
         * 2. 生成订单项
         * 3. 计算订单的总应付
         */
        List<OrderItem> orderItemList = new ArrayList<>();
        List<String> myShoppingCarList = new ArrayList<>(); // 我的购物车记录
        for (ShoppingCar _shoppingCar : shoppingCarList) {
            myShoppingCarList.add(_shoppingCar.getId());
            String photoWarehouseId = _shoppingCar.getPhotoWarehouseId();
            PhotoWarehouse photoWarehouse = photoWarehouseMapper.selectById(photoWarehouseId);

            OrderItem _orderItem = new OrderItem();
            _orderItem.setPhotoWarehouseId(_shoppingCar.getPhotoWarehouseId()); // 照片库标识
            if (_shoppingCar.getIsHdLogoPhoto()) { // 高清图
                _orderItem.setIsHdLogoPhoto(Boolean.TRUE); // 是否购买高清带logo
                _orderItem.setIsOriginPhoto(Boolean.FALSE); // 是否购买原图
                _orderItem.setOriginPrice(photoWarehouse.getHdLogoPhotoPrice()); // 原价
                _orderItem.setPayablePrice(photoWarehouse.getHdLogoPhotoPrice()); // 应付 todo 目前没有考虑会员情况

                originPriceOrder = originPriceOrder.add(photoWarehouse.getHdLogoPhotoPrice());
                payablePriceOrder = payablePriceOrder.add(photoWarehouse.getHdLogoPhotoPrice());
            } else if (_shoppingCar.getIsOriginPhoto()) { // 原图
                _orderItem.setIsHdLogoPhoto(Boolean.FALSE); // 是否购买高清带logo
                _orderItem.setIsOriginPhoto(Boolean.TRUE); // 是否购买原图
                _orderItem.setOriginPrice(photoWarehouse.getOriginPhotoPrice()); // 原价
                _orderItem.setPayablePrice(photoWarehouse.getOriginPhotoPrice()); // 应付 todo 目前没有考虑会员情况

                originPriceOrder = originPriceOrder.add(photoWarehouse.getOriginPhotoPrice());
                payablePriceOrder = payablePriceOrder.add(photoWarehouse.getOriginPhotoPrice());
            }
            orderItemList.add(_orderItem);
        }

        /* 生成订单 */
        Order _order = new Order();
        _order.setAccountId(accountId); // 账号标识
        _order.setOriginPrice(originPriceOrder); // 订单原价
        _order.setPayablePrice(payablePriceOrder); // 订单应付
        _order.setPaidPrice(null); // 订单实付 等第三方平台回调为准 todo 目前没有考虑会员情况
        _order.setPayChannels(""); // 支付渠道（积分、微信、支付宝） 等支付页面确定
        _order.setState(Order.State.WAIT_PAY); // 订单状态
        _order.setPayEndTime(null); // 支付时间 等第三方平台回调为准
        _order.setTransactionId(""); // 第三方平台支付标识
        orderMapper.insert(_order);

        /* 生成订单项 */
        for (OrderItem _orderItem : orderItemList) {
            _orderItem.setOrderId(_order.getId());
            orderItemMapper.insert(_orderItem);
        }

        /* 删除购物车类型 */
        if (!CollectionUtils.isEmpty(myShoppingCarList)) {
            shoppingCarService.deleteByShoppingCarId(myShoppingCarList, accountId);
        }

        OrderResponse response = OrderMapStruct.INSTANCES.toResponse(_order);
        response.setDiscount("无优惠"); // 暂无会员场景
        response.setIntegralPay(response.getPayablePrice().multiply(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_CASH_OUT))).intValue());
        response.setOrderItemResponseList(OrderItemMapStruct.INSTANCES.toResponseList(orderItemList));
        return response;
    }

    public OrderResponse findByMyOrder(String orderId, String accountId) {
        Order _order = orderMapper.selectOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getId, orderId)
                .eq(Order::getAccountId, accountId)
        );
        if (!Optional.ofNullable(_order).isPresent()) {
            throw new BusinessException("没有该订单");
        }

        /* 查询订单项 */
        List<OrderItem> orderItemList = orderItemMapper.selectList(new LambdaQueryWrapper<OrderItem>()
                .eq(OrderItem::getOrderId, orderId)
        );

        OrderResponse response = OrderMapStruct.INSTANCES.toResponse(_order);
        response.setDiscount("无优惠"); // 暂无会员场景
        response.setIntegralPay(response.getPayablePrice().multiply(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_CASH_OUT))).intValue());
        response.setOrderItemResponseList(OrderItemMapStruct.INSTANCES.toResponseList(orderItemList));
        return response;
    }

    /**
     * 功能描述：订单支付完成
     * 1. 更新订单业务表数据
     * 2. 更新订单项业务数据（已购买）
     *
     * @param orderId       订单标识
     * @param payChannels   支付渠道
     * @param transactionId 第三方支付凭证
     * @param paidPrice     实付金额（第三饭支付回来）
     * @param payTimeEnd    支付完成时间
     */
    @Transactional
    public void payToEnd(String orderId, String payChannels, String transactionId, BigDecimal paidPrice, LocalDateTime payTimeEnd) {
        Order _order = orderMapper.selectById(orderId);
        if (!Optional.ofNullable(_order).isPresent()) {
            throw new BusinessException("没有该订单");
        }
        /* 当前订单状态非待支付，则返回 */
        if (Order.State.WAIT_PAY != _order.getState()) {
            return;
        }

        _order.setPayChannels(payChannels); // 支付渠道（积分、微信、支付宝） 等支付页面确定
        _order.setState(Order.State.SUCCESS_PAY); // 订单状态
        _order.setTransactionId(transactionId); // 第三方平台支付标识
        _order.setPaidPrice(paidPrice); // 订单实付
        _order.setPayEndTime(payTimeEnd); // 支付时间
        orderMapper.updateById(_order);

        /* 更新订单项业务数据（已购买） */
        List<OrderItem> orderItemList = orderItemMapper.selectList(new LambdaQueryWrapper<OrderItem>()
                .eq(OrderItem::getOrderId, orderId)
        );
        orderItemList.stream().forEach(o -> {
            if (o.getIsHdLogoPhoto()) {
                photoAccountRecordService.setIsHdLogoPhoto(_order.getAccountId(), o.getPhotoWarehouseId());
            }
            if (o.getIsOriginPhoto()) {
                photoAccountRecordService.setIsOriginPhoto(_order.getAccountId(), o.getPhotoWarehouseId());
            }
        });

        // 奖励摄影师(积分/现金)
        rewardPhotoer(orderId);
    }

    @Transactional
    public void rewardPhotoer(String orderId) {
        Order _order = orderMapper.selectById(orderId);
        if (!Optional.ofNullable(_order).isPresent()) {
            throw new BusinessException("没有该订单");
        }
        /* 当前订单状态非待支付，则返回 */
        if (Order.State.SUCCESS_PAY != _order.getState()) {
            return;
        }

        /* 计算订单的所有照片摄影师的价格总和 */
        Map<String, BigDecimal> map = new HashMap<>();
        List<OrderItem> orderItemList = orderItemMapper.selectList(new LambdaQueryWrapper<OrderItem>()
                .eq(OrderItem::getOrderId, orderId)
        );
        orderItemList.stream().forEach(o -> {
            String photoWarehouseId = o.getPhotoWarehouseId(); // 照片标识
            String accountId = photoWarehouseMapper.selectById(photoWarehouseId).getAccountId();
            BigDecimal payablePrice = o.getPayablePrice(); // 照片实付

            if (Optional.ofNullable(map.get(accountId)).isPresent()) {
                map.put(accountId, map.get(accountId).add(payablePrice));
            } else {
                map.put(accountId, payablePrice);
            }
        });

        BigDecimal integralCashOut = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_CASH_OUT)); // 积分兑现金
        BigDecimal cashPhotoerIncome = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_PHOTOER_INCOME)); // 摄影师照片提成
        map.forEach((key, value) -> {
            String accountId = key;
            if (Arrays.asList(Order.PayChannels.WEIXIN, Order.PayChannels.ALIPAY).indexOf(_order.getPayChannels()) >= 0) { // 现金支付
                integralCashRecordService.savePayCash(accountId, IntegralCashRecord.Scene.BUG_PHOTO_COMMISSION_CASH, "购买照片提成现金", value.multiply(cashPhotoerIncome).divide(new BigDecimal("100")));
            } else { // 积分支付
                integralCashRecordService.savePayInvite(accountId, IntegralCashRecord.Scene.BUG_PHOTO_COMMISSION_INTEGRAL, "购买照片提成积分", value.multiply(integralCashOut));
            }
        });
    }

    /**
     * 功能描述：定时请求未关单的订单数
     *
     * @return
     */
    public void queryOrderByJob() throws Exception {
        /* 查询待支付的订单 */
        List<Order> waitPayOrderList = orderMapper.selectList(new LambdaQueryWrapper<Order>()
                .eq(Order::getPayChannels, Order.PayChannels.WEIXIN)
                .eq(Order::getState, Order.State.WAIT_PAY)
        );

        for (Order order : waitPayOrderList) {
            queryOrderByOrderId(order.getId());
        }
    }

    @Transactional
    public Boolean queryOrderByOrderId(String orderId) throws Exception {
        PaySuccessNotifyResponse payResponse = weixinPayAppService.queryOrderByOrderId(orderId);
        if (!com.alibaba.druid.util.StringUtils.equals(payResponse.getTrade_state(), "SUCCESS")) {
            log.error("微信支付 - 查询订单。支付异常。参数：[{}]", payResponse.toString());
            return Boolean.FALSE;
        }

        paySuccessToEnd(payResponse);
        return Boolean.TRUE;
    }

    public void paySuccessToEnd(PaySuccessNotifyResponse payResponse) {
        /* 处理订单为支付成功 */
        BigDecimal paidPrice = BigDecimal.valueOf(payResponse.getAmount().getPayer_total()).divide(new BigDecimal(100)); // 订单金额(分转元)

        ZonedDateTime zonedDateTime = ZonedDateTime.parse(payResponse.getSuccess_time());
        payToEnd(payResponse.getOut_trade_no(), Order.PayChannels.WEIXIN, payResponse.getTransaction_id(), paidPrice, zonedDateTime.toLocalDateTime());
    }

}
