package com.studio.nx.yxl.service;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.yxl.controller.admin.search.DownloadPageSearch;
import com.studio.nx.yxl.entity.PhotoAccountRecord;
import com.studio.nx.yxl.mapper.PhotoAccountRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/11 17:01
 */
@Service
public class PhotoAccountRecordService {

    @Autowired
    private PhotoAccountRecordMapper mapper;

    private Optional<PhotoAccountRecord> findById(String accountId, String photoWarehouseId) {
        return Optional.ofNullable(
                mapper.selectOne(new LambdaQueryWrapper<PhotoAccountRecord>()
                        .eq(PhotoAccountRecord::getAccountId, accountId)
                        .eq(PhotoAccountRecord::getPhotoWarehouseId, photoWarehouseId)
                        .eq(PhotoAccountRecord::getIsDel, Boolean.FALSE)
                ));
    }

    public Boolean isLikes(String accountId, String photoWarehouseId) {
        return findById(accountId, photoWarehouseId).map(o -> o.getIsLikes()).orElse(Boolean.FALSE);
    }

    public Boolean isCollections(String accountId, String photoWarehouseId) {
        return findById(accountId, photoWarehouseId).map(o -> o.getIsCollections()).orElse(Boolean.FALSE);
    }

    public Boolean isHdLogoPhoto(String accountId, String photoWarehouseId) {
        return findById(accountId, photoWarehouseId).map(o -> o.getIsHdLogoPhoto()).orElse(Boolean.FALSE);
    }

    public Boolean isOriginPhoto(String accountId, String photoWarehouseId) {
        return findById(accountId, photoWarehouseId).map(o -> o.getIsOriginPhoto()).orElse(Boolean.FALSE);
    }

    public void save(String accountId, String photoWarehouseId) {
        String id = IdUtil.createSnowflake(1L, 1L).nextIdStr();
        mapper.saveOrUpdate(id, accountId, photoWarehouseId);
    }

    public void setIsLikes(String accountId, String photoWarehouseId) {
        save(accountId, photoWarehouseId);

        Optional<PhotoAccountRecord> optional = findById(accountId, photoWarehouseId);
        if (optional.isPresent()) {
            PhotoAccountRecord record = optional.get();
            record.setIsLikes(!record.getIsLikes());
            mapper.updateById(record);
        }
    }

    public void setIsCollections(String accountId, String photoWarehouseId) {
        save(accountId, photoWarehouseId);

        Optional<PhotoAccountRecord> optional = findById(accountId, photoWarehouseId);
        if (optional.isPresent()) {
            PhotoAccountRecord record = optional.get();
            record.setIsCollections(!record.getIsCollections());
            mapper.updateById(record);
        }
    }

    public void setIsHdLogoPhoto(String accountId, String photoWarehouseId) {
        save(accountId, photoWarehouseId);

        Optional<PhotoAccountRecord> optional = findById(accountId, photoWarehouseId);
        if (optional.isPresent()) {
            PhotoAccountRecord record = optional.get();
            record.setIsHdLogoPhoto(!record.getIsHdLogoPhoto());
            mapper.updateById(record);
        }
    }

    public void setIsOriginPhoto(String accountId, String photoWarehouseId) {
        save(accountId, photoWarehouseId);

        Optional<PhotoAccountRecord> optional = findById(accountId, photoWarehouseId);
        if (optional.isPresent()) {
            PhotoAccountRecord record = optional.get();
            record.setIsOriginPhoto(!record.getIsOriginPhoto());
            mapper.updateById(record);
        }
    }

    public Integer countByIsHdLogoPhoto(DownloadPageSearch search) {
        LambdaQueryWrapper<PhotoAccountRecord> wrapper = new LambdaQueryWrapper();
        wrapper.eq(PhotoAccountRecord::getIsHdLogoPhoto, Boolean.TRUE);
        if (StringUtils.hasText(search.getAccountId())) {
            wrapper.eq(PhotoAccountRecord::getAccountId, search.getAccountId());
        }
        if (null != search.getStartTime() && null != search.getEndTime()) {
            wrapper.between(PhotoAccountRecord::getCreateAt, search.getStartTime(), search.getEndTime());
        }

        return mapper.selectCount(wrapper);
    }

    public Integer countByIsOriginPhoto(DownloadPageSearch search) {
        LambdaQueryWrapper<PhotoAccountRecord> wrapper = new LambdaQueryWrapper();
        wrapper.eq(PhotoAccountRecord::getIsOriginPhoto, Boolean.TRUE);
        if (StringUtils.hasText(search.getAccountId())) {
            wrapper.eq(PhotoAccountRecord::getAccountId, search.getAccountId());
        }
        if (null != search.getStartTime() && null != search.getEndTime()) {
            wrapper.between(PhotoAccountRecord::getCreateAt, search.getStartTime(), search.getEndTime());
        }

        return mapper.selectCount(wrapper);
    }

}
