package com.studio.nx.yxl.service;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.studio.nx.yxl.entity.PhotoGetLikeRecord;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.mapper.PhotoGetLikeRecordMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/11 17:01
 */
@Service
public class PhotoGetLikeRecordService {

    @Autowired
    private PhotoGetLikeRecordMapper mapper;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    public void like(String accountId, String nickname, String photoWarehouseId) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(photoWarehouseMapper.selectById(photoWarehouseId));
        if (optional.isPresent()) {
            String originAccountId = optional.get().getAccountId();

            /* 判断当前用户是否首次点赞 */
            Integer count = mapper.selectCount(new LambdaQueryWrapper<PhotoGetLikeRecord>()
                    .eq(PhotoGetLikeRecord::getAccountId, accountId)
                    .eq(PhotoGetLikeRecord::getPhotoWarehouseId, photoWarehouseId)
            );
            if (count == 0) {
                /* 点赞奖励 */
                integralCashRecordService.saveLike(accountId);

                /* 被点赞奖励 */
                integralCashRecordService.saveLiked(originAccountId);
            }

            String id = IdUtil.createSnowflake(1L, 1L).nextIdStr();
            mapper.like(id, accountId, nickname, photoWarehouseId, originAccountId);
        }
    }

    public void delLike(String accountId, String photoWarehouseId) {
        mapper.update(null, new LambdaUpdateWrapper<PhotoGetLikeRecord>()
                .eq(PhotoGetLikeRecord::getAccountId, accountId)
                .eq(PhotoGetLikeRecord::getPhotoWarehouseId, photoWarehouseId)
                .set(PhotoGetLikeRecord::getUpdateBy, accountId)
                .set(PhotoGetLikeRecord::getUpdateAt, LocalDateTime.now())
                .set(PhotoGetLikeRecord::getIsDel, Boolean.TRUE)
        );
    }

    /**
     * 功能描述：获取我的获赞统计数
     *
     * @param accountId
     * @return
     */
    public Integer countMyphotoGetLike(String accountId) {
        return mapper.countMyphotoGetLike(accountId);
    }

}
