package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.ResourcesConstant;
import com.studio.nx.yxl.controller.app.request.photowall.PhotoCollectionLikeRequest;
import com.studio.nx.yxl.controller.app.request.photowall.activity.PhotoWallActivityRequest;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityDayListResponse;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityPhotoListResponse;
import com.studio.nx.yxl.controller.app.response.photowall.activity.ActivityResponse;
import com.studio.nx.yxl.entity.PhotoAccountRecord;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.entity.PhotoWarehouseMapStruct;
import com.studio.nx.yxl.mapper.ActivityMapper;
import com.studio.nx.yxl.mapper.PhotoAccountRecordMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/20 17:15
 */
@Service
public class PhotoWallActivityService {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private ResourcesConstant resourcesStatic;

    @Autowired
    private ActivityMapper activityMapper;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private PhotoWarehouseService photoWarehouseService;

    @Autowired
    private PhotoAccountRecordMapper photoAccountRecordMapper;

    @Autowired
    private PhotoGetLikeRecordService photoGetLikeRecordService;

    public BasePageResponse<ActivityResponse> list(PhotoWallActivityRequest request, String accountId, Integer page, Integer size) {
        IPage<ActivityResponse> _page = activityMapper.findPhotoWallActivitySearchList(request.getKeyWord(), accountId, new Page(page, size));
        for (ActivityResponse response : _page.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        return new BasePageResponse().setTotal(_page.getTotal()).setData(_page.getRecords());
    }

    public ActivityResponse clientDetail(String activityId, String accountId) {
        ActivityResponse response = null;
        if (StringUtils.hasText(accountId)) {
            response = activityMapper.appDetailByAccountId(activityId, accountId);
        } else {
            response = activityMapper.h5DetailByAccountId(activityId);
        }
        if (!Optional.ofNullable(response).isPresent()) {
            throw new BusinessException("数据异常");
        }

        /* 活动的日期列表 */
        List<ActivityDayListResponse> activityDayListResponseList = new ArrayList<>();
        List<PhotoWarehouse> dayList = photoWarehouseMapper.findGroupByActivityId(activityId);
        /* add by 梁展鹏 20210810 由于数据查询语句为通用，所以数据库查询出来并没有对photoTime字段进行格式化。目前第一步格式化为yyyy-MM-dd格式 */
        dayList.stream().forEach(o -> {
            LocalDate localDate = LocalDate.parse(o.getPhotoTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")), DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            o.setPhotoTime(localDate.atStartOfDay());
        });
        /* add by 梁展鹏 20210810 第二步对photoTime字段进行分组 */
        dayList.stream()
                .collect(Collectors.groupingBy(PhotoWarehouse::getPhotoTime, LinkedHashMap::new, Collectors.toList()))
                .forEach((key, value) -> {
                    ActivityDayListResponse r = new ActivityDayListResponse();
                    r.setName(key.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))); // 由于时间还带时分秒，所以进行格式化

                    /* 过滤横图，假如没有就拿第一张 */
                    List<PhotoWarehouse> listPreview = value.stream().filter(o -> o.getIsHorizontal()).collect(Collectors.toList());
                    r.setCover(localService.fullPath(listPreview.size() > 0 ? listPreview.get(0).getListPreview() : value.get(0).getListPreview()));

                    activityDayListResponseList.add(r);
                });
        response.setActivityDayListResponseList(activityDayListResponseList);

        List<String> coverList = new ArrayList<>();
        for (String cover : response.getCover().split(",")) {
            coverList.add(localService.fullPath(cover));
        }
        if (coverList.size() > 0) {
            response.setCover(coverList.get(0)); // 封面
        }

        response.setShareTitle(String.format("%s", response.getName())); // 分享-标题
        response.setShareDesc(String.format("%s", response.getBrief())); // 分享-简介
        response.setShareLink(String.format("%s/h5/activity?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 分享-链接
        response.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图

        return response;
    }

    public ActivityResponse clientDetailByPhotoTime(String activityId, String photoTime, String accountId) {
        ActivityResponse response = null;
        if (StringUtils.hasText(accountId)) {
            response = activityMapper.appDetailByAccountId(activityId, accountId);
        } else {
            response = activityMapper.h5DetailByAccountId(activityId);
        }
        if (!Optional.ofNullable(response).isPresent()) {
            throw new BusinessException("数据异常");
        }

        List<ActivityPhotoListResponse> activityPhotoListResponseList = new ArrayList<>();
        List<PhotoWarehouse> dayList = photoWarehouseMapper.findGroupByActivityId(activityId);
        ActivityResponse finalResponse = response;
        dayList.stream()
                .filter(o -> photoTime.equals(o.getPhotoTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))))
                .forEach(o -> {
                    ActivityPhotoListResponse r = PhotoWarehouseMapStruct.INSTANCES.toActivityPhotoListResponse(o);

                    int hour = r.getPhotoTime().getHour();
                    if (hour >= 0 && hour < 10) {
                        r.setTimeSlot(finalResponse.getTimeList().get(0));
                    } else if (hour >= 10 && hour < 12) {
                        r.setTimeSlot(finalResponse.getTimeList().get(1));
                    } else if (hour >= 12 && hour < 14) {
                        r.setTimeSlot(finalResponse.getTimeList().get(2));
                    } else if (hour >= 14) {
                        r.setTimeSlot(finalResponse.getTimeList().get(3));
                    }
                    r.setListPreview(localService.fullPath(r.getListPreview()));
                    r.setDetailPreview(localService.fullPath(r.getDetailPreview()));
                    activityPhotoListResponseList.add(r);
                });
        response.setActivityPhotoListResponseList(activityPhotoListResponseList);

        List<String> coverList = new ArrayList<>();
        for (String cover : response.getCover().split(",")) {
            coverList.add(localService.fullPath(cover));
        }
        if (coverList.size() > 0) {
            response.setCover(coverList.get(0)); // 封面
        }

        response.setShareTitle(String.format("%s", response.getName())); // 分享-标题
        response.setShareDesc(String.format("%s", response.getBrief())); // 分享-简介
        response.setShareLink(String.format("%s/h5/activity?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 分享-链接
        response.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图

        return response;
    }

    /**
     * 点赞/收藏
     *
     * @param dto
     */
    @Transactional(rollbackOn = Exception.class)
    public void collection(PhotoCollectionLikeRequest dto, String accountId, String nickname) {
        PhotoAccountRecord _entity = photoAccountRecordMapper.selectOne(new LambdaQueryWrapper<PhotoAccountRecord>()
                .eq(PhotoAccountRecord::getAccountId, accountId)
                .eq(PhotoAccountRecord::getPhotoWarehouseId, dto.getPhotoWarehouseId())
                .last(" limit 1")
        );
        /* 判断账号与照片是否有关联关系 */
        if (ObjectUtils.isNotEmpty(_entity)) {
            if (dto.getType().equals(0)) { // 收藏
                _entity.setIsCollections(Boolean.TRUE);
            }
            if (dto.getType().equals(1)) { // 点赞
                _entity.setIsLikes(Boolean.TRUE);
            }
            photoAccountRecordMapper.updateById(_entity);
        } else {
            PhotoAccountRecord entity = new PhotoAccountRecord();
            entity.setAccountId(accountId);
            entity.setPhotoWarehouseId(dto.getPhotoWarehouseId());
            if (dto.getType().equals(0)) { // 收藏
                entity.setIsCollections(Boolean.TRUE);
            }
            if (dto.getType().equals(1)) { // 点赞
                entity.setIsLikes(Boolean.TRUE);
            }
            photoAccountRecordMapper.insert(entity);
        }

        /* 处理业务表 */
        if (dto.getType() == 0) { // 收藏
            photoWarehouseService.collections(dto.getPhotoWarehouseId(), Boolean.TRUE);
        } else if (dto.getType() == 1) { // 点赞
            photoGetLikeRecordService.like(accountId, nickname, dto.getPhotoWarehouseId());
            photoWarehouseService.likes(dto.getPhotoWarehouseId(), Boolean.TRUE);
        }
    }

    /**
     * 取消点赞/收藏
     */
    @Transactional(rollbackOn = Exception.class)
    public void unCollection(PhotoCollectionLikeRequest dto, String accountId, String nickname) {
        PhotoAccountRecord _entity = photoAccountRecordMapper.selectOne(new LambdaQueryWrapper<PhotoAccountRecord>()
                .eq(PhotoAccountRecord::getAccountId, accountId)
                .eq(PhotoAccountRecord::getPhotoWarehouseId, dto.getPhotoWarehouseId())
                .last(" limit 1")
        );
        if (!Optional.ofNullable(_entity).isPresent()) {
            return;
        }

        if (dto.getType().equals(0)) { // 收藏
            _entity.setIsCollections(Boolean.FALSE);
        }
        if (dto.getType().equals(1)) { // 点赞
            _entity.setIsLikes(Boolean.FALSE);
        }
        photoAccountRecordMapper.updateById(_entity);

        /* 处理业务表 */
        if (dto.getType() == 0) { // 收藏
            photoWarehouseService.collections(dto.getPhotoWarehouseId(), Boolean.FALSE);
        } else if (dto.getType() == 1) { // 点赞
            photoGetLikeRecordService.delLike(accountId, dto.getPhotoWarehouseId());
            photoWarehouseService.likes(dto.getPhotoWarehouseId(), Boolean.FALSE);
        }
    }

}
