package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.util.DateUtils;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.ChoicePhotoRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.PhotoWallKindergartenInfoRequest;
import com.studio.nx.yxl.controller.app.request.photowall.kindergarten.PhotoWallKindergartenRequest;
import com.studio.nx.yxl.controller.app.response.photowall.PhotoInfoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.ChoicePhotoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.PhotoWallKindergartenInfoResponse;
import com.studio.nx.yxl.controller.app.response.photowall.kindergarten.PhotoWallKindergartenResponse;
import com.studio.nx.yxl.dto.KindergartenPhotoDto;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.entity.PhotoWarehouseMapStruct;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/19 19:25
 */
@Service
public class PhotoWallKindergartenService {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    public BasePageResponse<PhotoWallKindergartenResponse> list(PhotoWallKindergartenRequest request, Integer page, Integer size) {
        IPage<PhotoWallKindergartenResponse> p = kindergartenMapper.findPhotoWallKindergartenSearchList(request.getKeyWord(), new Page(page, size));
        for (PhotoWallKindergartenResponse response : p.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public PhotoWallKindergartenInfoResponse photoInfo(String kindergartenId, PhotoWallKindergartenInfoRequest request) {
        List<KindergartenPhotoDto> photoDtoList = photoWarehouseMapper.findKindergartenPhotoInfo(kindergartenId, request.getType());
        if (CollectionUtils.isEmpty(photoDtoList)) {
            return new PhotoWallKindergartenInfoResponse();
        }

        /* 照片时间格式化 */
        photoDtoList.stream().forEach(o -> o.setListPreview(localService.fullPath(o.getListPreview())));
        for (KindergartenPhotoDto photoDto : photoDtoList) {
            if (Optional.ofNullable(photoDto.getPhotoTime()).isPresent()) {
                photoDto.setYearMonth(photoDto.getPhotoTime().format(DateTimeFormatter.ofPattern("yyyy-MM")));
                photoDto.setYear(photoDto.getPhotoTime().format(DateTimeFormatter.ofPattern("yyyy")));
                photoDto.setMonth(photoDto.getPhotoTime().format(DateTimeFormatter.ofPattern("MM")));
                photoDto.setDay(photoDto.getPhotoTime().format(DateTimeFormatter.ofPattern("dd")));
                photoDto.setWeekDay(DateUtils.dateToWeek(photoDto.getPhotoTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))));
            }
        }

        /* 获取照片的【年月/yyyy-MM】 */
        LinkedHashSet<String> yearMonth = new LinkedHashSet<>();
        photoDtoList.stream().forEach((o) -> {
            yearMonth.add(o.getYearMonth());
        });

        /* 年月倒序 */
        List<String> yearMonthList = yearMonth.stream()
                .sorted(Comparator.reverseOrder())
                .collect(Collectors.toList());

        PhotoWallKindergartenInfoResponse response = new PhotoWallKindergartenInfoResponse();
        response.setYearMonthList(yearMonthList);
        response.setKindergartenPhotoList(photoDtoList);

        return response;
    }

    public List<ChoicePhotoResponse> photoListByDay(String kindergartenId, String photoTime) {
        List<ChoicePhotoResponse> photoWarehouseList = photoWarehouseMapper.findPhotoByDay(kindergartenId, photoTime);
        photoWarehouseList.stream().forEach(o -> {
            o.setDetailPreview(localService.fullPath(o.getDetailPreview()));
            o.setListPreview(localService.fullPath(o.getListPreview()));
        });

        return photoWarehouseList;
    }

    public PhotoInfoResponse photoInfoById(String id, String accountId) {
        PhotoInfoResponse photoInfo = photoWarehouseMapper.findPhotoInfoById(id, accountId);
        /* add by 梁展鹏 20210728 如果照片创建人与登录人一致，那么就是自己上传的照片，允许直接下载 */
        if (photoInfo.getAccountId().equals(accountId)) {
            photoInfo.setIsHdLogoPhoto(Boolean.TRUE);
            photoInfo.setIsOriginPhoto(Boolean.TRUE);
        }
        photoInfo.setDetailPreview(localService.fullPath(photoInfo.getDetailPreview()));
        photoInfo.setMemberPrice(photoInfo.getOriginPhotoPrice().multiply(new BigDecimal("0.9"))); // todo 会员价格暂时为 原图价格的90%
        photoInfo.setHdLogoPhotoIntegral(photoInfo.getHdLogoPhotoPrice().multiply(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_CASH_OUT))).intValue());

        /* add by 梁展鹏 20210905 为了逃避AppStore的内购要求审核，这里hardcode为特殊账号时，照片为0元，并且是已购买状态 */
        if (Arrays.asList("1418474590370840577").indexOf(accountId) >= 0) {
            photoInfo.setIsHdLogoPhoto(Boolean.TRUE);
            photoInfo.setHdLogoPhotoPrice(BigDecimal.ZERO);
            photoInfo.setHdLogoPhotoOriginPrice(BigDecimal.ZERO);
            photoInfo.setIsOriginPhoto(Boolean.TRUE);
            photoInfo.setOriginPhotoOriginPrice(BigDecimal.ZERO);
            photoInfo.setOriginPhotoPrice(BigDecimal.ZERO);
        }

        return photoInfo;
    }

    public BasePageResponse<ChoicePhotoResponse> beautifulList(ChoicePhotoRequest request, Integer page, Integer size) {
        IPage<PhotoWarehouse> p = photoWarehouseMapper.beautifulPage(request.getKindergartenId(), new Page(page, size));

        List<ChoicePhotoResponse> choicePhotoResponses = new ArrayList<>();
        p.getRecords().stream().forEach(o -> {
            ChoicePhotoResponse choicePhotoResponse = PhotoWarehouseMapStruct.INSTANCES.toChoicePhotoResponse(o);
            choicePhotoResponse.setDetailPreview(localService.fullPath(choicePhotoResponse.getDetailPreview()));
            choicePhotoResponse.setListPreview(localService.fullPath(choicePhotoResponse.getListPreview()));
            choicePhotoResponses.add(choicePhotoResponse);
        });

        // todo add by 梁展鹏 暂时先根据照片时间有序取出，然后打乱返回前端。总体还是有序的。。。咱不是完整无序
        Collections.shuffle(choicePhotoResponses);

        return new BasePageResponse().setTotal(p.getTotal()).setData(choicePhotoResponses);
    }

}
