package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.alioss.service.AliOssService;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.oss.minio.service.MinioService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.OssConstant;
import com.studio.nx.yxl.controller.admin.request.FileRequest;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseAuditMultiRequest;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseAuditRequest;
import com.studio.nx.yxl.controller.admin.request.PhotoWarehouseSaveRequest;
import com.studio.nx.yxl.controller.admin.response.PhotoWarehouseResponse;
import com.studio.nx.yxl.controller.admin.search.PhotoWarehouseSearch;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.math.BigDecimal;
import java.net.URI;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.Arrays;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/11 17:02
 */
@Service
public class PhotoWarehouseService {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private LocalService localService;

    @Autowired
    private MinioService minioService;

    @Autowired
    private AliOssService aliOssService;

    @Autowired
    private PhotoWarehouseMapper mapper;

    @Autowired
    private PhotoWarehouseProcessingService photoWarehouseProcessingService;

    /**
     * 功能描述：摄影师上传图片
     *
     * @param request
     * @param accountId
     */
    @SneakyThrows
    public void adminPhotoerSave(PhotoWarehouseSaveRequest request, String accountId) {
        String kindergartenId = request.getKindergartenId();
        String activityId = request.getActivityId();
        FileRequest fileRequest = request.getFileRequest();

        String uploadPath = copy(kindergartenId, activityId, Boolean.FALSE, fileRequest.getPath());

        /* 3. 业务数据保存入库 */
        PhotoWarehouse _entity = new PhotoWarehouse();
        _entity.setKindergartenId(kindergartenId); // 幼儿园标识
        _entity.setActivityId(activityId); // 活动标识
        _entity.setAccountId(accountId); // 账号标识

        _entity.setOriginFilename(fileRequest.getOriginalFilename()); // 原始文件名称
//        _entity.setPhotoTime(); // 拍照时间（读取图片信息）
       // _entity.setListPreview("/kindergarten" + uploadPath); // 列表页预览图（幼儿园logo、品牌logo）
       // _entity.setDetailPreview("/kindergarten" + uploadPath); // 详情页预览图（水印、幼儿园logo、品牌logo）

        /* 高清图带logo字段 */
//        _entity.setHdLogoPhotoSize(0L); // 高清带logo大小
        _entity.setHdLogoPhotoOriginPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_HD_LOGO_PHOTO_ORIGIN_PRICE))); // 高清带logo原价格
        _entity.setHdLogoPhotoPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_HD_LOGO_PHOTO_PRICE))); // 高清带logo价格
//        _entity.setHdLogoPhotoPixel(""); // 高清带logo像素
//        _entity.setHdLogoPhoto("); // 高清带logo

        /* 原图字段 */
        _entity.setOriginPhotoSize(fileRequest.getSize()); // 原图大小
        _entity.setOriginPhotoOriginPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_ORIGIN_PHOTO_ORIGIN_PRICE))); // 原图原价格
        _entity.setOriginPhotoPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_ORIGIN_PHOTO_PRICE))); // 原图价格
//        _entity.setOriginPhotoPixel(""); // 原图像素
        _entity.setOriginPhoto(uploadPath); // 原图

        _entity.setLikes(0); // 点赞数
        _entity.setCollections(0); // 收藏数
        _entity.setIsTake(Boolean.FALSE); // 是否为随手拍
        _entity.setIsBeautiful(Boolean.FALSE); // 是否为精选
        _entity.setIsReady(Boolean.FALSE); // 是否裂变图片
        _entity.setAuditStatus(PhotoWarehouse.AuditStatus.WAIT); // 审核状态（1：审核通过、0：未审核、-1:审核失败）
//        _entity.setAuditDate(null); // 审核时间
//        _entity.setAuditMsg(""); // 审核备注

        mapper.insert(_entity);

        /* 提前裂变图片 */
        photoWarehouseProcessingService.oneByOne(_entity);
    }

    public void appPhotoerSave(PhotoWarehouseSaveRequest request, String accountId) {
        /* 业务配置一天限制随手拍照片张数 */
        Integer dayCount = Integer.valueOf(businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_TAKE_COUNT));

        /* 从缓存中获取当前用户当前上传照片张数 */
        String redisKey = String.format("take:%s", accountId);
        String redisCount = stringRedisTemplate.opsForValue().get(redisKey);
        if (!StringUtils.hasText(redisCount)) {
            redisCount = "0";
        }
        if (StringUtils.hasText(redisCount) && Integer.parseInt(redisCount) > dayCount) {
            throw new BusinessException("上传失败，原因：超过一天最大的上传次数。");
        }

        // 计算当前离零时还有多少秒
        Long tomorrowSeconds = ChronoUnit.SECONDS.between(LocalDateTime.now(), LocalDateTime.of(LocalDateTime.now().plusDays(1).toLocalDate(), LocalTime.MIN));
        stringRedisTemplate.opsForValue().set(redisKey, String.valueOf(Integer.parseInt(redisCount) + 1), tomorrowSeconds, TimeUnit.SECONDS);

        appPhotoerSave(request.getKindergartenId(), request.getFileRequest(), accountId);
    }

    public void appPhotoerSave(String kindergartenId, FileRequest fileRequest, String accountId) {
        String uploadPath = copy(kindergartenId, "", Boolean.TRUE, fileRequest.getPath());

        /* 3. 业务数据保存入库 */
        PhotoWarehouse _entity = new PhotoWarehouse();
        _entity.setKindergartenId(kindergartenId); // 幼儿园标识
//        _entity.setActivityId(""); // 活动标识
        _entity.setAccountId(accountId); // 账号标识

        _entity.setOriginFilename(fileRequest.getOriginalFilename()); // 原始文件名称
//        _entity.setPhotoTime(); // 拍照时间（读取图片信息）
//        _entity.setListPreview(""); // 列表页预览图（幼儿园logo、品牌logo）
//        _entity.setDetailPreview(""); // 详情页预览图（水印、幼儿园logo、品牌logo）

        /* 高清图带logo字段 */
//        _entity.setHdLogoPhotoSize(0L); // 高清带logo大小
        _entity.setHdLogoPhotoOriginPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_TASK_HD_LOGO_PHOTO_ORIGIN_PRICE))); // 高清带logo原价格
        _entity.setHdLogoPhotoPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_TASK_HD_LOGO_PHOTO_PRICE))); // 高清带logo价格
//        _entity.setHdLogoPhotoPixel(""); // 高清带logo像素
//        _entity.setHdLogoPhoto("); // 高清带logo

        /* 原图字段 */
        _entity.setOriginPhotoSize(fileRequest.getSize()); // 原图大小
        _entity.setOriginPhotoOriginPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_TASK_ORIGIN_PHOTO_ORIGIN_PRICE))); // 原图原价格
        _entity.setOriginPhotoPrice(new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Cash.CASH_TASK_ORIGIN_PHOTO_PRICE))); // 原图价格
//        _entity.setOriginPhotoPixel(""); // 原图像素
        _entity.setOriginPhoto(uploadPath); // 原图

        _entity.setLikes(0); // 点赞数
        _entity.setCollections(0); // 收藏数
        _entity.setIsTake(Boolean.TRUE); // 是否为随手拍
        _entity.setIsBeautiful(Boolean.FALSE); // 是否为精选
        _entity.setIsReady(Boolean.FALSE); // 是否裂变图片
        _entity.setAuditStatus(PhotoWarehouse.AuditStatus.WAIT); // 审核状态（1：审核通过、0：未审核、-1:审核失败）
//        _entity.setAuditDate(null); // 审核时间
//        _entity.setAuditMsg(""); // 审核备注

        mapper.insert(_entity);

        /* 提前裂变图片 */
        photoWarehouseProcessingService.oneByOne(_entity);
    }

    /**
     * 功能描述：复制文件到指定业务目录
     *
     * @param kindergartenId
     * @param activityId
     * @param path           文件路径
     * @return 返回新目录的相对路径
     */
    @SneakyThrows
    private String copy(String kindergartenId, String activityId, Boolean isTake, String path) {
        String filename = path.substring(path.lastIndexOf("/") + 1); // 获取旧文件名称（包括扩展名）

        /* add by 梁展鹏 20210726 上传照片时活动标识非必填 */
        String uploadPath = "";
        if (isTake) { // 幼儿园 - 随手拍
            uploadPath = String.format("%s-%s", kindergartenId, "take");
        } else if (StringUtils.hasText(activityId) && !"null".equals(activityId)) { // 幼儿园 - 活动
            uploadPath = String.format("%s-%s", kindergartenId, activityId);
        } else { // 幼儿园
            uploadPath = kindergartenId;
        }
        uploadPath = ( uploadPath + File.separator + filename ).replaceAll("\\\\","/");
        /* 因为目前上传文件是默认到[tmp]的分桶中，所以需要根据业务调整文件的目录 */
        aliOssService.copyObject(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, path, OssConstant.YXL_KINDERGARTEN.BUCKETNAME, uploadPath);
        String newFullPath = aliOssService.generatePresignedUrl(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, uploadPath, AliOssService.DURATION, AliOssService.DOWNLOAD_LIMIT_SPEED);

        /* 2. 删除临时目录的文件 */
        aliOssService.deleteFile(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, path);

        return new URI(newFullPath).getPath();
    }

    /**
     * 功能描述：照片点赞
     *
     * @param photoWarehouseId 照片库标识
     */
    public void likes(String photoWarehouseId, Boolean isAdd) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(mapper.selectById(photoWarehouseId));
        if (optional.isPresent()) {
            mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                    .eq(PhotoWarehouse::getId, photoWarehouseId)
                    .set(PhotoWarehouse::getLikes, optional.get().getLikes() + (isAdd ? 1 : -1))
            );
        }
    }

    /**
     * 功能描述：照片收藏
     *
     * @param photoWarehouseId 照片库标识
     */
    public void collections(String photoWarehouseId, Boolean isAdd) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(mapper.selectById(photoWarehouseId));
        if (optional.isPresent()) {
            mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                    .eq(PhotoWarehouse::getId, photoWarehouseId)
                    .set(PhotoWarehouse::getCollections, optional.get().getCollections() + (isAdd ? 1 : -1))
            );
        }
    }

    /**
     * 功能描述：精选图片
     *
     * @param photoWarehouseId
     */
    public void isBeautiful(String photoWarehouseId) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(mapper.selectById(photoWarehouseId));
        if (optional.isPresent()) {
            PhotoWarehouse _photoWarehouse = optional.get();
            _photoWarehouse.setIsBeautiful(!_photoWarehouse.getIsBeautiful());
            mapper.updateById(_photoWarehouse);
        }
    }

    /**
     * 功能描述：审核照片
     *
     * @param request
     */
    public void audit(PhotoWarehouseAuditRequest request) {
        mapper.auditMulti(Arrays.asList(request.getId()), PhotoWarehouse.AuditStatus.PASS == request.getAuditStatus() ? Boolean.TRUE : Boolean.FALSE, request.getAuditStatus(), request.getAuditMsg());
    }

    /**
     * 功能描述：批量审核照片
     *
     * @param request
     */
    public void auditMulti(PhotoWarehouseAuditMultiRequest request) {
        mapper.auditMulti(request.getIds(), PhotoWarehouse.AuditStatus.PASS == request.getAuditStatus() ? Boolean.TRUE : Boolean.FALSE, request.getAuditStatus(), request.getAuditMsg());
    }

    /**
     * 功能描述：更新照片价格
     *
     * @param dto
     */
    public void updateItem(String id, PhotoWarehouseResponse dto) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            PhotoWarehouse _photoWarehouse = optional.get();
            _photoWarehouse.setHdLogoPhotoOriginPrice(dto.getHdLogoPhotoOriginPrice());
            _photoWarehouse.setHdLogoPhotoPrice(dto.getHdLogoPhotoPrice());
            _photoWarehouse.setOriginPhotoOriginPrice(dto.getOriginPhotoOriginPrice());
            _photoWarehouse.setOriginPhotoPrice(dto.getOriginPhotoPrice());
            _photoWarehouse.setIsShow(dto.getIsShow());
            _photoWarehouse.setIsBeautiful(dto.getIsBeautiful());
            _photoWarehouse.setIsTake(dto.getIsTake());
            mapper.updateById(_photoWarehouse);
        }
    }

    /**
     * 功能描述：删除信息
     *
     * @param id
     */
    public void handlerIsDel(String id) {
        Optional<PhotoWarehouse> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            PhotoWarehouse _photoWarehouse = optional.get();
            _photoWarehouse.setIsDel(!_photoWarehouse.getIsDel());
            mapper.updateById(_photoWarehouse);
        }
    }

    /**
     * 功能描述：查询[图片库]列表
     */
    public BasePageResponse<PhotoWarehouseResponse> page(PhotoWarehouseSearch dto, Integer page, Integer size) {
        IPage<PhotoWarehouseResponse> p = mapper.adminPage(dto, new Page(page, size));
        p.getRecords().stream().forEach(o -> {
            o.setListPreview(localService.fullPath(o.getListPreview()));
            o.setDetailPreview(localService.fullPath(o.getDetailPreview()));

//            String bucketName = minioService.cutBucketName(o.getOriginPhoto());
//            if (StringUtils.hasText(o.getHdLogoPhoto())) {
//                o.setHdLogoPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getHdLogoPhoto()), 5, TimeUnit.MINUTES));
//            }
//            o.setOriginPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getOriginPhoto()), 5, TimeUnit.MINUTES));
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    /**
     * 功能描述：查询[图片库]详情
     *
     * @param photoWarehouseId 标识
     * @return
     */
    public PhotoWarehouseResponse findAdminById(String photoWarehouseId) {
        PhotoWarehouseResponse dto = mapper.findAdminById(photoWarehouseId);
        if (!Optional.ofNullable(dto).isPresent()) {
            return null;
        }

        dto.setListPreview(localService.fullPath(dto.getListPreview()));
        dto.setDetailPreview(localService.fullPath(dto.getDetailPreview()));

//        String bucketName = minioService.cutBucketName(dto.getOriginPhoto());
//        if (StringUtils.hasText(dto.getHdLogoPhoto())) {
//            dto.setHdLogoPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, dto.getHdLogoPhoto()), 5, TimeUnit.MINUTES));
//        }
//        dto.setOriginPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, dto.getOriginPhoto()), 5, TimeUnit.MINUTES));
        return dto;
    }

}
