package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.minio.service.MinioService;
import com.studio.nx.uiac.controller.response.RoleResponse;
import com.studio.nx.uiac.service.AccountRoleService;
import com.studio.nx.uiac.service.RoleService;
import com.studio.nx.yxl.controller.admin.request.VerifyRequest;
import com.studio.nx.yxl.controller.admin.response.PhotoerAuditAdminPageResponse;
import com.studio.nx.yxl.controller.admin.search.PhotoerSearch;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.mapper.PhotoerAuditMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class PhotoerAuditService {

    @Autowired
    private MinioService minioService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private AccountRoleService accountRoleService;

    @Autowired
    private PhotoerAuditMapper mapper;

    /**
     * 功能描述：查询[摄影师]详情
     *
     * @param id 标识
     * @return
     */
    public Optional<PhotoerAudit> findById(String id) {
        PhotoerAudit _entity = mapper.selectById(id);
        if (!Optional.ofNullable(_entity).isPresent()) {
            return Optional.empty();
        }

        String bucketName = minioService.cutBucketName(_entity.getBodyPhoto());
        if (StringUtils.hasText(_entity.getBodyPhoto())) {
            _entity.setBodyPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, _entity.getBodyPhoto()), 5, TimeUnit.MINUTES));
        }
        if (StringUtils.hasText(_entity.getIdCardFront())) {
            _entity.setIdCardFront(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, _entity.getIdCardFront()), 5, TimeUnit.MINUTES));
        }
        if (StringUtils.hasText(_entity.getIdCardBack())) {
            _entity.setIdCardBack(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, _entity.getIdCardBack()), 5, TimeUnit.MINUTES));
        }
        if (StringUtils.hasText(_entity.getSampleReels())) {
            _entity.setSampleReels(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, _entity.getSampleReels()), 5, TimeUnit.MINUTES)); // 作品集
        }

        return Optional.ofNullable(_entity);
    }

    /**
     * 功能描述：获取账号的摄影师认证记录（只有一条）
     *
     * @param accountId
     * @return
     */
    public Optional<PhotoerAudit> findByAccountId(String accountId) {
        return Optional.ofNullable(
                mapper.selectOne(new LambdaQueryWrapper<PhotoerAudit>()
                        .eq(PhotoerAudit::getAccountId, accountId)
                )
        );
    }

    /**
     * 功能描述：查询[摄影师]列表
     */
    public BasePageResponse<PhotoerAuditAdminPageResponse> page(PhotoerSearch dto, Integer page, Integer size) {
        IPage<PhotoerAuditAdminPageResponse> p = mapper.adminPage(dto, new Page(page, size));
        p.getRecords().stream().forEach(o -> {
            String bucketName = minioService.cutBucketName(o.getBodyPhoto());

            if (StringUtils.hasText(o.getBodyPhoto())) {
                o.setBodyPhoto(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getBodyPhoto()), 5, TimeUnit.MINUTES));
            }
            if (StringUtils.hasText(o.getIdCardFront())) {
                o.setIdCardFront(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardFront()), 5, TimeUnit.MINUTES));
            }
            if (StringUtils.hasText(o.getIdCardBack())) {
                o.setIdCardBack(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getIdCardBack()), 5, TimeUnit.MINUTES));
            }
            if (StringUtils.hasText(o.getSampleReels())) {
                o.setSampleReels(minioService.getPresignedObjectUrl(bucketName, minioService.cutFilename(bucketName, o.getSampleReels()), 5, TimeUnit.MINUTES));
            }
        });

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    /**
     * 功能描述：替换[摄影师]审核状态
     *
     * @param photoerAuditId
     */
    public void handleIsAudit(String photoerAuditId, VerifyRequest request, String accountId) {
        Optional<PhotoerAudit> optional = findById(photoerAuditId);
        if (!optional.isPresent()) {
            throw new BusinessException("摄影师申请信息不存在");
        }

        PhotoerAudit _entity = optional.get();
        if (PhotoerAudit.AuditStatus.PASS == _entity.getAuditStatus()) {
            throw new BusinessException("摄影师 已通过 审核。");
        }
        if (PhotoerAudit.AuditStatus.FAIL == _entity.getAuditStatus()) {
            throw new BusinessException("摄影师 未能通过 审核，原因：" + _entity.getAuditMsg());
        }

        mapper.update(null, new LambdaUpdateWrapper<PhotoerAudit>()
                .set(PhotoerAudit::getAuditStatus, request.getAuditStatus())
                .set(PhotoerAudit::getAuditMsg, request.getAuditMsg())
                .set(PhotoerAudit::getAuditAccountId, accountId)
                .set(PhotoerAudit::getAuditDate, LocalDateTime.now())
                .eq(PhotoerAudit::getId, photoerAuditId)
        );

        /* add by 梁展鹏 20210909 只有在审批通过才进行 */
        if (PhotoerAudit.AuditStatus.PASS == request.getAuditStatus()) {
            /* 摄影师审核通过增加摄影师角色 */
            Optional<RoleResponse> roleResponseDtoOptional = roleService.findByCode("photoer");
            if (roleResponseDtoOptional.isPresent()) {
                String photoerAccountId = _entity.getAccountId();
                String photoerRoleId = roleResponseDtoOptional.get().getId();
                accountRoleService.save(photoerAccountId, photoerRoleId);
            }
        }
    }

}
