package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.yxl.entity.Kindergarten;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.entity.PhotoerSignInRecord;
import com.studio.nx.yxl.mapper.KindergartenMapper;
import com.studio.nx.yxl.mapper.PhotoerSignInRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/7/10 10:57
 */
@Service
public class PhotoerSignInRecordService {

    @Autowired
    private KindergartenMapper kindergartenMapper;

    @Autowired
    private PhotoerAuditService photoerAuditService;

    @Autowired
    private PhotoerSignInRecordMapper photoerSignInRecordMapper;

    public void signIn(String kindergartenId, String accountId) {
        Kindergarten kindergarten = kindergartenMapper.selectById(kindergartenId);
        if (!Optional.ofNullable(kindergarten).isPresent()) {
            throw new BusinessException("幼儿园不存在");
        }

        Optional<PhotoerAudit> optional = photoerAuditService.findByAccountId(accountId);
        if (!optional.isPresent()) {
            throw new BusinessException("您还没申请摄影师认证");
        }
        PhotoerAudit photoerAudit = optional.get();
        if (PhotoerAudit.AuditStatus.PASS != photoerAudit.getAuditStatus()) {
            throw new BusinessException("您的摄影师认证还没通过");
        }

        PhotoerSignInRecord entity = new PhotoerSignInRecord();
        entity.setAccountId(accountId);
        entity.setKindergartenId(kindergarten.getId());
        entity.setKindergartenName(kindergarten.getName());
        photoerSignInRecordMapper.insert(entity);
    }

    public BasePageResponse<PhotoerSignInRecord> record(String accountId, Integer page, Integer size) {
        Page<PhotoerSignInRecord> p = photoerSignInRecordMapper.selectPage(new Page(page, size), new LambdaQueryWrapper<PhotoerSignInRecord>()
                .eq(PhotoerSignInRecord::getAccountId, accountId)
                .eq(PhotoerSignInRecord::getIsDel, Boolean.FALSE)
                .orderByDesc(PhotoerSignInRecord::getCreateAt)
        );

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

}
