package com.studio.nx.yxl.service;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.controller.app.request.photowall.PhotoCollectionLikeRequest;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ShoppingCarAddRequest;
import com.studio.nx.yxl.controller.app.request.shoppingcar.ToCollectionRequest;
import com.studio.nx.yxl.controller.app.response.shoppingcar.ShoppingCarResponse;
import com.studio.nx.yxl.controller.app.response.shoppingcar.ShoppingCarSelectDto;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.entity.ShoppingCar;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import com.studio.nx.yxl.mapper.ShoppingCarMapper;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;


/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/26 17:26
 */
@Service
public class ShoppingCarService {

    @Autowired
    private LocalService localService;

    @Autowired
    private ShoppingCarMapper mapper;

    @Autowired
    private PhotoWarehouseMapper photoWarehouseMapper;

    @Autowired
    private PhotoWallActivityService photoWallActivityService;

    @Autowired
    private PhotoAccountRecordService photoAccountRecordService;

    public Integer carNumber(String accountId) {
        return mapper.selectCount(new LambdaQueryWrapper<ShoppingCar>()
                .eq(ShoppingCar::getAccountId, accountId)
                .eq(ShoppingCar::getIsDel, Boolean.FALSE)
        );
    }

    public List<ShoppingCarResponse> carList(String accountId) {
        List<ShoppingCarResponse> result = new ArrayList<>();
        List<ShoppingCarSelectDto> carList = mapper.findCarList(accountId);
        carList.stream().forEach(o -> {
            result.add(to(o));
        });
        return result;
    }

    public ShoppingCar addShoppingCar(ShoppingCarAddRequest dto, String accountId) {
        /* 判断照片id是否符合购买逻辑 */
        PhotoWarehouse photoWarehouse = photoWarehouseMapper.selectById(dto.getPhotoWarehouseId());
        if (!Optional.ofNullable(photoWarehouse).isPresent()) {
            throw new BusinessException("数据异常");
        }
        if (photoWarehouse.getIsReady() == Boolean.FALSE || photoWarehouse.getIsShow() == Boolean.FALSE || photoWarehouse.getAuditStatus() != PhotoWarehouse.AuditStatus.PASS) {
            throw new BusinessException("照片不能加入到购物车");
        }

        /* 判断照片id是否已经购买 */
        if (ShoppingCar.Type.HD_LOGO_PHOTO.equals(dto.getType()) && photoAccountRecordService.isHdLogoPhoto(accountId, dto.getPhotoWarehouseId())) {
            throw new BusinessException("您已经购买过该高清图了");
        } else if (ShoppingCar.Type.ORIGIN_PHOTO.equals(dto.getType()) && photoAccountRecordService.isOriginPhoto(accountId, dto.getPhotoWarehouseId())) {
            throw new BusinessException("您已经购买过该原图了");
        }

        String shoppingCarId = IdUtil.createSnowflake(1L, 1L).nextIdStr();
        Boolean isHdLogoPhoto = ShoppingCar.Type.HD_LOGO_PHOTO.equals(dto.getType()) ? Boolean.TRUE : Boolean.FALSE;
        Boolean isOriginPhoto = ShoppingCar.Type.ORIGIN_PHOTO.equals(dto.getType()) ? Boolean.TRUE : Boolean.FALSE;
        mapper.addShoppingCar(shoppingCarId, accountId, dto.getPhotoWarehouseId(), isHdLogoPhoto, isOriginPhoto);

        ShoppingCar _shoppingCar = new ShoppingCar();
        _shoppingCar.setId(shoppingCarId);
        _shoppingCar.setAccountId(accountId);
        _shoppingCar.setPhotoWarehouseId(dto.getPhotoWarehouseId());
        _shoppingCar.setIsHdLogoPhoto(isHdLogoPhoto);
        _shoppingCar.setIsOriginPhoto(isOriginPhoto);

        return _shoppingCar;
    }

    public ShoppingCarResponse addShoppingCar2Response(ShoppingCarAddRequest dto, String accountId) {
        PhotoWarehouse photoWarehouse = photoWarehouseMapper.selectById(dto.getPhotoWarehouseId());
        if (!Optional.ofNullable(photoWarehouse).isPresent()) {
            throw new BusinessException("数据异常");
        }
        if (photoWarehouse.getIsReady() == Boolean.FALSE || photoWarehouse.getIsShow() == Boolean.FALSE || photoWarehouse.getAuditStatus() != PhotoWarehouse.AuditStatus.PASS) {
            throw new BusinessException("照片还没初始化结束，不能加入到购物车");
        }

        ShoppingCar _shoppingCar = addShoppingCar(dto, accountId);
        Boolean isHdLogoPhoto = ShoppingCar.Type.HD_LOGO_PHOTO.equals(dto.getType()) ? Boolean.TRUE : Boolean.FALSE;
        Boolean isOriginPhoto = ShoppingCar.Type.ORIGIN_PHOTO.equals(dto.getType()) ? Boolean.TRUE : Boolean.FALSE;

        ShoppingCarSelectDto selectDto = new ShoppingCarSelectDto();
        selectDto.setShoppingCarId(_shoppingCar.getId()); // 购物车项标识
        selectDto.setPhotoWarehouseId(dto.getPhotoWarehouseId()); // 图片标识
        selectDto.setListPreview(photoWarehouse.getListPreview()); // 列表预览图
        selectDto.setIsHorizontal(photoWarehouse.getIsHorizontal()); // 是否横图
        selectDto.setIsHdLogoPhoto(isHdLogoPhoto); // 是否高清带logo图
        selectDto.setIsOriginPhoto(isOriginPhoto); // 是否原图
        selectDto.setHdLogoPhotoPrice(photoWarehouse.getHdLogoPhotoPrice()); // 高清带logo价格
        selectDto.setOriginPhotoPrice(photoWarehouse.getOriginPhotoPrice()); // 原图价格
        return to(selectDto);
    }

    private ShoppingCarResponse to(ShoppingCarSelectDto shoppingCarSelectDto) {
        ShoppingCarResponse shoppingCarResponse = new ShoppingCarResponse();
        shoppingCarResponse.setShoppingCarId(shoppingCarSelectDto.getShoppingCarId());
        shoppingCarResponse.setPhotoWarehouseId(shoppingCarSelectDto.getPhotoWarehouseId());
        shoppingCarResponse.setIsHorizontal(shoppingCarSelectDto.getIsHorizontal());
        shoppingCarResponse.setListPreview(localService.fullPath(shoppingCarSelectDto.getListPreview()));
        // 判断购物车图片类型，查出不同的价格
        if (shoppingCarSelectDto.getIsOriginPhoto()) {
            shoppingCarResponse.setPhotoType(2);
            shoppingCarResponse.setPrice(shoppingCarSelectDto.getOriginPhotoPrice());
        }
        if (shoppingCarSelectDto.getIsHdLogoPhoto()) {
            shoppingCarResponse.setPhotoType(1);
            shoppingCarResponse.setPrice(shoppingCarSelectDto.getHdLogoPhotoPrice());
        }

        return shoppingCarResponse;
    }

    public void toCollection(ToCollectionRequest dto, String accountId, String nickname) {
        if (CollectionUtils.isEmpty(dto.getPhotoWarehouseIdList())) {
            return;
        }

        dto.getPhotoWarehouseIdList().stream().forEach(id -> {
            PhotoCollectionLikeRequest photoCollectionLikeRequest = new PhotoCollectionLikeRequest();
            photoCollectionLikeRequest.setPhotoWarehouseId(id);
            photoCollectionLikeRequest.setType(0);
            photoWallActivityService.collection(photoCollectionLikeRequest, accountId, nickname);
        });

        /* 根据照片id、账号标识。清空对应的购物车记录 */
        deleteByPhotoWarehouseId(dto.getPhotoWarehouseIdList(), accountId);
    }

    public void deleteByPhotoWarehouseId(List<String> photoWarehouseIdList, String accountId) {
        if (CollectionUtils.isEmpty(photoWarehouseIdList)) {
            return;
        }

        mapper.delete(new LambdaQueryWrapper<ShoppingCar>()
                .in(ShoppingCar::getPhotoWarehouseId, photoWarehouseIdList)
                .eq(ShoppingCar::getAccountId, accountId)
        );
    }

    public void deleteByShoppingCarId(List<String> shoppingCarIdList, String accountId) {
        if (CollectionUtils.isEmpty(shoppingCarIdList)) {
            return;
        }

        mapper.delete(new LambdaQueryWrapper<ShoppingCar>()
                .in(ShoppingCar::getId, shoppingCarIdList)
                .eq(ShoppingCar::getAccountId, accountId)
        );
    }

}
