package com.studio.nx.yxl.service;

import cn.hutool.core.date.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.util.DateTimeUtils;
import com.studio.nx.framework.util.DateUtils;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.controller.app.response.my.signin.CurrentSignInResponse;
import com.studio.nx.yxl.controller.app.response.my.signin.DayInfoResponse;
import com.studio.nx.yxl.controller.app.response.my.signin.EveryDayTaskResponse;
import com.studio.nx.yxl.entity.OperateSignInRecord;
import com.studio.nx.yxl.mapper.OperateSignInRecordMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 功能描述：
 *
 * @Author 何锦枫
 * @Date 2021/6/27 10:25
 */
@Service
public class SignInService {

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private OperateSignInRecordMapper operateSignInRecordMapper;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    public CurrentSignInResponse currentSignIn(String accountId) {
        CurrentSignInResponse response = new CurrentSignInResponse();

        // 用户当前积分
        response.setIntegralSum(integralCashRecordService.countMyIntegral(accountId));

        Date now = new Date();
        // 当前月份
        String yearMonth = DateUtils.format(now, "yyyy年MM月");
        response.setYearMonth(yearMonth);

        //当前周签到情况
        //当前周的第一天和最后一天
        Date weekStart = CalendarUtil.beginOfWeek(Calendar.getInstance(), false).getTime();
        Date weekEnd = CalendarUtil.endOfWeek(Calendar.getInstance(), false).getTime();
        List<String> detHashList = getDetHashList(weekStart, weekEnd);

        OperateSignInRecord operateSignInRecord = operateSignInRecordMapper.selectOne(new LambdaQueryWrapper<OperateSignInRecord>()
                .eq(OperateSignInRecord::getAccountId, accountId)
                .eq(OperateSignInRecord::getIsDel, Boolean.FALSE)
        );

        LocalDateTime nowDate = LocalDateTime.now();
        String nowDateFormat = LocalDateTimeUtil.format(nowDate, DatePattern.NORM_DATE_PATTERN);
        //连续签到天数
        Integer signInDays;
        //最后签到日期
        String lastSignInDateFormat = null;
        //当前日期的前一天
        String preDateByDate = DateUtils.getPreDateByDate(nowDateFormat);
        if (ObjectUtils.isNotEmpty(operateSignInRecord)) {
            LocalDateTime lastSignInDate = operateSignInRecord.getLastSignInDate();
            lastSignInDateFormat = LocalDateTimeUtil.format(lastSignInDate, DatePattern.NORM_DATE_PATTERN);
            //当天是否已签到
            if (lastSignInDateFormat.equals(nowDateFormat)) {
                response.setIsSignIn(Boolean.TRUE);
                response.setSignInDays(operateSignInRecord.getSignInDays());
                signInDays = operateSignInRecord.getSignInDays();
            } else {
                //最后一天签到日期为昨天，为连续签到
                if (lastSignInDateFormat.equals(preDateByDate)) {
                    response.setSignInDays(operateSignInRecord.getSignInDays());
                    signInDays = operateSignInRecord.getSignInDays();
                } else {
                    //最后一天签到不是昨天和今天，连续签到设置为0天
                    response.setSignInDays(0);
                    signInDays = 0;
                }
            }
        } else {
            //没有签到记录，连续签到设置为0
            response.setSignInDays(0);
            signInDays = 0;
        }
        //已签到的日期
        List<String> signInList = new ArrayList<>();
        if (signInDays > 0) {
            if(nowDateFormat.equals(lastSignInDateFormat)){
                if(signInDays==1){
                    signInList.add(lastSignInDateFormat);
                }else {
                    Date lastSignInDate= DateUtils.format(lastSignInDateFormat, "yyyy-MM-dd");
                    Date beginDate = DateTimeUtils.dateAddDays(lastSignInDate, -signInDays+1);
                    signInList = getDetHashList(beginDate, lastSignInDate);
                }
            }else{
                Date preDate = DateUtils.format(preDateByDate, "yyyy-MM-dd");
                Date beginDate = DateTimeUtils.dateAddDays(preDate, -signInDays+1);
                signInList = getDetHashList(beginDate, preDate);
            }
        }

        List<Integer> dayList = new ArrayList<>();
        List<DayInfoResponse> dayInfoResponseList = new ArrayList<>();
        List<String> finalSignInList = signInList;
        detHashList.stream().forEach(o -> {
            DayInfoResponse dayInfo = new DayInfoResponse();
            finalSignInList.stream().forEach(w -> {
                if (o.equals(w)) {
                    dayInfo.setSignInStatus(Boolean.TRUE);
                }
            });
            Date formatDate = DateUtils.format(o, "yyyy-MM-dd");
            dayInfo.setTimeInfo(o);
            dayInfo.setDay(DateUtil.dayOfMonth(formatDate));
            dayInfo.setWeek(DateUtils.dateToWeek(o).substring(2));

            dayList.add(DateUtil.dayOfMonth(formatDate));
            dayInfoResponseList.add(dayInfo);
        });
        response.setDayList(dayList);
        response.setWeekInfo(dayInfoResponseList);
        response.setEveryDayTask(getEveryDayTask());
        return response;
    }

    /**
     * 功能描述：获取每日任务列表
     * @return
     */
    private List<EveryDayTaskResponse> getEveryDayTask() {
        BigDecimal value1 = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SHARE_POINTS)); // 分享资讯
        BigDecimal value2 = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_LIKE_TO_POINTS)); // 点赞照片
        BigDecimal value3 = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_POINTS_BY_LIKE)); // 被点赞照片
        return new ArrayList<EveryDayTaskResponse>() {{
            add(new EveryDayTaskResponse().setName("分享资讯").setValue(String.format("+%s 积分", value1)));
            add(new EveryDayTaskResponse().setName("点赞照片").setValue(String.format("+%s 积分", value2)));
            add(new EveryDayTaskResponse().setName("被点赞照片").setValue(String.format("+%s 积分", value3)));
        }};
    }

    /**
     * 获取两个日期中的所有日期
     */
    public static List<String> getDetHashList(Date begin, Date end) {
        List<String> hashList = new ArrayList<>();
        //创建日期范围生成器
        List<DateTime> dateList = DateUtil.rangeToList(begin, end, DateField.DAY_OF_YEAR);
        String hash = null;
        for (DateTime dt : dateList) {
            hash = dt.toString("yyyy-MM-dd");
            hashList.add(hash);
        }
        return hashList;
    }

    /**
     * 签到
     */
    @Transactional(rollbackOn = Exception.class)
    public Integer signIn(String accountId) {
        String operateSignInRecordId = "";
        OperateSignInRecord operateSignInRecord = operateSignInRecordMapper.selectOne(new LambdaQueryWrapper<OperateSignInRecord>()
                .eq(OperateSignInRecord::getAccountId, accountId)
                .eq(OperateSignInRecord::getIsDel, Boolean.FALSE)
        );

        LocalDateTime nowDate = LocalDateTime.now();
        String nowDateFormat = LocalDateTimeUtil.format(nowDate, DatePattern.NORM_DATE_PATTERN);

        // 当前日期的前一天
        String preDateByDate = DateUtils.getPreDateByDate(nowDateFormat);
        if (ObjectUtils.isNotEmpty(operateSignInRecord)) {
            operateSignInRecordId = operateSignInRecord.getId();
            LocalDateTime lastSignInDate = operateSignInRecord.getLastSignInDate();
            String lastSignInDateFormat = LocalDateTimeUtil.format(lastSignInDate, DatePattern.NORM_DATE_PATTERN);
            // 当天是否已签到，已签到跳过
            if (lastSignInDateFormat.equals(nowDateFormat)) {
                return 0;
            } else {
                // 最后一天签到日期为昨天，为连续签到
                if (lastSignInDateFormat.equals(preDateByDate)) {
                    operateSignInRecord.setSignInDays(operateSignInRecord.getSignInDays() + 1);
                } else {
                    operateSignInRecord.setSignInDays(1);
                }
                operateSignInRecord.setLastSignInDate(nowDate);
                operateSignInRecordMapper.updateById(operateSignInRecord);
            }
        } else {
            // 首次签到则传入默认值
            OperateSignInRecord _entity = new OperateSignInRecord();
            _entity.setAccountId(accountId);
            _entity.setSignInDays(1);
            _entity.setLastSignInDate(nowDate);
            operateSignInRecordMapper.insert(_entity);

            operateSignInRecordId = _entity.getId();
        }

        BigDecimal value = new BigDecimal(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SIGN_IN_BY_DAY));
        integralCashRecordService.saveIntegralSignInByDay(accountId);

        /* 判断是否达到连续签到奖励标准 */
        isReward(operateSignInRecordId, accountId);

        return value.intValue();
    }

    @Transactional
    public void isReward(String id, String accountId) {
        OperateSignInRecord operateSignInRecord = operateSignInRecordMapper.selectById(id);
        if (!Optional.ofNullable(operateSignInRecord).isPresent()) {
            return;
        }

        int signInDays = operateSignInRecord.getSignInDays();
        if (signInDays <= 0) {
            return;
        }
        int integralSignInByEveryDay = Integer.parseInt(businessConfigService.getByKey(BusinessConfigKey.Integral.INTEGRAL_SIGN_IN_BY_EVERY_DAY));
        if (integralSignInByEveryDay <= 0) {
            return;
        }

        /* 连续签到次数取余是否达标 */
        if (signInDays % integralSignInByEveryDay == 0) {
            integralCashRecordService.saveIntegralSignInByEveryDayPoints(accountId);
        }
    }

}
