package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.yxl.controller.admin.request.TopSearchKeywordsSaveRequest;
import com.studio.nx.yxl.controller.admin.search.TopSearchKeywordsSearchRequest;
import com.studio.nx.yxl.entity.TopSearchKeywords;
import com.studio.nx.yxl.mapper.TopSearchKeywordsMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/15 21:18
 */
@Service
public class TopSearchKeywordsService {

    @Autowired
    private TopSearchKeywordsMapper mapper;

    public List<TopSearchKeywords> list() {
        return mapper.selectList(new LambdaQueryWrapper<TopSearchKeywords>()
                .eq(TopSearchKeywords::getIsDel, Boolean.FALSE)
                .eq(TopSearchKeywords::getIsShow, Boolean.TRUE)
                .orderByDesc(TopSearchKeywords::getSort)
        );
    }

    public BasePageResponse<TopSearchKeywords> adminPage(TopSearchKeywordsSearchRequest request, Integer page, Integer size) {
        LambdaQueryWrapper<TopSearchKeywords> wrapper = new LambdaQueryWrapper();
        wrapper.eq(TopSearchKeywords::getIsDel, Boolean.FALSE);

        if (StringUtils.hasText(request.getName())) {
            wrapper.like(TopSearchKeywords::getName, request.getName());
        }
        if (null != request.getIsShow()) {
            wrapper.eq(TopSearchKeywords::getIsShow, request.getIsShow());
        }

//        wrapper.orderByDesc(TopSearchKeywords::getIsShow);
        wrapper.orderByDesc(TopSearchKeywords::getSort);
        wrapper.orderByDesc(TopSearchKeywords::getCreateAt);

        Page<TopSearchKeywords> p = mapper.selectPage(new Page(page, size), wrapper);

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public Optional<TopSearchKeywords> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    public void save(TopSearchKeywordsSaveRequest request) {
        TopSearchKeywords _entity = new TopSearchKeywords();
        _entity.setName(request.getName());
        _entity.setSort(request.getSort());
        _entity.setIsShow(request.getIsShow());
        mapper.insert(_entity);
    }

    public void update(String id, TopSearchKeywordsSaveRequest request) {
        Optional<TopSearchKeywords> optional = findById(id);
        if (optional.isPresent()) {
            TopSearchKeywords _entity = optional.get();
            _entity.setName(request.getName());
            _entity.setSort(request.getSort());
            _entity.setIsShow(request.getIsShow());
            mapper.updateById(_entity);
        }
    }

    public void isShow(String id) {
        Optional<TopSearchKeywords> optional = findById(id);
        if (optional.isPresent()) {
            TopSearchKeywords _entity = optional.get();
            _entity.setIsShow(!_entity.getIsShow());
            mapper.updateById(_entity);
        }
    }

    public void isDel(String id) {
        Optional<TopSearchKeywords> optional = findById(id);
        if (optional.isPresent()) {
            TopSearchKeywords _entity = optional.get();
            _entity.setIsDel(!_entity.getIsDel());
            mapper.updateById(_entity);
        }
    }

}
