package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.config.service.BusinessConfigService;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.yxl.constant.BusinessConfigKey;
import com.studio.nx.yxl.constant.ResourcesConstant;
import com.studio.nx.yxl.controller.admin.search.TweetSearchDto;
import com.studio.nx.yxl.controller.app.request.tweet.TweetRequest;
import com.studio.nx.yxl.controller.app.response.tweet.TweetInfoCollectionResponse;
import com.studio.nx.yxl.controller.app.response.tweet.TweetListResponse;
import com.studio.nx.yxl.dto.TweetDto;
import com.studio.nx.yxl.entity.Tweet;
import com.studio.nx.yxl.entity.TweetMapStruct;
import com.studio.nx.yxl.mapper.TweetMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class TweetService {

    @Autowired
    private LocalService localService;

    @Autowired
    private BusinessConfigService businessConfigService;

    @Autowired
    private ResourcesConstant resourcesStatic;

    @Autowired
    private TweetMapper mapper;

    /**
     * 功能描述：查询[资讯]详情(H5专用）
     * 追加阅读数+1逻辑
     * todo 缓存失效时间
     * @param id 标识
     * @return
     */
//    @Cacheable(cacheNames = "yxl:tweet", key = "#id")
    public Tweet findByIdInH5(String id) {
        addRead(id);
        Tweet tweet = mapper.selectById(id);
        tweet.setCover(localService.fullPath(tweet.getCover()));
        return tweet;
    }

    /**
     * 功能描述：查询[资讯]详情
     *
     * @param id 标识
     * @return
     */
    public Tweet findByIdInAdmin(String id) {
        Tweet tweet = mapper.selectById(id);
        tweet.setCover(localService.fullPath(tweet.getCover()));
        return tweet;
    }

    /**
     * 功能描述：客户端查询[资讯]列表
     */
    public BasePageResponse<TweetListResponse> appList(TweetRequest dto, Integer page, Integer size) {
        LambdaQueryWrapper<Tweet> wrapper = new QueryWrapper().lambda();
        if (StringUtils.hasText(dto.getKeyWord())) {
            wrapper.like(Tweet::getTitle, dto.getKeyWord());
        }

        wrapper.eq(Tweet::getIsDel, Boolean.FALSE);
        wrapper.eq(Tweet::getIsShow, Boolean.TRUE);
        wrapper.orderByDesc(Tweet::getSort);
        wrapper.orderByDesc(Tweet::getCreateAt);

        Page<Tweet> p = mapper.selectPage(new Page(page, size), wrapper);

        List<TweetListResponse> list = new ArrayList();
        for (Tweet tweet : p.getRecords()) {
            TweetListResponse response = TweetMapStruct.INSTANCES.toListResponse(tweet);
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }

            /* add by 梁展鹏 20210830 增加参数使app内访问隐藏引导下载浮动框 */
            response.setLink(String.format("%s/h5/tweet?id=%s&hidden=1", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 访问地址

            response.setShareTitle(String.format("%s", response.getTitle())); // 分享-标题
            response.setShareDesc(String.format("%s", response.getBrief())); // 分享-简介
            response.setShareLink(String.format("%s/h5/tweet?id=%s", businessConfigService.getByKey(BusinessConfigKey.Business.BUSINESS_H5_WEB_DOMAIN), response.getId())); // 分享-链接
            response.setShareImgUrl(resourcesStatic.getShareImg()); // 分享-小图

            list.add(response);
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    /**
     * 功能描述：查询[APP-资讯]点赞收藏情况
     *
     * @param tweetId 标识
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Optional<TweetInfoCollectionResponse> appTweetInfoCollection(String tweetId, String accountId) {
        return Optional.ofNullable(mapper.findByInfoCollection(tweetId, accountId));
    }

    /**
     * 功能描述：查询[资讯]列表
     */
    public BasePageResponse<TweetDto> page(TweetSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<Tweet> wrapper = new QueryWrapper().lambda();
        if (StringUtils.hasText(dto.getTitle())) {
            wrapper.like(Tweet::getTitle, dto.getTitle());
        }
        if (dto.getStatus() != null) {
            wrapper.eq(Tweet::getIsShow, dto.getStatus());
        }
        if (dto.getStartTime() != null && dto.getEndTime() != null) {
            wrapper.between(Tweet::getCreateAt, dto.getStartTime(), dto.getEndTime());
        }
        wrapper.eq(Tweet::getIsDel, Boolean.FALSE);
        wrapper.orderByDesc(Tweet::getSort);
        wrapper.orderByDesc(Tweet::getCreateAt);

        Page<Tweet> p = mapper.selectPage(new Page(page, size), wrapper);
        for (Tweet response : p.getRecords()) {
            List<String> coverList = new ArrayList<>();
            for (String cover : response.getCover().split(",")) {
                coverList.add(localService.fullPath(cover));
            }
            if (coverList.size() > 0) {
                response.setCover(coverList.get(0)); // 封面
            }
        }

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public void saveItem(TweetDto dto) {
        Tweet _tweet = new Tweet();
        _tweet.setTitle(dto.getTitle());
        _tweet.setAuthor(dto.getAuthor());
        _tweet.setCover(localService.cutPath(dto.getCover()));
        _tweet.setBrief(dto.getBrief());
        _tweet.setContent(dto.getContent());
        _tweet.setSort(dto.getSort());
        _tweet.setIsShow(dto.getIsShow());

        mapper.insert(_tweet);
    }

//    @CacheEvict(cacheNames = "yxl:tweet", key = "#id")
    public void updateItem(String id, TweetDto dto) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Tweet _tweet = optional.get();
            _tweet.setTitle(dto.getTitle());
//            _tweet.setAuthor(dto.getAuthor());
            _tweet.setBrief(dto.getBrief());
            _tweet.setCover(localService.cutPath(dto.getCover()));
            _tweet.setContent(dto.getContent());
            _tweet.setSort(dto.getSort());
            _tweet.setIsShow(dto.getIsShow());

            mapper.updateById(_tweet);
        }
    }

    /**
     * 功能描述：阅读数加1
     *
     * @param id
     */
    public void addRead(String id) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Tweet _tweet = optional.get();
            _tweet.setLooks(_tweet.getLooks() + 1);
            mapper.updateById(_tweet);
        }
    }

    /**
     * 功能描述：替换[资讯]显示状态
     *
     * @param id
     */
    public void handleIsShow(String id) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Tweet _tweet = optional.get();
            _tweet.setIsShow(!_tweet.getIsShow());
            mapper.updateById(_tweet);
        }
    }

    /**
     * 功能描述：替换[资讯]删除状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(id));
        if (optional.isPresent()) {
            Tweet _tweet = optional.get();
            _tweet.setIsDel(!_tweet.getIsDel());
            mapper.updateById(_tweet);
        }
    }

    /**
     * 功能描述：调整资讯的点赞数
     * @param tweetId
     */
    public void likesNum(String tweetId, Boolean isAdd) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(tweetId));
        if (optional.isPresent()) {
            mapper.update(null, new LambdaUpdateWrapper<Tweet>()
                    .eq(Tweet::getId, tweetId)
                    .set(Tweet::getLikes, optional.get().getLikes() + (isAdd ? 1 : -1))
            );
        }
    }

    /**
     * 功能描述：调整资讯的收藏数
     * @param tweetId
     */
    public void collectionsNum(String tweetId, Boolean isAdd) {
        Optional<Tweet> optional = Optional.ofNullable(mapper.selectById(tweetId));
        if (optional.isPresent()) {
            mapper.update(null, new LambdaUpdateWrapper<Tweet>()
                    .eq(Tweet::getId, tweetId)
                    .set(Tweet::getCollections, optional.get().getCollections() + (isAdd ? 1 : -1))
            );
        }
    }

}
