package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.yxl.controller.admin.search.VipLevelSearchDto;
import com.studio.nx.yxl.dto.VipLevelDto;
import com.studio.nx.yxl.entity.VipLevel;
import com.studio.nx.yxl.mapper.VipLevelMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Optional;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class VipLevelService {

    @Autowired
    private VipLevelMapper mapper;

    /**
     * 功能描述：查询[会员等级]详情
     *
     * @param id 标识
     * @return
     */
    public Optional<VipLevel> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    /**
     * 功能描述：查询[会员等级]列表
     */
    public BasePageResponse<VipLevelDto> page(VipLevelSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<VipLevel> lambdaQueryWrapper = new QueryWrapper().lambda();
        lambdaQueryWrapper.eq(VipLevel::getIsDel, false);

        lambdaQueryWrapper.orderByDesc(VipLevel::getIsEnable);
        lambdaQueryWrapper.orderByDesc(VipLevel::getSort);

        Page<VipLevel> p = mapper.selectPage(new Page(page, size), lambdaQueryWrapper);

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    /**
     * 功能描述：保存[会员等级]
     *
     * @param dto
     */
    public VipLevel saveItem(VipLevelDto dto) {
        VipLevel _viplevel = new VipLevel();
        _viplevel.setSort(dto.getSort());
        _viplevel.setIsEnable(dto.getIsEnable());
        _viplevel.setName(dto.getName());
        _viplevel.setYearPrice(dto.getYearPrice());
        _viplevel.setMonthPrice(dto.getMonthPrice());
        _viplevel.setSale(dto.getSale());
        mapper.insert(_viplevel);

        return _viplevel;
    }

    /**
     * 功能描述：更新[会员等级]
     *
     * @param id
     * @param dto
     */
    public VipLevel updateItem(String id, VipLevelDto dto) {

        Optional<VipLevel> optional = findById(id);
        VipLevel _viplevel = optional.get();
        _viplevel.setSort(dto.getSort());
        _viplevel.setIsEnable(dto.getIsEnable());
        _viplevel.setName(dto.getName());
        _viplevel.setYearPrice(dto.getYearPrice());
        _viplevel.setMonthPrice(dto.getMonthPrice());
        _viplevel.setSale(dto.getSale());
        mapper.updateById(_viplevel);

        return _viplevel;
    }

    /**
     * 功能描述：替换[会员等级]显示状态
     *
     * @param id
     */
    public void handleIsEnable(String id) {
        Optional<VipLevel> optional = findById(id);
        if (optional.isPresent()) {
            VipLevel _viplevel = optional.get();
            _viplevel.setIsEnable(!_viplevel.getIsEnable());
            mapper.updateById(_viplevel);
        }
    }

    /**
     * 功能描述：替换[会员等级]删除状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        Optional<VipLevel> optional = findById(id);
        if (optional.isPresent()) {
            VipLevel _viplevel = optional.get();
            _viplevel.setIsDel(!_viplevel.getIsDel());
            mapper.updateById(_viplevel);
        }
    }

}
