package com.studio.nx.yxl.service;

import cn.hutool.core.lang.Assert;
import com.drew.imaging.ImageMetadataReader;
import com.drew.imaging.ImageProcessingException;
import com.drew.metadata.Metadata;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.geometry.Positions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.ClassPathResource;
import org.springframework.util.StopWatch;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

@Slf4j
@DisplayName("处理图片服务-单元测试")
@AutoConfigureMockMvc
@SpringBootTest
public class ProcessingPhotoServiceTest {

    /* 全局水印的透明度 */
    private final static Float WATERMARK_OPACITY = 0.8F;

    /* 列表预览图的压缩质量、宽度、高度 */
    private final static Float PHOTO_PREVIEW_LIST_QUALITY = 1F;
    private final static Integer PHOTO_PREVIEW_LIST_WIDTH = 800;
    private final static Integer PHOTO_PREVIEW_LIST_HEIGHT = 600;

    @Test
    public void metaData() throws ImageProcessingException, IOException {
        Metadata metadata1 = ImageMetadataReader.readMetadata(new File("/Users/liangzp/Downloads/tmp/2021-09-01 102424(1).heic"));
        Assert.isTrue(Boolean.TRUE);
    }

    @DisplayName("处理测试")
    @Test
    public void processing() throws IOException {
        StopWatch watch = new StopWatch();
        watch.start();

//        for (float i = 0.1f; i < 1f; i = i + 0.1f) {
//            String img1 = "/Users/liangzp/Downloads/yxl/test/3I6A5635.jpg";
//            String watermark1 = "/Users/liangzp/Downloads/yxl/test-logo/测试LOGO1.png";
//            String toFile1 = String.format("/Users/liangzp/Downloads/yxl/3I6A5635_%.0f_%.0f.jpg", 10f, i * 100);
//            Thumbnails.of(img1)
//                    .scale(0.1f) // 等比缩放
//                    .outputQuality(i) // 输出质量
//                    .watermark(Positions.BOTTOM_RIGHT, ImageIO.read(new File(watermark1)), 0.8F)
//                    .toFile(toFile1);
//            WatermarkUtil.addStringWatermark(toFile1, toFile1, "jpg", "幼享乐", 80, Color.gray, 0.8F);
//        }

        String file = String.format("/Users/liangzp/Downloads/tmp/2021-09-01 102424(1).jpg");
        String toFile = String.format("/Users/liangzp/Downloads/tmp/_tmp.jpg");
        InputStream yxlWatermark = new ClassPathResource("static/yxl/fission/preview_watermark.png").getInputStream();
        InputStream yxlLogo = new ClassPathResource("static/yxl/fission/preview_logo.png").getInputStream();
        InputStream KindergartenLogo = new ClassPathResource("static/yxl/fission/preview_logo.png").getInputStream();

        Thumbnails.Builder<File> thumbnails = Thumbnails.of(file);
        thumbnails.watermark(Positions.CENTER, ImageIO.read(yxlWatermark), WATERMARK_OPACITY);
        thumbnails.watermark(Positions.BOTTOM_LEFT, ImageIO.read(yxlLogo), WATERMARK_OPACITY);
        thumbnails.watermark(Positions.BOTTOM_RIGHT, ImageIO.read(KindergartenLogo), WATERMARK_OPACITY);
        thumbnails.size(PHOTO_PREVIEW_LIST_WIDTH, PHOTO_PREVIEW_LIST_HEIGHT);
        thumbnails.outputQuality(PHOTO_PREVIEW_LIST_QUALITY); // 输出质量
        thumbnails.outputFormat("jpg");
        thumbnails.toFile(toFile);

//        WatermarkUtil.addStringWatermark(toFile1, toFile1, "jpg", "幼享乐", 80, Color.gray, 0.8F);

//        MetadataUtil.ImageMetadata _imageMetadata1 = MetadataUtil.parseImgInfo("/Users/liangzp/Downloads/yxl/test/3I6A5446.jpg");
//        MetadataUtil.ImageMetadata _imageMetadata3 = MetadataUtil.parseImgInfo("/Users/liangzp/Downloads/yxl/test/3I6A5635.jpg");

        watch.stop();
        log.info("processing in [{}] ms", watch.getTotalTimeMillis());
    }

}
